;*******************************************************************************************
;   Beschreibung:	Frequenzzhler mit 16 Mhz Quarz	und DDS Frequenzgenerator mit AD9833 Chip							 * 
;																														 *							 *
;*******************************************************************************************
;                                                                                          *
;    Dateiname:	 	FZ_DDS_P16F876A_V3.asm						                               		 *
;    Datum:        	17.10.2020                                                            *
;    Version: 		3.0                                                                  *
;                                                                                          *
;    Autor:       	Werner Hirsch                                                       *
;    Internet:      www.wernerhirsch.de                                                           *
;    e-mail:		wernerhirschhh@aol.com                                                          *
;                                                                                          *
;*******************************************************************************************
;    Bemerkungen:                                                                          *
;																														 *
;	Bankumschaltungen:				Seiten:
;	BANK_0:	bcf STATUS,RP0			Page 0: bcf PCLATH,3
;			bcf STATUS,RP1					bcf PCLATH,4
;	BANK_1:	bsf STATUS,RP0			Page 1: bsf PCLATH,3
;			bcf STATUS,RP1					bcf PCLATH,4
;	BANK_2:	bcf STATUS,RP0			Page 2: bcf PCLATH,3
;			bsf STATUS,RP1					bsf PCLATH,4
;	BANK_3:	bsf STATUS,RP0			Page 3: bsf PCLATH,3
;			bsf STATUS,RP1					bsf PCLATH,4														 *
;                                                                                          *
;*******************************************************************************************


	list      p=16f876a			;definiert den verwendeten Prozessor
	#include <p16f876a.inc>		;in diesem File stehen die Adressen und Bitnamen

	ERRORLEVEL      -302		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -305		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -306		;SUPPRESS BANK SELECTION MESSAGES
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _HS_OSC & _WRT_OFF & _LVP_OFF & _DEBUG_OFF & _CPD_OFF 
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus


;==============================
;  DEFINITIONEN
;==============================
#define IN_VCO			PORTA, 0		;PIN RA0 (pin 2) Spannungseingang fr VCO
#define OUT_DAT			PORTA, 1		;PIN RA1 (pin 3) Serielle Daten fr DDS,DAC,74HC4094
#define OUT_CLK			PORTA, 2		;PIN RA2 (pin 4) Serielle Clock fr DDS,DAC,74HC4094
#define DDS_CS			PORTA, 3		;PIN RA3 (pin 5) Chip Select AD9833 (DDS Funktionsgenerator)
#define IN_COUNTF		PORTA, 4		;PIN RA4 (pin 6) Zhler Eingang Frequenz
#define VOL_CS			PORTA, 5		;PIN RA5 (pin 7) Chip Select 74HC4094 (Bit0-5:DDS Volume,Bit6: Wellenform: 0=Sin,Tri, 1=Sqr)
#define OUT_VT_ONOFF	PORTB, 0		;Pin RB0 (pin 21) Vorteiler Ein/Aus
#define IN_MENUE		PORTB, 1		;Pin RB1 (pin 22) Eingang "Men"
#define IN_OK			PORTB, 2		;Pin RB2 (pin 23) Eingang "OK"
#define IN_UP			PORTB, 3		;Pin RB3 (pin 24) Eingang "Up"
#define IN_DOWN			PORTB, 4		;Pin RB4 (pin 25) Eingang "Down"
#define IN_RIGHT		PORTB, 5		;Pin RB5 (pin 26) Eingang "Right"
#define IN_LEFT			PORTB, 6		;Pin RB6 (pin 27) Eingang "Left"
#define TTL_ENABLE		PORTB, 7		;Pin RB7 (pin 28) TTL Ausgang: 1 = On, 0 = Off
#define DAC_CS			PORTC, 0		;PIN RC0 (pin 11) Chip Select Digital/Analog Converter (DAC)
#define ME_DB0			PORTC, 1		;PIN RC1 (pin 12) DB0 fr Messeingang 
#define IN_COUNTP		PORTC, 2		;PIN RC2 (pin 13) Zhler Eingang Periode (CCP1)
#define ME_DB1			PORTC, 3		;PIN RC3 (pin 14) DB1 fr Messeingang 
#define DISP_EN			PORTC, 4		;PIN RC4 (pin 15) Mit der negativen Flanke von DISP_EN werden die parallelen Daten bernommen
#define DISP_RS			PORTC, 5		;PIN RC5 (pin 16) Unterscheidung zwischen Befehl und Daten 
#define SI_CLK			PORTC, 6		;PIN RC6 (pin 17) Serieller Takt fr Display und DDS Daten 
#define SI_DAT			PORTC, 7		;PIN RC7 (pin 18) Serielle Daten zum Display und DDS

;=====================================
;  KONSTANTEN
;=====================================
HIGH_BYTE_S 	EQU 0x3E
LOW_BYTE_S		EQU 0xF7

;=====================================
;  VARIABLEN BANK 0 (0x20 bis 0x7F)
;=====================================
w_temp        	EQU	0x7D        ; werden fr Interrupts verwendet, um den Status vor 
status_temp   	EQU	0x7E        ; der Interruptroutine zu sichern
pclath_temp		EQU	0x7F


COUNT_H_S		EQU	0x20		;Zhler fr Loops Timer 1
COUNT_ADJ		EQU	0x21		;Zhler fr Feinabgleich Timer 1
MESS_END		EQU	0x22		;Status Register: 	Bit 0: Sekundentakt
								;					Bit 1:
								;					Bit 2:
								;					Bit 3:
STAT_INT		EQU	0x23		;Statusregister fr Interruptroutinen
LOOP_1			EQU 0x24		;Zhler fr Wartezeit
SHIFT_REG		EQU	0x25		;Register fr Daten, die an das Display oder den DDS Generator gesendet werden sollen
SHIFT_CNT		EQU	0x26		;Zhler fr die serielle Datenausgabe fr das Display oder den DDS Generator
PRESC_TMR0		EQU 0x27		;Zhler fr Anzahl Multiplikationen x2 bei Prescaler TMR0
DISP_PRESC		EQU 0x28		;Display Register fr Prescaler Wert
PROGRAM			EQU 0x29		;Register fr zu whlendes Programm, 0=Frequenz,1=Periode,2=Zhler,3=VCO, 4=Wobble

ZAHL_A0			EQU	0x2A		;Zahl A Bits 0-7
ZAHL_A1			EQU	0x2B		;Zahl A Bits 8-15
ZAHL_A2			EQU	0x2C		;Zahl A Bits 16-23
ZAHL_A3			EQU	0x2D		;Zahl A Bits 24-31
ZAHL_B0			EQU	0x2E		;Zahl B Bits 0-7
ZAHL_B1			EQU	0x2F		;Zahl B Bits 8-15
ZAHL_B2			EQU	0x30		;Zahl B Bits 16-23
ZAHL_B3			EQU	0x31		;Zahl B Bits 24-31
ZAHL_C0			EQU	0x32		;Zahl C Bits 0-7
ZAHL_C1			EQU	0x33		;Zahl C Bits 8-15
ZAHL_C2			EQU	0x34		;Zahl C Bits 16-23
ZAHL_C3			EQU	0x35		;Zahl C Bits 24-31
ZAHL_C4			EQU	0x36		;Zahl C Bits 32-39
ZAHL_C5			EQU	0x37		;Zahl C Bits 40-47
ZAHL_C6			EQU	0x38		;Zahl C Bits 48-55
ZAHL_C7			EQU	0x39		;Zahl C Bits 56-63
TEMP_10			EQU	0x3A		;Allgemeines Zwischenspeicherregister
TEMP_11			EQU	0x3B		;Allgemeines Zwischenspeicherregister
TEMP_12			EQU	0x3C		;Allgemeines Zwischenspeicherregister
TEMP_13			EQU	0x3D		;Allgemeines Zwischenspeicherregister
TEMP_14			EQU	0x3E		;Allgemeines Zwischenspeicherregister
TEMP_15			EQU	0x3F		;Allgemeines Zwischenspeicherregister
TEMP_16			EQU	0x40		;Allgemeines Zwischenspeicherregister
KOMMA			EQU	0x43		;Register fr Kommaposition der Displayanzeige
DISP_0			EQU	0x44		;Display Nachkommastelle oder Einer
DISP_1			EQU	0x45		;Display Einer oder Zehner
DISP_2			EQU	0x46		;Display Zehner oder Hunderter
DISP_3			EQU	0x47		;Display Hunderter oder Tausender
DISP_4			EQU	0x48		;Display Tausender oder Zehntausender
DISP_5			EQU	0x49		;Display Zehntausender oder Hunderttausender
DISP_6			EQU	0x4A		;Display Hunderttausender oder Millionen
DISP_7			EQU	0x4B		;Display Millionen oder Zehnmillionen
DISP_8			EQU	0x4C		;Display Zehnmillionen oder Hundertmillionen
BEFEHL			EQU 0x4D		;Register fr Tastaturbefehle
TEMP_DIG_CNT	EQU 0x4E		;Zhler fr Menposition horizontal
TEMP_CNT_MAX	EQU 0x4F		;Max Zhlerwert fr Menposition horizontal
TEMP_DIG_ADR	EQU 0x50		;Display Adressregister fr das zu ndernde Digit
TEMP_DIG_VAL	EQU 0x51		;Display Wert fr das zu ndernde Digit
TEMP_DIG_STAT	EQU 0x52		;Status Register	bit 0: wenn 1: Update der Anzeige des zu ndernden Wertes
								;					bit 1: wenn 1: Erhhung des zu ndernden Wertes
								;					bit 2: wenn 1: Reduzierung des zu ndernden Wertes
MENU_COUNT		EQU	0x53		;Zhler fr Menposition
MENU_COUNT_MAX	EQU	0x54		;Maximale Menposition
MENU_STAT		EQU	0x55		;Status Register fr Men	bit 0: Men 1.Stufe
								;							bit 1: Men 2.Stufe Messen
								;							bit 2: Men 2.Stufe DDS Generator
								;							bit 3: Men 2.Stufe Einstellungen
								;							bit 5: Display Update Bit fr Eingang und DDS Einstellungen
								;							bit 6: Men 2.Stufe Einstellungen, Update Display
								;							bit 7: Men 1.Stufe, Update Display
EINGANG			EQU	0x56		;Register fr Messeingang: 0 = TTL, 1 = Analog, 2 = DDS, 3 = Vorteiler
MUL_VT			EQU	0x57		;Register fr Vorteilerfaktor
UNIT			EQU	0x58		;Register fr Einheit k = kilo, m = milli, u = micro, n = nano

FREQREG_L		EQU	0x59		;Frequenzregister fr VCO / Wobble, Fu L-Byte
FREQREG_M		EQU	0x5A		;Frequenzregister fr VCO / Wobble, Fu M-Byte
FREQREG_H		EQU	0x5B		;Frequenzregister fr VCO / Wobble, Fu H-Byte
KENNLIN			EQU	0x5C		;Register fr Kennlinie VCO / Wobble, 0 = Linear, 1 = Logarithmisch
DEKADE			EQU	0x5D		;Register fr Anzahl Dekaden VCO / Wobble, => 1 - 3 Dekaden
AD_DA_L			EQU	0x5E		;Register fr 10 Bit AD/DA Wert VCO / Wobble, Low Byte
AD_DA_H			EQU	0x5F		;Register fr 10 Bit AD/DA Wert VCO / Wobble, High Byte
AD_DA_REST		EQU	0x60		;Register fr Rest

TTL_ON_OFF		EQU	0x61		;TTL Ausgang: 0=Aus, 1=Ein
DDS_VOL			EQU 0x62		;Register fr DDS Volumen (0-63 entspricht 0dB bis -63dB)
DDS_FREQ_0		EQU 0x63		;Register fr DDS Frequenz (x10) 0 (32Bit, Bits 0-7)
DDS_FREQ_1		EQU	0x64		;Register fr DDS Frequenz (x10) 1 (32Bit, Bits 8-15)
DDS_FREQ_2		EQU	0x65		;Register fr DDS Frequenz (x10) 2 (32Bit, Bits 16-23)
DDS_FREQ_3		EQU	0x66		;Register fr DDS Frequenz (x10) 3 (32Bit, Bits 24-31)
DDS_PHASE_L		EQU	0x67		;Register fr DDS Phase (Winkel x10), Low Byte (16Bit, Bits 0-7)
DDS_PHASE_H		EQU	0x68		;Register fr DDS Phase (Winkel x10), High Byte (16Bit, Bits 8-15)
DDS_MCLK_0		EQU 0x69		;Register fr DDS MCLK 0 (32Bit, Bits 0-7)
DDS_MCLK_1		EQU	0x6A		;Register fr DDS MCLK 1 (32Bit, Bits 8-15)
DDS_MCLK_2		EQU	0x6B		;Register fr DDS MCLK 2 (32Bit, Bits 16-23)
DDS_MCLK_3		EQU	0x6C		;Register fr DDS MCLK 3 (32Bit, Bits 24-31)
DDS_WAVEFORM	EQU	0x6D		;Register fr Wellenform:  0=Sinus, 1=Dreieck, 2=Rechteck
DDS_CONTROL_L	EQU	0x6E		;Register Control Byte, Low Byte (16Bit, Bits 0-7)
DDS_CONTROL_H	EQU	0x6F		;Register Control Byte, High Byte (16Bit, Bits 8-15)
DDS_FREQREG_0	EQU	0x70		;Frequenzregister 0 (32Bit, Bits 0-7)
DDS_FREQREG_1	EQU	0x71		;Frequenzregister 1 (32Bit, Bits 8-15)
DDS_FREQREG_2	EQU	0x72		;Frequenzregister 2 (32Bit, Bits 16-23)
DDS_FREQREG_3	EQU	0x73		;Frequenzregister 3 (32Bit, Bits 24-31)
DDS_PHASEREG_L	EQU	0x74		;Phasenregister, Low Byte (16Bit, Bits 0-7)
DDS_PHASEREG_H	EQU	0x75		;Phasenregister, High Byte (16Bit, Bits 8-15)
OFFSET_NULL_L	EQU	0x76		;Register fr Offset Null Low Byte
OFFSET_NULL_H	EQU	0x77		;Register fr Offset Null High Byte
OFFSET_100_L	EQU	0x78		;Register fr Offset -100% Low Byte
OFFSET_100_H	EQU	0x79		;Register fr Offset -100% High Byte
OFFSET_VAL		EQU	0x41		;Offset +/-100%, Vorzeichen in Bit 7: 0=+, 1=-
DAC_L			EQU	0x7A		;Register fr DAC Daten Low Byte
DAC_H			EQU	0x7B		;Register fr DAC Daten High Byte
DAC_GAIN		EQU	0x7C		;Register fr DAC Verstrkung:  1 = 0-2048mV, 0 = 0-4096mV

;=====================================
;  VARIABLEN BANK 1 (0xA0 bis 0xEF)
;=====================================
TEMP_0			EQU	0xA0		;Allgemeines Zwischenspeicherregister
TEMP_1			EQU	0xA1		;Allgemeines Zwischenspeicherregister
TEMP_2			EQU	0xA2		;Allgemeines Zwischenspeicherregister
TEMP_3			EQU	0xA3		;Allgemeines Zwischenspeicherregister
WOB_M_A			EQU	0xA4		;Register fr Wobble, Frequenzerhhung: Bit0: 0=Auto, 1=Man, Bit1: 0=Kont, 1=Burst
WOB_STEP		EQU	0xA5		;Register fr Wobble, Schritte/Durchlauf: 1=1024, 2=512, 3=256, 4=128, 5=64, 6=32
WOB_STEPW		EQU	0xA6		;Register fr Wobble, Schrittweite: 1=1024, 2=512, 4=256, 8=128, 16=64, 32=32
WOB_ZEIT		EQU	0xA7		;Register fr Wobble, Zeit/Schritt: 0=10ms, 1=20ms, 2=50ms, 3=100ms, 4=200ms, 5=500ms,
								;   6=1000ms, 7=2000ms
WOB_ZEIT_H		EQU	0xA8		;Register fr Wobble, Zeit/Schritt Zhler: High Byte
WOB_ZEIT_LN1	EQU	0xA9		;Register fr Wobble, Zeit/Schritt Zhler: Low Byte, Loop n-1
WOB_ZEIT_LN		EQU	0xAA		;Register fr Wobble, Zeit/Schritt Zhler: Low Byte, Loop n
WOB_LOOP		EQU	0xAB		;Register fr Wobble, Zeit/Schritt Zhler: Anzahl Loops
WOB_COUNT_L		EQU	0xAC		;Register fr Wobble, Schritt Zhler: Low Byte
WOB_COUNT_H		EQU	0xAD		;Register fr Wobble, Schritt Zhler: High Byte
FD1DDS_0		EQU	0xAE		;Register fr VCO/Wobble, Frequenzregister Dekade1: 32Bit, Bits 0-7
FD1DDS_1		EQU	0xAF		;Register fr VCO/Wobble, Frequenzregister Dekade1: 32Bit, Bits 8-15
FD1DDS_2		EQU	0xB0		;Register fr VCO/Wobble, Frequenzregister Dekade1: 32Bit, Bits 16-23
FD1DDS_3		EQU	0xB1		;Register fr VCO/Wobble, Frequenzregister Dekade1: 32Bit, Bits 24-31
FD2DDS_0		EQU	0xB2		;Register fr VCO/Wobble, Frequenzregister Dekade2: 32Bit, Bits 0-7
FD2DDS_1		EQU	0xB3		;Register fr VCO/Wobble, Frequenzregister Dekade2: 32Bit, Bits 8-15
FD2DDS_2		EQU	0xB4		;Register fr VCO/Wobble, Frequenzregister Dekade2: 32Bit, Bits 16-23
FD2DDS_3		EQU	0xB5		;Register fr VCO/Wobble, Frequenzregister Dekade2: 32Bit, Bits 24-31
FD3DDS_0		EQU	0xB6		;Register fr VCO/Wobble, Frequenzregister Dekade3: 32Bit, Bits 0-7
FD3DDS_1		EQU	0xB7		;Register fr VCO/Wobble, Frequenzregister Dekade3: 32Bit, Bits 8-15
FD3DDS_2		EQU	0xB8		;Register fr VCO/Wobble, Frequenzregister Dekade3: 32Bit, Bits 16-23
FD3DDS_3		EQU	0xB9		;Register fr VCO/Wobble, Frequenzregister Dekade3: 32Bit, Bits 24-31
DEK_AKTUELL		EQU	0xBA		;Register fr VCO/Wobble, Aktuelle Dekade
WOB_BURST		EQU	0xBB		;Register fr VCO/Wobble, kontinuierlich = 0, Burst = 1
BU_VAL_0		EQU	0xBC		;Register fr VCO/Wobble, 
BU_VAL_1		EQU	0xBD		;Register fr VCO/Wobble, 
BU_VAL_2		EQU	0xBE		;Register fr VCO/Wobble, 
BU_VAL_3		EQU	0xBF		;Register fr VCO/Wobble, 
DELTAP_L		EQU	0xC0		;Register fr VCO/Wobble, 
DELTAP_M		EQU	0xC1		;Register fr VCO/Wobble, 
DELTAP_H		EQU	0xC2		;Register fr VCO/Wobble, 
TBURST_L		EQU	0xC3		;Register fr VCO/Wobble, Burstlnge L-byte (Timer0)
TBURST_M		EQU	0xC4		;Register fr VCO/Wobble, Burstlnge M-byte
TBURST_H		EQU	0xC5		;Register fr VCO/Wobble, Burstlnge H-byte


;=====================================
;  VARIABLEN EEPROM (0x2100 bis 0x21FF)
;=====================================

		ORG 	0x2100			;Adresse des internen EEPROM fr den Brenner  (2100 = erster Speicherplatz)
;Variable DDS_MCLK Register, 25.000.000Hz => 0x017D7840 (Oszillatorfrequenz AD9833)
		de		0x40			;Adresse 0x00,	DDS_MCLK_0 
		de		0x78			;Adresse 0x01,	DDS_MCLK_1
		de		0x7D			;Adresse 0x02,	DDS_MCLK_2
		de		0x01			;Adresse 0x03,	DDS_MCLK_3
;Variable Vorteiler
		de		0x40			;Adresse 0x04,	Vorteilerfaktor
;Variable Offset DDS und DAC Verstrkung
		de		0x00			;Adresse 0x05,	Sinus/Dreieck Low Byte
		de		0x02			;Adresse 0x06,	Sinus/Dreieck High Byte
		de		0xDB			;Adresse 0x07,	Rechteck Low Byte
		de		0x01			;Adresse 0x08,	Rechteck High Byte
		de		0x01			;Adresse 0x09,	DAC Verstrkung: 1 = 0-2048mV, 0 = 0-4096mV
;Variable Starteinstellungen
		de		0x04			;Adresse 0x0A,	Programm: 0=Frequenz,1=Periode,2=Zhler,3=VCO,4=Wobbel
		de		0x00			;Adresse 0x0B,	Messeingang: 0 = TTL, 1 = Analog, 2 = DDS, 3 = Vorteiler
		de		0xC8			;Adresse 0x0C,	VCO/Wobbel Fu L-Byte
		de		0x00			;Adresse 0x0D,	VCO/Wobbel Fu M-Byte
		de		0x00			;Adresse 0x0E,	VCO/Wobbel Fu H-Byte
		de		0x01			;Adresse 0x0F,	VCO/Wobbel Kennlinie: 0 = Linear, 1 = Logarithmisch
		de		0x03			;Adresse 0x10,	VCO/Wobbel Dekaden: 1-3
		de		0x03			;Adresse 0x11,	Wobbel Steps: 1=1024, 2=512, 3=256, 4=128, 5=64, 6=32
		de		0x02			;Adresse 0x12,	Wobbel Man/Aut: 0 = Auto, 1 = Manuell, 2 = Auto+Burst, 3 = Manuell+Burst
		de		0x08			;Adresse 0x13,	Wobbel delta t: 0=1ms, 1=2ms, 2=5ms, 3=10ms, 4=20ms, 5=50ms,
								;								6=100ms, 7=200ms, 8=500ms, 9=1000ms, 10=2000ms
		de		0x00			;Adresse 0x14,	DDS Wellenform: 0=Sinus, 1=Dreieck, 2=Rechteck
		de		0x00			;Adresse 0x15,	DDS Ausgangspegel: 0-63 entspricht 0dB bis -63dB
		de		0x10			;Adresse 0x16,	DDS Frequenz (x10): Byte 0 (LSB)
		de		0x27			;Adresse 0x17,	DDS Frequenz (x10): Byte 1
		de		0x00			;Adresse 0x18,	DDS Frequenz (x10): Byte 2
		de		0x00			;Adresse 0x19,	DDS Frequenz (x10): Byte 3 (MSB)
		de		0x00			;Adresse 0x1A,	DDS Phasenwinkel (x10): Low Byte
		de		0x00			;Adresse 0x1B,	DDS Phasenwinkel (x10): High Byte
		de		0x01			;Adresse 0x1C,	TTL Ausgang: 0=Aus, 1=Ein
;Variable Offset DDS und DAC Verstrkung
		de		0xDB			;Adresse 0x1D,	-100% Sinus/Dreieck Low Byte
		de		0x03			;Adresse 0x1E,	-100% Sinus/Dreieck High Byte
		de		0xB6			;Adresse 0x1F,	-100% Rechteck Low Byte
		de		0x03			;Adresse 0x20,	-100% Rechteck High Byte
		de		0x00			;Adresse 0x21,	Offset +/-100%, Vorzeichen in Bit 7: 0=+, 1=-

;__________________________________________________________________________________________


		ORG     0x000			; Adresse, an die nach einem Reset gesprungen wird
		
  		goto    start           ; Springe zum Programmanfang

		ORG     0x004           ; Adresse, an die nach einem Interrupt gesprungen wird
		movwf	w_temp          ; Sichere den Inhalt des W-Registers
		movf    STATUS,w        ; Verschiebe STATUS-Register ins W-Register
		movwf	status_temp     ; Sichere den Inhalt des STATUS-Registers
		movf    PCLATH,w        ; Verschiebe PCLATH-Register ins W-Register
		movwf	pclath_temp     ; Sichere den Inhalt des PCLATH-Registers		


	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4				;
	bcf STATUS,RP0				;Select BANK_0

	btfss STAT_INT,1			;Bit1 = Interruptroutine fr Periodenmessung: Timer 1 auf "Null" setzen 
	goto int_zaehler
	bsf T1CON,TMR1ON			;Timer 1 starten
	bcf PIR1,CCP1IF				;Interrupt Flag lschen
	clrf TMR0					;Timer0 lschen
	bsf STATUS,RP0				;Select BANK_1
	bcf PIE1,CCP1IE				;Capture Interrupt verbieten
	bcf STATUS,RP0				;Select BANK_0
	bcf STAT_INT,1				;Bit lschen, da nur einmalig ausgefhrt wird
	bsf MESS_END,1				;setzen des Statusbit fr CPP1 Interrupt
	goto Int_end

int_zaehler
	btfss STAT_INT,2			;Bit2 = Interruptroutine fr Zhler
	goto int_wobbel
	bcf PIR1,CCP1IF				;Interrupt Flag lschen
	bsf MESS_END,3				;Setze Update Bit
	btfss IN_DOWN
	goto $+4
	bcf INTCON,PEIE				;Disable periphere Interrupts (capture)
	bsf MESS_END,2				;Setze Bit wenn IN_DOWN Eingang (Enable Zhler) high ist, d.h. Zhlung gesperrt
	goto Int_end
	incfsz ZAHL_A0
	goto Int_end
	incfsz ZAHL_A1
	goto Int_end
	incfsz ZAHL_A2
	goto Int_end
	incf ZAHL_A3

int_wobbel
	btfss STAT_INT,3			;Bit3 = Interruptroutine fr Burst Timer
	goto Int_end
	bcf PIR1, TMR1IF			;Timer 1 Interrupt flag lschen
	decfsz COUNT_H_S			;  reduziere Zeitzhler um 1 und prfe, ob Zhler =0
	goto Int_end				;Zeitzhler ist noch nicht 0, zurck zur Schleife

	bsf STATUS,RP0				;Select BANK_1	
	movfw WOB_ZEIT_LN1
	bcf STATUS,RP0				;Select BANK_0
	movwf TMR1H

	decfsz COUNT_ADJ			;Timer 1 wieder einstellen
	goto $+9
	bsf STATUS,RP0				;Select BANK_1	
	movfw WOB_ZEIT_LN
	bcf STATUS,RP0				;Select BANK_0
	movwf TMR1H
	bsf STATUS,RP0				;Select BANK_1	
	movfw WOB_LOOP
	bcf STATUS,RP0				;Select BANK_0
	movwf COUNT_ADJ
	bsf STATUS,RP0				;Select BANK_1	
	movfw WOB_ZEIT_H
	bcf STATUS,RP0				;Select BANK_0
	movwf COUNT_H_S
	bsf MESS_END,7				;Setze Schrittweiten Ende Bit
	
Int_end	
	btfss IN_MENUE				;Abfrage des Men Tasters
	clrf INTCON					;wenn gedrckt (low) disable alle Interupts

		movf    pclath_temp,w   ; hole die Kopie des PCLATH-Registers
		movwf	PCLATH          ; Sichere die Daten wieder zurck in das PCLATH-Register
		movf    status_temp,w  	; hole die Kopie des STATUS-Registers
		movwf	STATUS          ; Sichere die Daten wieder zurck in das STATUS-Register
		swapf   w_temp,f
		swapf   w_temp,w        ; Sichere die Daten wieder zurck in das W-Register
		retfie                  ; Rcksprung vom Interrupt
;__________________________________________________________________________________________

start							; Beginn des Programms
	; Initialisierungen
	bcf STATUS,RP0			;Select BANK_0
	bcf STATUS,RP1
	clrf PORTA				;Lsche alle Ausgangsports
	clrf PORTB
	clrf PORTC
	movlw b'10000000'		;bit 7+6:10=Fosc/32
	movwf ADCON0			;
	bsf STATUS,RP0			;Select BANK_1
	movlw b'10001110'		;bit 7=1: rechtsbndig, bit 6=0: f/32, bit 3-0:1110 RA0 ist Analogeingang, Vref = Vdd/GND
	movwf ADCON1
	movlw b'11010001'		;setze RA1 bis RA3 und RA5 als Ausgang, den Rest als Eingang
	movwf TRISA
	movlw b'01111110'		;setze RB1-RB6 als Eingang, RB0 + RB7 als Ausgang
	movwf TRISB
	movlw b'00000100'		;setze RC0-RC1 und RC3-RC7 als Ausgang, RC2 als Eingang
	movwf TRISC
	movlw b'01101000'		;Pull Up Port B enabled, RB0 Interrupt bei steigender Flanke, TMR0 extern, Prescaler 1:1
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0

	bsf DDS_CS				;setze Chip select Funktionsgenerator auf high
	bsf DAC_CS				;setze Chip select DAC auf high
	bsf OUT_VT_ONOFF		;setze Freigabe Eingang Vorteiler auf high

	clrf MENU_STAT
	movlw 0x01
	movwf MENU_COUNT
	movwf DISP_PRESC

	bsf PCLATH,4			;Select Page 2
	call Init_display		;Initialisierung des Displays
	bcf PCLATH,4			;Select Page 0

;Initialisierung des DDS chips AD9833
	bsf PCLATH,3			;Select Page 1
	movlw b'00100001'		;Control Register: 0010 0001 0000 0000 (Bit8: Reset = 1 ) 
	movwf DDS_CONTROL_H
	movlw 0x00
	movwf DDS_CONTROL_L
	call write_dds_control	;Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	call write_dds_control	;Control Register zur Sicherheit noch mal schreiben

	call read_parameter		;laden der Konstanten aus dem internen EEPROM in die F-Register
	bcf PCLATH,3			;Select Page 0

							;setzen der Ausgnge fr den gewhlten Messeingang:
	btfsc EINGANG,0			;prfen ob Bit 0 von Eingang low ist
	bsf ME_DB0				;Bit 0 ist high, setze Ausgang auf high
	btfsc EINGANG,1			;prfen ob Bit 1 von Eingang low ist
	bsf ME_DB1				;Bit 1 ist high, setze Ausgang auf high

	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call dds_settings		;Unterprogramm fr alle DDS Einstellungen
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4

	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue

;Testbereich Anfang
	movlw d'8'
	movwf ZAHL_C0
	movlw d'255'
	movwf ZAHL_C1
	movlw d'0'
	movwf ZAHL_C2




;Testbereich Ende	

main						;Beginn der Hauptschleife
	movlw d'2'				;prfen, ob Programm 2(Zhler) gewhlt ist
	subwf PROGRAM,w
	btfsc STATUS,Z
	goto Zaehler			;ja, Programm 2(Zhler) ist gewhlt, springe zum Programmteil "Zaehler"

	movlw d'3'				;prfen, ob Programm >=3 (VCO=3,Wobbel=4) gewhlt ist
	subwf PROGRAM,w
	btfsc STATUS,C
	goto vco_wob			;ja, Programm >=3 ist gewhlt, springe zum Programmteil "vco_wob"


Frequenz
	bcf DDS_CONTROL_H,0		;lschen des Reset Bits im Control Register (DDS starten)
	bsf PCLATH,3			;Select Page 1
	call write_dds_control	;Aufruf des Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	bcf PCLATH,3			;Select Page 0
	movlw d'3'				;prfen, ob Vorteilereingang gewhlt ist
	subwf EINGANG,w
	btfsc STATUS,Z
	bcf OUT_VT_ONOFF		;ja, Vorteilereingang ist gewhlt, einschalten
	movlw d'2'				;prfen, ob DDS Eingang gewhlt ist
	subwf EINGANG,w
	btfsc STATUS,Z
	bsf TTL_ENABLE			;ja, TTL Ausgang einschalten

	movlw b'01000000'		;Komma nach der 1sten Stelle
	movwf KOMMA				;
	movlw 0x20				;ASCII fr "Blank"
	movwf UNIT				;
	clrf DISP_0
	clrf DISP_1
	clrf DISP_2
	clrf DISP_3
	clrf DISP_4
	clrf DISP_5
	clrf DISP_6
	clrf DISP_7
	bsf PCLATH,4			;Select Page 2
	call disp_main
	bcf PCLATH,4			;Select Page 0

frequenz_1
							;Im ersten Schritt wird mit Prescaler 1:32 die Frequenz ber 0,002048 sek grob gemessen 
	bsf STATUS,RP0			;Select BANK_1
	clrf PIE1   			;
	movlw b'01100100'		;TMR0 extern(Prescaler 1:32 fr Test)
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0
	movlw b'00000001'		;TMR1 On, Prescaler 1:1
	movwf T1CON
	clrf INTCON				;Interrupts verbieten

	call clear_ZAHL_C		;Register fr Frequenzzhler lschen
	clrf MESS_END			;Register lschen
	clrf TMR1L
	movlw 0xE0				;Timer 1 auf 2048 micro sek einstellen :16.000.000 / 4 * 0,002048 = 8.192 Takte
	movwf TMR1H				; 65.536 - 8.192 = 57.344 => 0xE000
	movlw d'5'				;Zeitdifferenz Timer 1 innerhalb der Messschleife (5 Takte)
	movwf TMR1L
	clrf TMR0				;lschen des Timer0
	bcf PIR1, TMR1IF		;Timer 1 Interrupt flag lschen
	bcf INTCON, T0IF		;Timer 0 Interrupt flag lschen

freq1_loop
	btfsc PIR1, TMR1IF
	goto auswert_freq1
	btfss INTCON, T0IF
	goto freq1_loop
	bcf INTCON, T0IF
	incf ZAHL_C1
	goto freq1_loop
auswert_freq1
	movfw TMR0
	movwf ZAHL_C0
	btfss INTCON, T0IF		;Abfrage Timer 0 berlauf Flag (Anzahl L/H bergnge)
	goto auswert_freq2		;Flag nicht gesetzt, weiter 
	movlw d'10'				;wenn ZAHL_C0 zwischen 245 und 255, Flag spter gesetzt, wenn zwischen 0 und 10 frher 
	subwf ZAHL_C0,w
	btfsc STATUS,C
	goto auswert_freq2		;Ergebnis ist positiv, d.h. Flag spter gesetzt, weiter
	incf ZAHL_C1			;Ergebnis ist negativ, d.h. Flag frher gesetzt, erhhe Zhler um 1


auswert_freq2
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
							;wenn Frequenz <109375Hz => Periodenmessung (109375Hz/32*0,002048s = 7 => 0x0007)
	movlw 0x07				;Hole Wert "7" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x00				;Hole Wert "0" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto Periode			;Ergebnis ist negativ, d.h. Frequenz <109375Hz
							;wenn Frequenz <2000000Hz => Prescaler=1 (2000000Hz/32*0,002048s = 128 => 0x0080)
	movlw 0x80				;Hole Wert "128" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x00				;Hole Wert "0" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_1			;Ergebnis ist negativ, d.h. Frequenz <2000000Hz
							;wenn Frequenz <4000000Hz => Prescaler=2 (4000000Hz/32*0,002048s = 256 => 0x0100)
	movlw 0x00				;Hole Wert "0" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x01				;Hole Wert "1" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_2			;Ergebnis ist negativ, d.h. Frequenz <4000000Hz
							;wenn Frequenz <8000000Hz => Prescaler=4 (8000000Hz/32*0,002048s = 512 => 0x0200)
	movlw 0x00				;Hole Wert "0" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x02				;Hole Wert "2" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_4			;Ergebnis ist negativ, d.h. Frequenz <8000000Hz
							;wenn Frequenz <16000000Hz => Prescaler=8 (16000000Hz/32*0,002048s = 1024 => 0x0400)
	movlw 0x00				;Hole Wert "0" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x04				;Hole Wert "4" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_8			;Ergebnis ist negativ, d.h. Frequenz <16000000Hz
							;wenn Frequenz <32000000Hz => Prescaler=16 (32000000Hz/32*0,002048s = 2048 => 0x0800)
	movlw 0x00				;Hole Wert "0" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x08				;Hole Wert "8" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_16			;Ergebnis ist negativ, d.h. Frequenz <32000000Hz
							;wenn Frequenz <64000000Hz => Prescaler=32 (64000000Hz/32*0,002048s = 4096 => 0x1000)
	movlw 0x00				;Hole Wert "0" ins W-Register
	subwf ZAHL_C0,w			;Subtrahiere W-Register vom Low Byte
	movlw 0x10				;Hole Wert "16" ins W-Register
	btfss STATUS,C			;prfen, ob ein bertrag bei der Subtraktion aufgetreten ist
	addlw d'1'				;es ist ein bertrag aufgetreten, erhhe W-Register um 1
	subwf ZAHL_C1,w			;Subtrahiere W-Register vom High Byte 
	btfss STATUS, C			;prfen, ob das Ergebnis negativ ist
	goto presc_32			;Ergebnis ist negativ, d.h. Frequenz <64000000Hz
	goto presc_64			;Ergebnis ist positiv, d.h. Frequenz >=64000000Hz

presc_1
	bsf STATUS,RP0			;Select BANK_1
	movlw b'01101000'		;TMR0 extern(Prescaler 1:1)
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0
	movlw d'1'
	movwf PRESC_TMR0
	movwf DISP_PRESC
	goto messen_freq

presc_2
	bsf STATUS,RP0			;Select BANK_1
	movlw b'01100000'		;TMR0 extern(Prescaler 1:2)
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0
	movlw d'2'
	movwf PRESC_TMR0
	movwf DISP_PRESC
	goto messen_freq

presc_4
	bsf STATUS,RP0			;Select BANK_1
	movlw b'01100001'		;TMR0 extern(Prescaler 1:4)
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0
	movlw d'3'
	movwf PRESC_TMR0
	movlw d'4'
	movwf DISP_PRESC
	goto messen_freq

presc_8
	bsf STATUS,RP0			;Select BANK_1
	movlw b'01100010'		;TMR0 extern(Prescaler 1:8)
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0
	movlw d'4'
	movwf PRESC_TMR0
	movlw d'8'
	movwf DISP_PRESC
	goto messen_freq

presc_16
	bsf STATUS,RP0			;Select BANK_1
	movlw b'01100011'		;TMR0 extern(Prescaler 1:16)
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0
	movlw d'5'
	movwf PRESC_TMR0
	movlw d'16'
	movwf DISP_PRESC
	goto messen_freq

presc_32
	bsf STATUS,RP0			;Select BANK_1
	movlw b'01100100'		;TMR0 extern(Prescaler 1:32)
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0
	movlw d'6'
	movwf PRESC_TMR0
	movlw d'32'
	movwf DISP_PRESC
	goto messen_freq

presc_64
	bsf STATUS,RP0			;Select BANK_1
	movlw b'01100101'		;TMR0 extern(Prescaler 1:64)
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0
	movlw d'7'
	movwf PRESC_TMR0
	movlw d'64'
	movwf DISP_PRESC
	goto messen_freq


messen_freq
	call clear_ZAHL_C		;lsche ZAHL_C Register fr die Frequenzmessung
	clrf MESS_END
	movlw HIGH_BYTE_S		;Zhler auf 1 sek einstellen
	movwf COUNT_H_S
	clrf TMR1L
	movlw LOW_BYTE_S
	movwf TMR1H	
	bcf PIR1, TMR1IF		;Timer 1 Interrupt flag lschen
	movlw d'2'				;Zeitdifferenz (5Takte) Timer 1 innerhalb der Messschleife
	movwf TMR1L
	clrf TMR0
	bcf INTCON, T0IF		;Timer 0 Interrupt flag lschen	
	nop
	nop
	nop
freq_loop
	btfss PIR1, TMR1IF		;Abfrage des Timer 1 berlauf Bits (Zeitmessung)
	goto $+8				;kein berlauf, weiter 
	movfw TMR0				;berlauf vorhanden, hole sicherheitshalber TMR0 Wert ins W-Register
	bcf PIR1, TMR1IF		;lsche Flag,
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	decfsz COUNT_H_S		;  reduziere Zeitzhler um 1 und prfe, ob Zhler =0
	goto $+2				;Zhler >0, weiter
	goto adj_freq			;Zhler =0, d.h. 1 Sekunde ist um, 
	btfss INTCON, T0IF		;Abfrage Timer 0 berlauf Bits (Anzahl L/H bergnge)
	goto freq_loop			;kein berlauf, zurck zur Schleife 
	bcf INTCON, T0IF		;berlauf vorhanden, lsche Flag,
	incfsz ZAHL_C1			;Erhhe Zhler um 1 und prfe berlauf
	goto freq_loop			;kein berlauf, zurck zur Schleife 
	incfsz ZAHL_C2			;Erhhe Zhler um 1 und prfe berlauf
	goto freq_loop			;kein berlauf, zurck zur Schleife 
	incf ZAHL_C3			;Erhhe Zhler um 1
	goto freq_loop			;zurck zur Schleife


adj_freq
	movwf ZAHL_C0			;sichern des TMR0 Wert vom W-Register ins ZAHL_C0 Register
	btfss INTCON, T0IF		;Abfrage Timer 0 berlauf Flag (Anzahl L/H bergnge)
	goto adj_freq1			;Flag nicht gesetzt, weiter 
	movlw d'10'				;wenn ZAHL_C0 zwischen 245 und 255, Flag spter gesetzt, wenn zwischen 0 und 10 frher 
	subwf ZAHL_C0,w
	btfsc STATUS,C
	goto adj_freq1			;Ergebnis ist positiv, d.h. Flag spter gesetzt, weiter
	incfsz ZAHL_C1			;Ergebnis ist negativ, d.h. Flag frher gesetzt, erhhe Zhler um 1 und prfe berlauf
	goto adj_freq1			;kein berlauf, zurck zur Schleife 
	incfsz ZAHL_C2			;Erhhe Zhler um 1 und prfe berlauf
	goto adj_freq1			;kein berlauf, zurck zur Schleife 
	incf ZAHL_C3			;Erhhe Zhler um 1	

adj_freq1
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	decf PRESC_TMR0				;reduziere den Zhler um 1 und prfe ob 0
	btfsc STATUS,Z				;
	goto adj_freq_end			;Multiplikation mit 2xZhlerinhalt

	bcf STATUS,C				;lsche Status Bit, damit es beim rotieren nicht strt
	rlf ZAHL_C0					;x2
	rlf ZAHL_C1					;x2
	rlf ZAHL_C2					;x2
	rlf ZAHL_C3					;x2
	goto adj_freq1	

adj_freq_end
	call test_vt			;Unterprogramm zum prfen ob Vorteiler gewhlt ist
							; und Multiplikation mit dem Vorteiler Faktor
test_freq_peri
	movfw PROGRAM			;prfen, ob Programm 0, d.h. Frequenz gewhlt ist
	btfss STATUS,Z
	goto freq_to_peri		;Programm ist >0<, d.h. Periode ist gewhlt, umrechnen der Frequenz in Periode
							;Programm ist 0, d.h. Frequenz ist gewhlt,

	clrf ZAHL_B0			;Lsche Zahl_B Register 
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	movlw 0x01				;lade Disp 0-3 mit Wert 1 (Messzeit)
	movwf DISP_0
	clrf DISP_1
	clrf DISP_2
	clrf DISP_3
	goto mhz_ps

freq_to_peri				;Programmteil zur Umwandlung von Frequenz in Periode
	movfw ZAHL_C0			;Kopiere Frequenz aus ZAHL_C ins Nennerregister (ZAHL_A) fr die Division,
	movwf ZAHL_A0
	movwf DISP_0
	movfw ZAHL_C1			
	movwf ZAHL_A1
	movwf DISP_1
	movfw ZAHL_C2
	movwf ZAHL_A2
	movwf DISP_2
	movfw ZAHL_C3
	movwf ZAHL_A3
	movwf DISP_3

	call clear_ZAHL_C		;lsche Zahl_C Register
	movlw 0x00				;lade Wert 1.000.000.000 (0x3B9ACA00) in Zahl_C Register
	movwf ZAHL_C0
	movlw 0xCA
	movwf ZAHL_C1
	movlw 0x9A
	movwf ZAHL_C2
	movlw 0x3B
	movwf ZAHL_C3
	call div32
	goto mhz_ps


Periode
	clrf STAT_INT			;Lsche Register und setze Bit fr Periode Interrup Routine
	bsf STAT_INT,1
	bsf STATUS,RP0			;Select BANK_1
	bcf PIE1, TMR1IE		;Timer1 interrupt verbieten
	movlw b'00101000'
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0

	movlw A'P'
	movwf DISP_PRESC

	clrf ZAHL_A0			;Vorbereiten zur Syncronisierung der positiven Flanke von RC2 mit Timer1
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call clear_ZAHL_C		;Lsche Register ZAHL_C fr die Zeitmessung
	clrf MESS_END
	movlw b'00000101'		;Capture mode, every rising edge
	movwf CCP1CON
	bcf T1CON,TMR1ON		;Timer 1 stoppen
	bcf PIR1, TMR1IF		;Timer 1 Interrupt flag lschen
	movlw d'14'
	movwf TMR1L
	clrf TMR1H	
	bcf PIR1,CCP1IF			;CCP1 Interrupt flag lschen
	bsf STATUS,RP0			;Select BANK_1
	bsf PIE1,CCP1IE			;CPP1 Interrupt erlauben
	bcf STATUS,RP0			;Select BANK_0
	bsf INTCON,PEIE			;Enables all unmasked peripheral interrupts
	bsf INTCON,GIE			;Interups erlauben
startloop_peri				;Warten auf die positive Flanke von RC2 (CPP1)
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms (Abbruch)
	goto menue

	btfss PIR1, TMR1IF		;
	goto startloop_p1				;0x09 = 9 (goto btfss MESS_END,1)
	bcf PIR1, TMR1IF
	incfsz ZAHL_C2
	goto $+2
	incf ZAHL_C3
	btfss ZAHL_C3,0			;Abfrage ob maximale Wartezeit abgelaufen ist (0x01000000 entspricht 16.777.216 Takte = 4,194304 s)
	goto startloop_p1		;Maximale Wartezeit noch nicht abgelaufen, weiter
	bcf INTCON,GIE			;maximale Wartezeit ist abgelaufen, Interups verbieten
	goto Frequenz			;maximale Messzeit ist erreicht, Abbruch und zurck zu Frequenz
startloop_p1
	btfss MESS_END,1		;Abfrage der positiven Flanke von RC2
	goto startloop_peri		;noch keine positive Flanke, zurck zur Warteschleife
	clrf MESS_END			;positive Flanke liegtan, Beginn der Perioden/Frequenzmessung

	bcf PIR1, TMR1IF
	clrf ZAHL_C2
	clrf ZAHL_C3
peri_loop
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	btfss PIR1, TMR1IF		;Abfrage des Timer1 berlaufbits
	goto $+0x08				;kein Timer 1 berlauf, weiter
	bcf PIR1, TMR1IF
	incf ZAHL_C2
	movlw 0x3D				;Abfrage ob Messzeit abgelaufen ist (0x003D0000 entspricht 3.997.696 Takte = 0,999424 s)
	subwf ZAHL_C2,w
	bcf PIR1,CCP1IF			;CCP1 Interrupt flag lschen
	btfsc STATUS,Z
	goto peri_loop1			;Messzeit von 0,999424 s abgelaufen
	btfss INTCON,TMR0IF
	goto peri_loop
	bcf INTCON,TMR0IF
	incfsz ZAHL_A1
	goto peri_loop	
	incf ZAHL_A2
	goto peri_loop			;Messzeit ist noch nicht abgelaufen, zurck zur Schleife
peri_loop1					;Messzeit von 0,999424 s abgelaufen, warten auf die nchste positive Flanke
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	btfss PIR1,CCP1IF		;Abfrage auf positive Flanke
	goto $+3				;Flag ist nicht gesetzt, weiter
	clrf CCP1CON
	goto auswert_peri
	btfss PIR1, TMR1IF
	goto peri_loop1
	bcf PIR1, TMR1IF
	incfsz ZAHL_C2
	goto $+2
	incf ZAHL_C3
	btfss ZAHL_C3,0			;Abfrage ob maximale Messzeit abgelaufen ist (0x01000000 entspricht 16.777.216 Takte = 4,194304 s)
	goto peri_loop1			;maximale Messzeit ist noch nicht abgelaufen, zurck zur Schleife
	goto Frequenz			;maximale Messzeit ist erreicht, Abbruch und zurck zu Frequenz 

auswert_peri
	movfw TMR0				;Flag ist gesetzt, Sichern des Timer0 Wertes in Zahl_A0 Register
	movwf ZAHL_A0
	btfss INTCON,TMR0IF
	goto $+4
	incfsz ZAHL_A1
	goto $+2	
	incf ZAHL_A2

	movfw CCPR1L			;sichern der Timer 1 Werte in Zahl C Registern
	movwf ZAHL_C0
	movfw CCPR1H
	movwf ZAHL_C1
	movfw ZAHL_C0			;temporres sichern der Mezeit (ZAHL_C) in Displayregistern
	movwf DISP_0
	movfw ZAHL_C1
	movwf DISP_1
	movfw ZAHL_C2
	movwf DISP_2
	movfw ZAHL_C3
	movwf DISP_3

	movlw 0x03				;prfen, ob Eingang Vorteiler gewhlt ist
	subwf EINGANG,w			;
	btfss STATUS,Z
	goto test_peri_freq		;Eingang Vorteiler ist nicht gewhlt, weiter bei Auswertung Periode ohne Vorteiler
							;Vorteiler ist gewhlt, multiplizieren der L=>H bergnge von RB0 (ZAHL_A) mit Vorteilerwert

	clrf ZAHL_B1			;lsche Multiplikationsregister (ZAHL_B)
	clrf ZAHL_B2
	clrf ZAHL_B3
	movfw MUL_VT			;Kopiere Vorteiler Faktor nach ZAHL_B0, wenn MUL_VT = 0 (=256) erhhe ZAHL_B1 um 1
	movwf ZAHL_B0
	btfsc STATUS,Z
	incf ZAHL_B1
	call mul32				;Aufruf des Unterprogramms zur Multiplikation, Das Ergebnis steht in ZAHL_C

	movfw ZAHL_C0			;Ergebnis zurck nach ZAHL_A (LH bergnge)
	movwf ZAHL_A0
	movfw ZAHL_C1
	movwf ZAHL_A1
	movfw ZAHL_C2
	movwf ZAHL_A2
	movfw ZAHL_C3
	movwf ZAHL_A3


test_peri_freq
	movfw PROGRAM			;prfen, ob Programm 0, d.h. Frequenz gewhlt ist
	btfss STATUS,Z
	goto auswert_peri_1		;Programm ist >0<, d.h. Periode ist gewhlt
			;Frequenz ist gewhlt, Frequenz = Anzahl der L=>H bergnge von RB0 (ZAHL_A) x 4.000.000 / Mezeit (ZAHL_C)

	movlw 0x00				;lade Wert 4.000.000 (0x003D0900) in Zhlerregister ZAHL_B fr die Multiplikation
	movwf ZAHL_B0	
	movlw 0x09
	movwf ZAHL_B1
	movlw 0x3D
	movwf ZAHL_B2
	movlw 0x00
	movwf ZAHL_B3
	call mul32				;Multipliziern der Anzahl der L=>H bergnge von RB0 (ZAHL_A) mit 4.000.000 (ZAHL_B), das
							;Ergebnis steht in ZAHL_C

	movfw DISP_0			;Rcksichern der Mezeit in Nennerregister (ZAHL_A) fr die Division
	movwf ZAHL_A0
	movfw DISP_1
	movwf ZAHL_A1
	movfw DISP_2
	movwf ZAHL_A2
	movfw DISP_3
	movwf ZAHL_A3
	call div32				;Division durch die Mezeit (ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B)

	goto mhz_ps				;Weiter bei Millihertz / Picosekunde

auswert_peri_1
	movfw ZAHL_A0			;zwischenspeichern der L/H bergnge in Disp_4-7 Registern
	movwf DISP_4
	movfw ZAHL_A1
	movwf DISP_5
	movfw ZAHL_A2
	movwf DISP_6
	movfw ZAHL_A3
	movwf DISP_7

	movfw DISP_0			;rcksichern der Mezeit aus Displayregistern nach ZAHL_A
	movwf ZAHL_A0
	movfw DISP_1
	movwf ZAHL_A1
	movfw DISP_2
	movwf ZAHL_A2
	movfw DISP_3
	movwf ZAHL_A3
	movlw d'250'			;Lade Multiplikations Register mit Wert 250
	movwf ZAHL_B0			;Messzeit x 1000 / 4MHz = Messzeit  x 250
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32				; A x B = C Messzeit steht in C
	movfw DISP_4			;rcksichern der L/H bergnge aus Displayregistern nach ZAHL_A
	movwf ZAHL_A0
	movfw DISP_5
	movwf ZAHL_A1
	movfw DISP_6
	movwf ZAHL_A2
	movfw DISP_7
	movwf ZAHL_A3
	call div32				;im ersten Schritt wird die Messzeit durch die Anzahl der L=>H bergnge von RB0 geteilt
							; ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B
	movfw ZAHL_A0			;temporres sichern der L=>H bergnge (ZAHL_A) in Displayregistern
	movwf DISP_0
	movfw ZAHL_A1
	movwf DISP_1
	movfw ZAHL_A2
	movwf DISP_2
	movfw ZAHL_A3
	movwf DISP_3


mhz_ps
	movlw 0x01				;wenn F/P <1  (0x00000001) => mHz/ps, wenn Frequenz >=1 gehe zu Hz/ns
	movwf TEMP_10
	movlw 0x00
	movwf TEMP_11
	movlw 0x00
	movwf TEMP_12
	movlw 0x00
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+2				;F/P <1 => mHz/ps
	goto hz_ns				;F/P >=1 gehe zu Hz/ns
							;F/P und Rest x 1.000.000.000, Rest durch Mezeit teilen und zu Frequenz addieren
	call copy_1bn_zahla		;kopiere Wert 1.000.000.000 (0x3B9ACA00) nach Zahl_A
	call fliesskomma1
	call copy_1bn_zahla		;kopiere Wert 1.000.000.000 (0x3B9ACA00) nach Zahl_A
	call fliesskomma2

	movlw A'm'				;ASCII fr "m" milli Hz
	btfsc PROGRAM,0			;prfen, ob Programm = Frequenz (0) oder Periode (1)
	movlw A'p'				;ASCII fr "p" pico Sekunde
	movwf UNIT				;
	goto display_main

hz_ns
	movlw 0xE8				;wenn F/P <1.000 ist (0x000003E8) => Hz/ns,
	movwf TEMP_10			;  wenn F/P >=1.000 ist, gehe zu kHz/us
	movlw 0x03
	movwf TEMP_11
	movlw 0x00
	movwf TEMP_12
	movlw 0x00
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+2				;F/P <1.000 => Hz/ns
	goto khz_us				;F/P >=1.000 gehe zu kHz/us 
							;F/P und Rest x 1.000.000, Rest durch Mezeit teilen und zu Frequenz addieren
	call copy_1mn_zahla		;kopiere Wert 1.000.000 (0x000F4240) nach Zahl_A
	call fliesskomma1
	call copy_1mn_zahla		;kopiere Wert 1.000.000 (0x000F4240) nach Zahl_A
	call fliesskomma2

	movlw 0x20				;ASCII fr "blank" 
	btfsc PROGRAM,0			;prfen, ob Programm = Frequenz (0) oder Periode (1)
	movlw A'n'				;ASCII fr "n" nano Sekunde
	movwf UNIT				;
	goto display_main

khz_us
	movlw 0x40				;wenn F/P <1.000.000 ist (0x000F4240) => kHz/us,
	movwf TEMP_10			;  wenn F/P >=1.000.000, gehe zu MHz/ms
	movlw 0x42
	movwf TEMP_11
	movlw 0x0F
	movwf TEMP_12
	movlw 0x00
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+2				;F/P <1.000.000 => kHz/us
	goto mhz_ms				;F/P >=1.000.000 gehe zu MHz/ms
							;F/P und Rest x 1.000, Rest durch Mezeit teilen und zu Frequenz addieren
	call copy_1th_zahla		;kopiere Wert 1.000 (0x000003E8) nach Zahl_A
	call fliesskomma1
	call copy_1th_zahla		;kopiere Wert 1.000 (0x000003E8) nach Zahl_A
	call fliesskomma2

	movlw A'k'				;ASCII fr "k" kilo Hz
	btfsc PROGRAM,0			;prfen, ob Programm = Frequenz (0) oder Periode (1)
	movlw A'u'				;ASCII fr "u" mikro Sekunde
	movwf UNIT				;
	goto display_main

mhz_ms
	movlw 0x00				;wenn F/P <1.000.000.000 ist (0x3B9ACA00) => MHz/ms,
	movwf TEMP_10			;  wenn F/P >=1.000.000.000, gehe zu GHz/s
	movlw 0xCA
	movwf TEMP_11
	movlw 0x9A
	movwf TEMP_12
	movlw 0x3B
	call test_kleiner		;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	btfss STATUS,C
	goto $+2				;F/P <1.000.000.000 => MHz/ms
	goto ghz_s				;F/P >=1.000.000.000 gehe zu GHz/s

	movlw A'M'				;ASCII fr "M" Mega Hz
	btfsc PROGRAM,0			;prfen, ob Programm = Frequenz (0) oder Periode (1)
	movlw A'm'				;ASCII fr "m" milli Sekunde
	movwf UNIT				;
	goto display_main

ghz_s
	movlw 0xE8				;F/P >=1.000.000.000, lade Wert 1.000 (0x000003E8) ins Nennerregister (ZAHL_A) fr die Division,
	movwf ZAHL_A0			;   die Frequenz steht noch in ZAHL_C
	movlw 0x03
	movwf ZAHL_A1
	movlw 0x00
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3
	call div32				;teile Frequenz durch 1.000 (ZAHL_C / ZAHL_A = ZAHL_C)

	movlw A'G'				;ASCII fr "G" Giga Hz
	btfsc PROGRAM,0			;prfen, ob Programm = Frequenz (0) oder Periode (1)
	movlw 0x20				;ASCII fr " "  Sekunde
	movwf UNIT				;
	goto display_main


display_main
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen der 32-Bit Periode in die einzelnen Digits (99.999.999)zur Anzeige auf dem Display
	movfw DISP_8
	btfsc STATUS,Z
	goto $+6
	movlw d'3'
	movwf LOOP_1
	movlw b'00010000'		;Komma nach der 3ten Stelle
	movwf KOMMA				;
	goto display_main1
	movfw DISP_7
	btfsc STATUS,Z
	goto $+6
	movlw d'2'
	movwf LOOP_1
	movlw b'00100000'		;Komma nach der 2ten Stelle
	movwf KOMMA				;
	goto display_main1
	movlw d'1'
	movwf LOOP_1
	movlw b'01000000'		;Komma nach der 1ten Stelle
	movwf KOMMA				;
display_main1
	decfsz LOOP_1
	goto $+2
	goto display_main2
	movfw DISP_1
	movwf DISP_0
	movfw DISP_2
	movwf DISP_1
	movfw DISP_3
	movwf DISP_2
	movfw DISP_4
	movwf DISP_3
	movfw DISP_5
	movwf DISP_4
	movfw DISP_6
	movwf DISP_5
	movfw DISP_7
	movwf DISP_6
	movfw DISP_8
	movwf DISP_7
	clrf DISP_8
	goto display_main1

display_main2
	bsf PCLATH,4			;Select Page 2
	call disp_main
	bcf PCLATH,4			;Select Page 0
	goto frequenz_1

Zaehler
	movlw d'3'				;Prfen, ob Eingang Vorteiler gewhlt ist
	subwf EINGANG,w
	btfsc STATUS,Z
	bcf OUT_VT_ONOFF		;ja, VT Eingang einschalten
	movlw d'2'				;prfen, ob DDS Eingang gewhlt ist
	subwf EINGANG,w
	btfsc STATUS,Z
	bsf TTL_ENABLE			;ja, TTL Ausgang einschalten
	bcf DDS_CONTROL_H,0		;lschen des Reset Bits im Control Register (DDS starten)
	bsf PCLATH,3			;Select Page 1
	call write_dds_control	;Aufruf des Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	bcf PCLATH,3			;Select Page 0
	clrf DISP_0				;Lsche Anzeigeregister fr Display
	clrf DISP_1
	clrf DISP_2
	clrf DISP_3
	clrf DISP_4
	clrf DISP_5
	clrf DISP_6
	clrf DISP_7
	clrf DISP_8
	bsf PCLATH,4			;Select Page 2
	call clear_display
	call disp_zaehl
	bcf PCLATH,4			;Select Page 0
	movlw b'00000101'		;Bit 3-0: Capture mode: every rising edge
	movwf CCP1CON
	bsf STATUS,RP0			;Select BANK_1
	bsf PIE1,CCP1IE			;CPP1 Interrupt erlauben
	bcf STATUS,RP0			;Select BANK_0
zaehler_1
	clrf STAT_INT
	bsf STAT_INT,2
	clrf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	clrf MESS_END

	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	btfsc IN_DOWN
	goto $-3
	bsf INTCON,PEIE			;Periphere Interups erlauben
	bcf PIR1,CCP1IF			;CCP1 Interrupt flag lschen
	bsf INTCON,GIE			;Interups erlauben
zaehl_loop
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	btfss IN_DOWN
	goto $+3
	bsf MESS_END,2
	goto $+4
	btfss MESS_END,3		;Abfrage Status flag
	goto zaehl_loop
	clrf MESS_END			;Status flag lschen
	movfw ZAHL_A0			;Kopiere Register 
	movwf ZAHL_C0
	movfw ZAHL_A1			;
	movwf ZAHL_C1
	movfw ZAHL_A2			;
	movwf ZAHL_C2
	movfw ZAHL_A3			;
	movwf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen der 32-Bit Zhlers in die einzelnen Digits (99.999.999)zur Anzeige auf dem Display
	bsf PCLATH,4			;Select Page 2
	call disp_zaehl
	bcf PCLATH,4			;Select Page 0
	btfss MESS_END,2
	goto zaehl_loop
	bcf INTCON,GIE			;Interups verbieten	
	movlw d'250'			;Wartezeit von 250ms
	bsf PCLATH,4			;Select Page 2
	call wait 		
	bcf PCLATH,4			;Select Page 0
	goto zaehler_1


vco_wob
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call prep_vco_wob		;Unterprogramm zur Berechnung von statischen Daten, 
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4

	bcf STATUS,RP0			;Select BANK_0	
	btfsc MESS_END,3		;prfe Fehler Bit (allgemein)
	goto vco_wob_fehler		;Obere Frequenz >=Maximalfrequenz Hz, gehe zu Fehler

	bsf PCLATH,4			;Select Page 2
	call clear_display
	call disp_vco
	bcf PCLATH,4			;Select Page 0

	btfss PROGRAM,2			;Prfen ob Programm VCO (=3 => 00000011) oder Wobbel (=4 => 00000100) gewhlt ist (Bit 2 prfen) 
	goto vco_messen			;Bit 2 ist nicht gesetzt, d.h. VCO ist gewhlt

	bcf PIR1, TMR1IF		;Timer 1 Interrupt flag lschen
	bsf T1CON,TMR1ON		;Timer 1 starten
	goto vco_wob_dac

wob_loop
	bsf STATUS,RP0			;Select BANK_1	
	btfss WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto inc_wobfreq		;Wobbel Automatisch ist gewhlt
	bcf STATUS,RP0			;Select BANK_0	

wob_man
	bsf PCLATH,3			;Select Page 1
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	bcf PCLATH,3			;Select Page 0
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto menue				;wenn ja berspringe diesen Befehl, wenn nein Rcksprung ins Men	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto menue				;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,5			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	goto inc_wobfreq		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,6			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	goto dec_wobfreq		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto inc_wobfreq		;Bit ist nicht gesetzt, weiter bei nchster Prfung
	btfsc BEFEHL,4			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	goto dec_wobfreq		;wenn ja berspringe diesen Befehl
	goto wob_man			;zurck zur Schleife 

inc_wobfreq					;Erhhung des 10bit (0-1024) Registers WOB_COUNT_L/H um die Schrittweite WOB_STEPW. Ist der Endwert 
							;erreicht (1024 = 0x0400) wird das Register bei der nchsten Erhhung auf 0x0000 zurckgesetzt,
	bsf STATUS,RP0			;Select BANK_1
	btfsc WOB_COUNT_H,2		;prfen ob Endwert von 1024 erreicht ist
	goto $+6				;ja, Endwert von 1024 ist erreicht => rcksetzen des Registers auf 0
	movfw WOB_STEPW			;nein, "Endwert Bit" ist nicht gesetzt, erhhe Register um Schrittweite 
	addwf WOB_COUNT_L,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incf WOB_COUNT_H,f		;bertrag vorhanden, erhhe High Byte um 1
	goto wobfreq_end		;
	clrf WOB_COUNT_L		;rcksetzen der Register auf 0
	clrf WOB_COUNT_H
	goto wobfreq_end
dec_wobfreq
	bsf STATUS,RP0			;Select BANK_1
	movfw WOB_COUNT_L		;prfen, ob Minimalwert von 0 erreicht ist
	btfss STATUS,Z
	goto $+4				;noch nicht 0
	movfw WOB_COUNT_H		;
	btfsc STATUS,Z
	goto $+6				;Minimalwert von 0 ist erreicht rcksetzen der Register auf Wert 1024
	movfw WOB_STEPW			;nein, "Endwert Bit" ist nicht gesetzt, reduziere Register um Schrittweite 
	subwf WOB_COUNT_L,f
	btfss STATUS,C			;prfe ob Ergebnis negativ
	decf WOB_COUNT_H,f		;bertrag vorhanden, reduziere High Byte um 1
	goto wobfreq_end		;kein bertrag vorhanden, Ergebnis ist positiv
	movlw 0x04				;Rcksetzen des Zhlers mit Wert 1024
	movwf WOB_COUNT_H
	clrf WOB_COUNT_L
	goto wobfreq_end
wobfreq_end
	bsf STATUS,RP0			;Select BANK_1	
	movfw WOB_COUNT_L		;kopieren der WOB_COUNT Register in AD_DA Register
	bcf STATUS,RP0			;Select BANK_0
	movwf AD_DA_L
	bsf STATUS,RP0			;Select BANK_1	
	movfw WOB_COUNT_H
	bcf STATUS,RP0			;Select BANK_0
	movwf AD_DA_H
	bsf STATUS,RP0			;Select BANK_1	
	btfss WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto vco_wob_dac		;Automatisch (0) ist gewhlt, weiter bei vco_wob_dac (keine Wartezeit)	
	bcf STATUS,RP0			;Select BANK_0, Manuell ist gewhlt, Wartezeit von 250ms
	bsf PCLATH,4			;Select Page 2
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	bcf PCLATH,4			;Select Page 0
	goto vco_wob_dac

vco_messen
	bsf	ADCON0,0			;ADC powered up
	bcf DDS_CONTROL_H,0		;lschen des Reset Bits	
	bsf PCLATH,3			;Select Page 1
	call write_dds_control	;Aufruf des Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	bcf PCLATH,3			;Select Page 0
vco_loop
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	call clear_ZAHL_C		;Lsche Register ZAHL_C
	clrf AD_DA_L			;Es werden 16 Werte gemessen, aufsummiert und daraus der Mittelwert berechnet (durch 16 geteilt)
	clrf AD_DA_H
	movlw d'16'				;schreibe Wert 16 in Zhler
	movwf LOOP_1
loop_summ
	bsf	ADCON0,2			;ADC starten
	btfsc ADCON0,2			;prfen ob der ADC fertig ist?
	goto $-1				;nein, weiter warten
	bsf STATUS,RP0			;Select BANK_1
	movfw ADRESL			;hole Low Byte (8 Bit) nach W und addiere zu AD/DA Register Low byte
	bcf STATUS,RP0			;Select BANK_0
	addwf AD_DA_L,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incf AD_DA_H,f			;bertrag vorhanden, erhhe High Byte um 1
	movfw ADRESH			;hole High Byte (2 Bit) nach W und addiere zu AD/DA Register High byte
	addwf AD_DA_H,f
	btfss ADRESH,1			;prfen ob AD Wert >511 ist
	goto loop_summ_sub1		;<=511, weiter
	incfsz AD_DA_L			;>511, erhhe Low Byte um 1 und prfe auf berlauf
	goto loop_summ_sub1		;kein berlauf
	incf AD_DA_H			;berlauf vorhanden, erhhe High Byte
loop_summ_sub1
	decfsz LOOP_1			;reduziere Zhler um 1 und prfe ob 0
	goto loop_summ			;Zhler hat noch nicht bis 0 runtergezhlt, zurck zur Schleife

	clrf AD_DA_REST			;Mittelwert berechnen; AD_DA_L/AD_DA_H Register durch 16 teilen (4x nach rechts verschieben)
	bsf AD_DA_REST,3		;setze Bit 3 => wenn 4 mal geschoben, erscheint das Bit im Carry
	bcf STATUS,C			;lsche Carry Bit,
	rrf AD_DA_H
	rrf AD_DA_L
	rrf AD_DA_REST
	btfss STATUS,C			;prfe, ob 4x geschoben wurde, d.h. das Carry Bit = High,
	goto $-4				;Carry Bit ist noch Low, d.h. noch nicht 4 mal geschoben, zurck zur Schleife 

	btfss AD_DA_REST,7		;Rest kann zwischen 0 und 240 liegen (00000000 - 11110000), Rest 128 (10000000) entspricht 0,5
	goto vco_wob_dac		;Rest < 128 (<0,5) (0xxxxxxx)
	incfsz AD_DA_L			;Rest >= 128 (>=0,5) (1xxxxxxx), erhhe Low Byte um 1 und prfe den berlauf
	goto vco_wob_dac		;kein berlauf
	incf AD_DA_H			;berlauf vorhanden, erhhe High Byte um 1

vco_wob_dac
	bcf STATUS,RP0			;Select BANK_0
	movfw AD_DA_L			;kopieren der AD_DA_L/H Register in die DAC Ausgabe Register
	movwf DAC_L
	movfw AD_DA_H
	movwf DAC_H
	movlw d'2'				;prfen, ob DAC Wert <512 ist (0x0200)
	subwf DAC_H,w			;
	btfss STATUS,C			;
	goto $+5				;ja, <512, weiter
	movlw d'1'				;nein, >=512, DAC Wert muss um 1 reduziert werden,
	subwf DAC_L,f			; da AD Wert 0-1024, DAC Wert aber nur 0-1023
	btfss STATUS,C			;
	decf DAC_H				;	

	bcf DAC_H,5				;Ausgang A (Frequenzausgabe) wird angesprochen (Bit 5 wird spter noch 2x nach links verschoben)
	bcf DAC_H,3				;setze Gain auf 0, d.h. 0-1023 entsprechen 4-4096 mV
	btfsc DAC_GAIN,0		;prfen ob gain auf 1 gesetzt werden soll
	bsf DAC_H,3				;ja, Gain = 1, d.h. 0-1023 entsprechen 2-2048 mV

	movfw KENNLIN			;prfen, ob Kennlinie Linear oder Logarithmisch gewhlt ist
	btfss STATUS,Z
	goto vco_wob_log		;Kennlinie >0<, d.h. nicht linear, goto vco_wob_log

	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call vco_wob_lin		;Kennlinie =0, d.h. linear, Aufruf Unterprogramm vco_wob_lin auf Page 3
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4
	goto vco_wob_ausgabe

vco_wob_log					;DEK_AKTUELL
	btfss AD_DA_H,2			;prfen ob AD_DA Register >1023
	goto dekaden1 			;AD_DA Register <1024, 
	clrf ZAHL_B0			;AD_DA Register >1023, lade ZAHL_B Register mit Wert 65.536 (dez.; hex: 0x00010000)
	clrf ZAHL_B1
	movlw 0x01
	movwf ZAHL_B2
	clrf ZAHL_B3
	movfw DEKADE			;AD_DA Register >1023 und damit in der letzten Dekade, kopiere Anzahl Dekaden 
	bsf STATUS,RP0			;Select BANK_1
	movwf DEK_AKTUELL		; in Register fr aktuelle Dekade
	bcf STATUS,RP0			;Select BANK_0
	goto log_mul_dek		;weiter
dekaden1
	btfsc DEKADE,1			;prfen wieviel Dekaden gewhlt sind
	goto dekaden2			;Bit 1 ist gesetzt, d.h. 2 oder 3 Dekaden
	movlw 0x01				;Bit 1 ist nicht gesetzt, d.h. 1 Dekade
	bsf STATUS,RP0			;Select BANK_1
	movwf DEK_AKTUELL		;=1
	bcf STATUS,RP0			;Select BANK_0
	goto ad_da_neu
dekaden2
	btfsc DEKADE,0			;prfen wieviel Dekaden gewhlt sind
	goto dekaden3			;Bit 0 ist gesetzt, d.h. 3 Dekaden
	bcf STATUS,C			;lsche Carry Bit, damit es beim Rotieren nicht strt
	rlf AD_DA_L,f			;AD_DA Register mit 2 multiplizieren ( 1x nach links verschieben)
	rlf AD_DA_H,f
	movlw 0x02				;hole Wert 2 nach W, falls AD_DA Register >1023, d.h. Aktuelle Dekade = 2
	btfss AD_DA_H,2			;prfen ob AD_DA Register >1023
	movlw 0x01				;AD_DA Register <1024, Aktuelle Dekade = 1
	bsf STATUS,RP0			;Select BANK_1
	movwf DEK_AKTUELL		;Schreibe aktuelle Dekade ( 1 oder 2 ) ins DEK_AKTUELL Register
	bcf STATUS,RP0			;Select BANK_0
	btfsc AD_DA_H,2			;prfen ob AD_DA Register <1024
	bcf AD_DA_H,2			;AD_DA Register >1023, reduziere High Byte um 1024
	goto ad_da_neu
dekaden3
	movfw AD_DA_H			;AD_DA Register mit 3 multiplizieren ( 1x nach links verschieben = x2 und 1x addieren)
	movwf TEMP_10			;High Byte zwischenspeichern
	movfw AD_DA_L			;Low Byte nach W holen
	bcf STATUS,C			;lsche Carry Bit, damit es beim Rotieren nicht strt
	rlf AD_DA_L,f			;AD_DA Register mit 2 multiplizieren ( 1x nach links verschieben)
	rlf AD_DA_H,f
	addwf AD_DA_L,f			;Low Byte addieren
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf AD_DA_H,f			;es ist ein bertrag vorhanden und das High Byte wird um 1 erhht
	movfw TEMP_10			;zwischengespeichertes High Byte addieren
	addwf AD_DA_H,f
	movlw 0x03				;hole Wert 3 nach W, falls AD_DA Register >2047, d.h. Aktuelle Dekade = 3
	btfsc AD_DA_H,3			;prfen ob AD_DA Register <2048
	goto $+4
	movlw 0x02				;hole Wert 2 nach W, falls AD_DA Register >1023, d.h. Aktuelle Dekade = 2
	btfss AD_DA_H,2			;prfen ob AD_DA Register >1023
	movlw 0x01				;AD_DA Register <1024, Aktuelle Dekade = 1
	bsf STATUS,RP0			;Select BANK_1
	movwf DEK_AKTUELL		;Schreibe aktuelle Dekade ( 1,2 oder 3 ) ins DEK_AKTUELL Register
	bcf STATUS,RP0			;Select BANK_0
	btfss AD_DA_H,3			;prfen ob AD_DA Register >2047
	goto $+3				;AD_DA Register <2048, weiter bei nchster Prfung
	bcf AD_DA_H,3			;AD_DA Register >2047, reduziere High Byte um 2048
	goto ad_da_neu
	btfsc AD_DA_H,2			;prfen ob AD_DA Register <1024
	bcf AD_DA_H,2			;AD_DA Register >1023, reduziere High Byte um 1024

ad_da_neu
	clrf AD_DA_REST			;Lsche Rest Register
	btfsc AD_DA_L,0
	bsf AD_DA_REST,0
	btfsc AD_DA_L,1
	bsf AD_DA_REST,1
	bcf STATUS,C			;lsche Carry Bit, damit es beim Rotieren nicht strt
	rrf AD_DA_H				;AD_DA Register durch 4 teilen ( 2x nach rechts verschieben, Rest steht in AD_DA_REST)
	rrf AD_DA_L
	bcf STATUS,C			;lsche Carry Bit, damit es beim Rotieren nicht strt
	rrf AD_DA_H				;
	rrf AD_DA_L

	clrf ZAHL_B2
	clrf ZAHL_B3

	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call log_tabelle
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			

	movfw ZAHL_B0			;L-byte n, Differenz berechnen 
	subwf TEMP_12,f			;L-byte n+1
	movfw ZAHL_B1			;H-byte n
	btfss STATUS,C
	addlw d'1'
	subwf TEMP_13,f			;H-byte n+1, Differenz steht in TEMP_13/12 Registern

	movfw AD_DA_REST		;Differenz x Rest /4, Prfen ob Rest =0
	btfsc STATUS,Z
	goto log_mul_dek		;Rest ist 0, weiter bei log_mul_dek
	btfsc AD_DA_REST,1		;prfen ob Rest =1
	goto $+2				;Rest ist 2 oder 3, weiter bei nchster Prfung
	goto div_vier			;Rest ist 1, Differenz durch vier teilen

	bcf STATUS,C			;Rest ist 2 oder 3, Differenz mit 2 multiplizieren (1x nach links verschieben)
	rlf TEMP_12
	rlf TEMP_13
	btfss AD_DA_REST,0		;prfen ob Rest =3
	goto div_vier			;Rest ist 2, Differenz durch vier teilen
	bcf STATUS,C			;Rest ist 3
	rrf TEMP_13,w
	rrf TEMP_12,w	
	addwf TEMP_12,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incf TEMP_13,f			;es ist ein bertrag vorhanden, erhhe High Byte um 1
	bcf STATUS,C			;
	rrf TEMP_13,w
	addwf TEMP_13,f	

div_vier
	clrf TEMP_14			;wird fr Rest genutzt
	bcf STATUS,C			;lsche Carry Bit und teile Differenz x Rest durch 4 (2x nach rechts verschieben)
	rrf TEMP_13
	rrf TEMP_12
	rrf TEMP_14
	bcf STATUS,C			;lsche Carry Bit
	rrf TEMP_13
	rrf TEMP_12
	rrf TEMP_14
	btfss TEMP_14,7
	goto $+4
	incfsz TEMP_12
	goto $+2
	incf TEMP_13

	movfw TEMP_12
	addwf ZAHL_B0,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incfsz ZAHL_B1,f		;es ist ein bertrag vorhanden, erhhe High Byte um 1
	goto $+2
	incf ZAHL_B2
	movfw TEMP_13
	addwf ZAHL_B1,f
	btfsc STATUS,C			;prfen ob ein bertrag vorhanden ist
	incf ZAHL_B2,f			;es ist ein bertrag vorhanden, erhhe High Byte um 1
	clrf ZAHL_B3

log_mul_dek
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call log_mul_dek1		;Unterprogramm zum Kopieren des Multiplikators fr die aktuelle Dekade in ZAHL_C Register
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			

	call mul32				;Aufruf des Unterprogramms zur Multiplikation (ZAHL_A x ZAHL_B = ZAHL_C)

	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call div_2h16			;Aufruf des Unterprogramms zur Division durch 2^16
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			

vco_wob_ausgabe
	movfw ZAHL_C0			;Kopiere Frequenz (DDS Format) in DDS_FREQREG Register
	movwf DDS_FREQREG_0
	movfw ZAHL_C1
	movwf DDS_FREQREG_1
	movfw ZAHL_C2
	movwf DDS_FREQREG_2
	movfw ZAHL_C3
	movwf DDS_FREQREG_3

	btfsc MESS_END,1		;prfen, ob delta t <0,2s
	goto $+7				;delta t <0,2s, Unterprogramme "calc_frequenz" und "convert_hex_dig" werden nicht bentigt
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call calc_frequenz		;
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen der 32-Bit Zhlers in die einzelnen Digits (99.999.999)zur Anzeige auf dem Display	

	bsf STATUS,RP0			;Select BANK_1	
	btfss WOB_M_A,1			;prfen, ob Burst gewhlt ist
	goto $+6				;Burst ist nicht gewhlt
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call calc_burst_length	;Burst ist gewhlt, Berechnung der Burstlnge
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			
	bcf STATUS,RP0			;Select BANK_0	
	bsf PCLATH,3			;Select Page 1
	call dds_freqformat		;Umwandlung der DDS_FREQREG Register in das AD9833 Frequenzregister Format
	bcf PCLATH,3			;Select Page 0

	btfss PROGRAM,2			;Prfen ob Programm VCO (=3 => 00000011) oder Wobbel (=4 => 00000100) gewhlt ist (Bit 2 prfen) 
	goto vco_ausgabe		;Bit 2 ist nicht gesetzt, d.h. VCO ist gewhlt	

	bsf STATUS,RP0			;Select BANK_1	
	btfss WOB_M_A,1			;prfen, ob Burst gewhlt ist (Burst = 1, Normal = 0)
	goto wob_normal			;Burst ist nicht gewhlt

	bsf STATUS,RP0			;Select BANK_1	
	btfsc WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto wob_man_burst		;Wobbel Manuell ist gewhlt, berspringe den Timer fr delta t

	bcf STATUS,RP0			;Select BANK_0	
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	btfss MESS_END,7		;Abfrage, ob Timer fr delta t abgelaufen ist (von Interrupt)
	goto $-3				;Timer noch nicht abgelaufen, zurck zur Schleife

wob_man_burst
	bcf STATUS,RP0			;Select BANK_0		
	bcf INTCON,GIE			;Interrupts verbieten
	bcf MESS_END,7			;Timer flag lschen
	bsf PCLATH,3			;Select Page 1
	call send_dac_data		;Aufruf des Unterprogramms zum senden der Daten an den DAC
	call write_dds_freq		;Aufruf des Unterprogramms zum Schreiben des Frequenz Registers, DDS Chips AD9833
	bcf PCLATH,3			;Select Page 0

	bcf DDS_CONTROL_H,0		;lschen des Reset Bits	
	bsf STATUS,RP0			;Select BANK_1
	movfw TBURST_L
	bcf STATUS,RP0			;Select BANK_0
	movwf TMR0
	bcf INTCON,TMR0IF

	bsf PCLATH,3			;Select Page 1
	call write_dds_control	;Aufruf des Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	bcf PCLATH,3			;Select Page 0

	bsf DDS_CONTROL_H,0		;setzen des Reset Bits	
	bsf INTCON,GIE			;Interrupts erlauben

	btfss INTCON,TMR0IF		;Abfrage, ob Timer 0 bergelaufen ist
	goto $-1
	bcf INTCON,TMR0IF
	bcf STATUS,RP0			;Select BANK_0
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	bsf STATUS,RP0			;Select BANK_1
	incfsz TBURST_M			;erhhe Zhler und prfe ob 0
	goto $-8				;Zhler noch nicht 0,
	incfsz TBURST_H			;reduziere Zhler und prfe ob 0
	goto $-0x0A				;Zhler noch nicht 0,

	bcf STATUS,RP0			;Select BANK_0
	bcf INTCON,GIE
	bsf PCLATH,3			;Select Page 1
	call write_dds_control	;Aufruf des Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	bcf PCLATH,3			;Select Page 0
	bsf INTCON,GIE
	
	goto wob_ausgabe

wob_normal
	bsf STATUS,RP0			;Select BANK_1
	btfsc WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto wob_man_norm		;Wobbel Manuell ist gewhlt, berspringe den Timer fr delta t
	bcf STATUS,RP0			;Select BANK_0
	
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	btfss MESS_END,7		;Abfrage, ob Timer fr Schrittweite abgelaufen ist (von Interrupt)
	goto $-3
	bcf MESS_END,7			;Timer flag lschen

wob_man_norm
	bcf STATUS,RP0			;Select BANK_0	
	bsf PCLATH,3			;Select Page 1
	call write_dds_freq		;Aufruf des Unterprogramms zum Schreiben des Frequenz Registers, DDS Chips AD9833
	bcf PCLATH,3			;Select Page 0
	btfss DDS_CONTROL_H,0	;prfen ob DDS Reset Bit gesetzt ist
	goto $+4				;DDS Reset Bit ist nicht gesetzt, berspringe die nchsten 3 Befehle
	bcf DDS_CONTROL_H,0		;DDS Reset Bit ist gesetzt, lschen des Reset Bits	
	bsf PCLATH,3			;Select Page 1	
	call write_dds_control	;Aufruf des Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833 (DDS starten)
	bsf PCLATH,3			;Select Page 1
	call send_dac_data		;Aufruf des Unterprogramms zum senden der Daten an den DAC
	bcf PCLATH,3			;Select Page 0

wob_ausgabe
	btfsc MESS_END,2
	goto wob_loop			;Bit fr keine Anzeige ist gesetzt, zurck
	btfsc MESS_END,1
	goto wob_ausgabe1		;Bit fr grafische Anzeige ist gesetzt, goto wob_ausgabe1
	bsf PCLATH,4			;Select Page 2
	call disp_vco_freq		;Normale Frequenzanzeige wird verwendet
	bcf PCLATH,4			;Select Page 0
	goto wob_loop
wob_ausgabe1
	bsf PCLATH,4			;Select Page 2
	call disp_wob_graph
	bcf PCLATH,4			;Select Page 0
	goto wob_loop

vco_ausgabe
	bsf PCLATH,3			;Select Page 1
	call write_dds_freq		;Aufruf des Unterprogramms zum Schreiben des Frequenz Registers, DDS Chips AD9833
	call send_dac_data		;Aufruf des Unterprogramms zum senden der Daten an den DAC
	bcf PCLATH,3			;Select Page 0
	bsf PCLATH,4			;Select Page 2
	call disp_vco_freq
	bcf PCLATH,4			;Select Page 0
	bcf T1CON,TMR1ON		;Timer 1 stoppen
	movlw d'15'				;Wartezeit von ca. 250ms
	movwf COUNT_H_S
	clrf TMR1H				;Timer 1 lschen
	clrf TMR1L
	bcf PIR1,TMR1IF			;Flag lschen
	bsf T1CON,TMR1ON		;Timer 1 starten
vco_wait
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms
	goto menue
	btfss PIR1,TMR1IF
	goto vco_wait
	bcf PIR1,TMR1IF
	decfsz COUNT_H_S
	goto vco_wait
	goto vco_loop


vco_wob_fehler				;Fo > Fmax
	bsf PCLATH,4			;Select Page 2
	call clear_display
	bcf PCLATH,4			;Select Page 0
	btfss MESS_END,4		;prfen ob Fehler 1 anliegt
	goto $+5				;Fehler 1 liegt nicht an, =>weiter
	bsf PCLATH,4			;Select Page 2
	call disp_fehler1		;Fehler 1 liegt an, Aufruf des Unterprogramms zur Anzeige der Fehlermeldung "Fu/Dek zu hoch"
	bcf PCLATH,4			;Select Page 0
	goto fehler_loop
	bsf PCLATH,4			;Select Page 2
	call disp_fehler2		;Fehler 2 liegt an, Aufruf des Unterprogramms zur Anzeige der Fehlermeldung "Fu/dt zu klein"
	bcf PCLATH,4			;Select Page 0
fehler_loop
	bsf PCLATH,3			;Select Page 1
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle, mit Men oder OK wird die Meldung quittiert
	bcf PCLATH,3			;Select Page 0
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto fehler_end			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto fehler_end			;wenn ja berspringe diesen Befehl
	goto fehler_loop
fehler_end
	bsf PCLATH,4			;Select Page 2
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	bcf PCLATH,4			;Select Page 0
	goto menue



fliesskomma1
	movfw ZAHL_C0			;Zahl_C0-C3 nach Disp_4-7 Register kopieren
	movwf DISP_4
	movfw ZAHL_C1
	movwf DISP_5
	movfw ZAHL_C2
	movwf DISP_6
	movfw ZAHL_C3
	movwf DISP_7
	call mul32				; ZAHL_A x ZAHL_B = ZAHL_C

	movfw DISP_0			;Rcksichern der Mezeit in Nennerregister (ZAHL_A) fr die Division
	movwf ZAHL_A0
	movfw DISP_1
	movwf ZAHL_A1
	movfw DISP_2
	movwf ZAHL_A2
	movfw DISP_3
	movwf ZAHL_A3
	call div32				;Division durch die Mezeit (ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B)

	movfw ZAHL_C0			;temporres sichern des Zwischenergebnisses (Rest x Faktor / Mezeit) in Displayregistern
	movwf DISP_0
	movfw ZAHL_C1
	movwf DISP_1
	movfw ZAHL_C2
	movwf DISP_2
	movfw ZAHL_C3
	movwf DISP_3

	movfw DISP_4			;Rcksichern der ganzzahligen Frequenz in Multiplikationsregister (ZAHL_B) fr die Multiplikation
	movwf ZAHL_B0
	movfw DISP_5
	movwf ZAHL_B1
	movfw DISP_6
	movwf ZAHL_B2
	movfw DISP_7
	movwf ZAHL_B3	
	return

fliesskomma2
	call mul32				;Multiplizieren der ganzzahligen Frequenz mit Faktor: ZAHL_A x ZAHL_B = ZAHL_C

	movfw DISP_0			;Rcksichern des Zwischenergebnisses (Rest x Faktor / Mezeit) in Additionsregister ZAHL_B
	movwf ZAHL_B0			; der ganzzahlige Teil steht noch im zweiten Additionsregister ZAHL_C
	movfw DISP_1
	movwf ZAHL_B1
	movfw DISP_2
	movwf ZAHL_B2
	movfw DISP_3
	movwf ZAHL_B3	
	call add32				;Das Ergebnis der Addition steht in ZAHL_C (ZAHL_B + ZAHL_C = ZAHL_C)
	return

test_kleiner					;Unterprogramm zum prfen, ob ZAHL_C < W-Reg., Start mit hchstem Byte
	subwf ZAHL_C3,w				;subtrahiere W-Reg. von ZAHL_C3
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Ergebnis negativ (STATUS,C=0), Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0  (STATUS,Z=1)
	return						;Ergebnis positiv, Rcksprung
	movfw TEMP_12				;Ergebnis = 0, nchst kleineres Byte prfen, hole TEMP_12 ins W-Reg.
	subwf ZAHL_C2,w				;subtrahiere W-Reg. von ZAHL_C2
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	return						;Ergebnis positiv, Rcksprung
	movfw TEMP_11				;Ergebnis = 0, nchst kleineres Byte prfen, hole TEMP_11 ins W-Reg.
	subwf ZAHL_C1,w				;subtrahiere W-Reg. von ZAHL_C1
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	return						;Ergebnis positiv, Rcksprung
	movfw TEMP_10				;Ergebnis = 0, nchst kleineres Byte prfen, hole TEMP_10 ins W-Reg.
	subwf ZAHL_C0,w				;subtrahiere W-Reg. von ZAHL_C0
	return

test_vt
	movlw 0x03				;prfen, ob Eingang Vorteiler gewhlt ist
	subwf EINGANG,w			;
	btfss STATUS,Z
	return					;Eingang Vorteiler ist nicht gewhlt, zurck
	movfw ZAHL_C0			;Eingang Vorteiler ist gewhlt, multiplizieren der Frequenz mit dem Vorteiler Faktor
	movwf ZAHL_A0			;Kopiere die Frequenz aus Zahl_C in das eine Multiplikationsregister (ZAHL_A)
	movfw ZAHL_C1
	movwf ZAHL_A1
	movfw ZAHL_C2
	movwf ZAHL_A2
	movfw ZAHL_C3
	movwf ZAHL_A3
	clrf ZAHL_B1			;lsche Multiplikationsregister (ZAHL_B)
	clrf ZAHL_B2
	clrf ZAHL_B3
	movfw MUL_VT			;Kopiere Vorteiler Faktor nach ZAHL_B0, wenn MUL_VT = 0 (=256) erhhe ZAHL_B1 um 1
	movwf ZAHL_B0
	btfsc STATUS,Z
	incf ZAHL_B1
	call mul32				;Aufruf des Unterprogramms zur Multiplikation, Das Ergebnis steht in ZAHL_C
	return

copy_1bn_zahla
	movlw 0x00				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000.000.000 (0x3B9ACA00), Rest steht noch im 
	movwf ZAHL_A0			; zweiten Multiplikationsregister ZAHL_B
	movlw 0xCA
	movwf ZAHL_A1
	movlw 0x9A
	movwf ZAHL_A2
	movlw 0x3B
	movwf ZAHL_A3	
	return

copy_1mn_zahla		
	movlw 0x40				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000.000 (0x000F4240), Rest steht noch im 
	movwf ZAHL_A0			; zweiten Multiplikationsregister ZAHL_B
	movlw 0x42
	movwf ZAHL_A1
	movlw 0x0F
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	return

copy_1th_zahla
	movlw 0xE8				;Lade Multiplikationsregister ZAHL_A mit Wert 1.000 (0x000003E8), Rest steht noch im 
	movwf ZAHL_A0			; zweiten Multiplikationsregister ZAHL_B
	movlw 0x03
	movwf ZAHL_A1
	movlw 0x00
	movwf ZAHL_A2
	movlw 0x00
	movwf ZAHL_A3	
	return


clear_ZAHL_C
	clrf ZAHL_C0			;Register ZAHL_C0 - ZAHL_C7 lschen
	clrf ZAHL_C1
	clrf ZAHL_C2
	clrf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	return

;*******************************************************************************************
; 32 bit Division: ZAHL_C / ZAHL_A = ZAHL_C, Rest steht in ZAHL_B
; Die Werte fr den Zhler mssen in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H) stehen
; Die Werte fr den Nenner mssen in den Registern ZAHL_A0 (L) bis ZAHL_A3 (H) stehen
; Das ganzzahlige Ergebnis steht dann in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H)
; Der Rest steht in den Registern ZAHL_B0 (L) bis ZAHL_B3 (H)
;*******************************************************************************************

div32
	clrf ZAHL_B3			;Rest von vorherigen Berechnungen lschen, ZAHL_B
	clrf ZAHL_B2
	clrf ZAHL_B1
	clrf ZAHL_B0					
	movlw d'64'				;LOOP_1 mit 64 laden, da 64 bit im Zhler
	movwf LOOP_1
div32_loop
	rlf ZAHL_C0,F       	;den gesamtem Zhler um 1 nach links schieben, ZAHL_C
	rlf ZAHL_C1,F
	rlf ZAHL_C2,F
	rlf ZAHL_C3,F
	rlf ZAHL_C4,F
	rlf ZAHL_C5,F
	rlf ZAHL_C6,F
	rlf ZAHL_C7,F

	rlf ZAHL_B0,F			;das Register fr den Rest um 1 nach links schieben
	rlf ZAHL_B1,F
	rlf ZAHL_B2,F
	rlf ZAHL_B3,F

	rlf LOOP_1,F			;das Carrybit wird vorbergehend im LOOP_1 gespeichert
									;dadurch kann man sich ein Register sparen	
	bsf LOOP_1,0
	movf ZAHL_A0,W			;Subtrahiere Nenner _0 von Rest _0 (REST_0-NENNER_0=REST_0)
	subwf ZAHL_B0,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B1,F			;reduziere REST_1 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B2,F			;reduziere REST_2 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_1,0			;bertrag vorhanden, lsche Bit 0 in LOOP_1

div32_sub2
	movf ZAHL_A1,W			;Subtrahiere Nenner _1 von Rest _1 (REST_1-NENNER_1=REST_1)
	subwf ZAHL_B1,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub3			;kein bertrag
	movlw 0x01
	subwf ZAHL_B2,F			;reduziere REST_2 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub3			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_1,0			;bertrag vorhanden, lsche Bit 0 in LOOP_1

div32_sub3
	movf ZAHL_A2,W			;Subtrahiere Nenner _2 von Rest _2 (REST_2-NENNER_2=REST_2)
	subwf ZAHL_B2,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub4			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_1,0			;bertrag vorhanden, lsche Bit 0 in LOOP_1

div32_sub4
	movf ZAHL_A3,W			;Subtrahiere Nenner _3 von Rest _3 (REST_3-NENNER_3=REST_3)
	subwf ZAHL_B3,F			;

	btfss STATUS, C			;prfe, ob das Ergebnis positiv ist
	bcf LOOP_1,0			;wenn nein, lsche gespeichertes Carrybit in LOOP_1
	btfsc LOOP_1,0			;prfe das gespeicherte Carrybit in LOOP_1
	goto div32_jump			;wenn Carry=1: springe nach div16_jump

	movf ZAHL_A0,W			;wenn Carry=0:	Subtraktion wieder rckgngig machen 
	addwf ZAHL_B0,F			; 
	btfsc STATUS,C
	incfsz ZAHL_B1,F
	goto $+4
	incfsz ZAHL_B2,F
	goto $+2
	incf ZAHL_B3,F

	movf ZAHL_A1,W			;
	addwf ZAHL_B1,F			;
	btfsc STATUS,C
	incfsz ZAHL_B2,F
	goto $+2
	incf ZAHL_B3,F

	movf ZAHL_A2,W			; 
	addwf ZAHL_B2,F			; 
	btfsc STATUS,C
	incf ZAHL_B3,F

	movf ZAHL_A3,W			;
	addwf ZAHL_B3,F			; 

div32_jump
	bcf STATUS,C			;Carrybit lschen und das gespeicherte Carrybit in Register
	rrf LOOP_1,F			;LOOP_1 wieder nach rechts rausschieben.
	decfsz LOOP_1,F 		;LOOP_1 um 1 verringern und prfen ob 0
	goto div32_loop         ;Counter ist noch nicht 0, zurck zur Schleife
	 
	rlf ZAHL_C0,F			;ZAEHLER_L		gesamter Zhler noch einmal nach links schieben, damit
	rlf ZAHL_C1,F			;ZAEHLER_M		das richtige Ergebnis im Zhler steht
	rlf ZAHL_C2,F			;ZAEHLER_H
	rlf ZAHL_C3,F			;ZAEHLER_H
	rlf ZAHL_C4,F			;ZAEHLER_H
	rlf ZAHL_C5,F			;ZAEHLER_H
	rlf ZAHL_C6,F			;ZAEHLER_H
	rlf ZAHL_C7,F			;ZAEHLER_H

	return



;*******************************************************************************************
; 32 bit Multiplikation: ZAHL_A x ZAHL_B = ZAHL_C
; Die Werte fr die Multiplikation mssen in den Registern ZAHL_A0 (L) bis ZAHL_A3 (H) 
; und ZAHL_B0 (L) bis ZAHL_B3 (H) stehen
; Das Ergebnis steht dann in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H)
;*******************************************************************************************
mul32

	call clear_ZAHL_C		;Lschen des vorherigen Ergebnisses (ZAHL_C0 - ZAHL_C7)
	bsf ZAHL_C3,7			;um zu erkennen, wann 32 mal geschoben wurde
mul32_loop
	rrf ZAHL_A3,F			;verschiebe gesammtes Register Multiplikant1 um 1 nach rechts
	rrf ZAHL_A2,F			;
	rrf ZAHL_A1,F			;
	rrf ZAHL_A0,F			;wenn das LSB eine 1 ist wird das Carry gesetzt und es muss
	btfss STATUS,C			;eine Addition durchgefhrt werden ist das Carry = 0 entspricht
	goto mul32_rotate		;dies einer Multiplikation mit 0 und es wird nichts addiert

	movf ZAHL_B0,W			;Beginn der 32 bit Addition: Addiere Multiplikant2 _0 Register zu Ergebnis _4 Register
	addwf ZAHL_C4,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C5,F		;es ist ein bertrag vorhanden und das Ergebnis _5 Register wird um 1 erhht
	goto $+4
	incfsz ZAHL_C6,F		;wenn Ergebnis _5 Register berluft, erhhe Ergebnis _6 Register um 1
	goto $+2
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B1,W			;Addiere Multiplikant2 _1 Register zu Ergebnis _5 Register
	addwf ZAHL_C5,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C6,F		;es ist ein bertrag vorhanden und das Ergebnis _6 Register wird um 1 erhht
	goto $+2
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B2,W			;Addiere Multiplikant2 _2 Register zu Ergebnis _6 Register
	addwf ZAHL_C6,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B3,W			;Addiere Multiplikant2 _3 Register zu Ergebnis _7 Register
	addwf ZAHL_C7,F			;

mul32_rotate
	rrf ZAHL_C7,F			;Das 64 Bit Ergebnisregister wird um 1 nach rechts geschoben
	rrf ZAHL_C6,F
	rrf ZAHL_C5,F
	rrf ZAHL_C4,F
	rrf ZAHL_C3,F
	rrf ZAHL_C2,F
	rrf ZAHL_C1,F
	rrf ZAHL_C0,F
	btfss STATUS,C			;ist das Bit 7 von ZAHL_C3 im Carry angekommen wurde das Ergebnis
	goto mul32_loop			;insgesamt 32 mal rotiert und die Berechnung ist beendet
	return                  ;

;*******************************************************************************************
; 32 bit Addition: ZAHL_C + ZAHL_B = ZAHL_C
; Die Werte fr die Addition mssen in den Registern ZAHL_C0 (L) bis ZAHL_C3 (H)
; und ZAHL_B0 (L) bis ZAHL_B3 (H) stehen
; Das Ergebnis steht in den Registern ZAHL_C0 (L) bis ZAHL_C4 (H)
;*******************************************************************************************

add32
	movf ZAHL_B0,W			;Hole Low Byte Zahl2 ins W-Register
	addwf ZAHL_C0,F			;addiere Low Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C1,F		;es ist ein bertrag vorhanden und das L+1 Byte von Zahl2 wird um 1 erhht
	goto $+6				;kein bertrag beim L+1 Byte von Zahl2, 
	incfsz ZAHL_C2,F		;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht
	goto $+4				;kein bertrag beim L+2 Byte von Zahl2, 
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht

	movf ZAHL_B1,W			;Hole L+1 Byte Zahl2 ins W-Register
	addwf ZAHL_C1,F			;addiere L+1 Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C2,F		;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht
	goto $+4				;kein bertrag beim L+2 Byte von Zahl2, 
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht

	movf ZAHL_B2,W			;Hole L+2 Byte Zahl2 ins W-Register
	addwf ZAHL_C2,F			;addiere L+2 Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht

	movf ZAHL_B3,W			;Hole High Byte Zahl2 ins W-Register
	addwf ZAHL_C3,F			;addiere High Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht
	return

;*******************************************************************************************
; 32 bit Subtraktion: ZAHL_C - ZAHL_B = ZAHL_C
; Die Werte fr die Subtraktion mssen in den Registern ZAHL_B0 (L) bis ZAHL_B3 (H)
; und ZAHL_C0 (L) bis ZAHL_C4 (H) stehen, wobei ZAHL_B von ZAHL_C subtrahiert wird 
; Das Ergebnis steht in den Registern ZAHL_C0 (L) bis ZAHL_C4 (H)
;*******************************************************************************************

sub32
	movfw ZAHL_B0			;Hole 0/7-Byte von B nach W
	subwf ZAHL_C0,F			;Subtrahiere B von C, 0/7-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 8/15-Byte
	subwf ZAHL_C1,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf ZAHL_C2,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_2
	movfw ZAHL_B1			;Hole 8/15-Byte von B nach W
	subwf ZAHL_C1,F			;Subtrahiere B von C, 8/15-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf ZAHL_C2,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_3
	movfw ZAHL_B2			;Hole 16/23-Byte von B nach W
	subwf ZAHL_C2,F			;Subtrahiere B von C, 16/23-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_4			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 24/31-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_4			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 24/31-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_4
	movfw ZAHL_B3			;Hole 24/31-Byte von B nach W
	subwf ZAHL_C3,F			;Subtrahiere B von C, 24/31-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_end			;Ergebnis ist positiv (Carry Bit = H) weiter zum Ende des Unterprogramms
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_end
	return


convert_hex_dig				;Unterprogramm zum Zerlegen einer 32-Bit Zahl in die einzelnen Digits (max.999.999.999)zur Anzeige auf dem Display
	clrf DISP_8				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	clrf ZAHL_C4			;
	movlw 0x00				;Lade ZAHL_B mit 100.000.000 (0x05F5E100)
	movwf ZAHL_B0
	movlw 0xE1				;
	movwf ZAHL_B1
	movlw 0xF5				;
	movwf ZAHL_B2
	movlw 0x05				;
	movwf ZAHL_B3
	incf DISP_8				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_8				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_7				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x80				;Lade ZAHL_B mit 10.000.000 (0x00989680)
	movwf ZAHL_B0
	movlw 0x96				;
	movwf ZAHL_B1
	movlw 0x98				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_7				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_7				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_6				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x40				;Lade ZAHL_B mit 1.000.000 (0x000F4240)
	movwf ZAHL_B0
	movlw 0x42				;
	movwf ZAHL_B1
	movlw 0x0F				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_6				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_6				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_5				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0xA0				;Lade ZAHL_B mit 100.000 (0x000186A0)
	movwf ZAHL_B0
	movlw 0x86				;
	movwf ZAHL_B1
	movlw 0x01				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_5				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_5				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_4				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x10				;Lade ZAHL_B mit 10.000 (0x00002710)
	movwf ZAHL_B0
	movlw 0x27				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_4				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_4				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_3				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0xE8				;Lade ZAHL_B mit 1.000 (0x000003E8)
	movwf ZAHL_B0
	movlw 0x03				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_3				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_3				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_2				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x64				;Lade ZAHL_B mit 100 (0x00000064)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_2				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_2				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_1				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x0A				;Lade ZAHL_B mit 10 (0x0000000A)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_1				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_1				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	movfw ZAHL_C0			;
	movwf DISP_0
	return

convert_dig_hex				;Unterprogramm zum Umwandeln der einzelnen Digits (max.999.999.999) in eine 32-Bit Zahl
	call clear_ZAHL_C		;Lsche Additionsregister ZAHL_C

	movfw DISP_8			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x0C				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x00				;Lade Additionsregister ZAHL_B mit 100.000.000 (0x05F5E100)
	movwf ZAHL_B0
	movlw 0xE1				;
	movwf ZAHL_B1
	movlw 0xF5				;
	movwf ZAHL_B2
	movlw 0x05				;
	movwf ZAHL_B3	
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_8			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife

	movfw DISP_7			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x0B				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x80				;Lade ZAHL_B mit 10.000.000 (0x00989680)
	movwf ZAHL_B0
	movlw 0x96				;
	movwf ZAHL_B1
	movlw 0x98				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_7			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife

	movfw DISP_6			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x0B				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x40				;Lade ZAHL_B mit 1.000.000 (0x000F4240)
	movwf ZAHL_B0
	movlw 0x42				;
	movwf ZAHL_B1
	movlw 0x0F				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_6			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife

	movfw DISP_5			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x0B				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0xA0				;Lade ZAHL_B mit 100.000 (0x000186A0)
	movwf ZAHL_B0
	movlw 0x86				;
	movwf ZAHL_B1
	movlw 0x01				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_5			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife

	movfw DISP_4			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x0A				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x10				;Lade ZAHL_B mit 10.000 (0x00002710)
	movwf ZAHL_B0
	movlw 0x27				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_4			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife

	movfw DISP_3			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x0A				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0xE8				;Lade ZAHL_B mit 1.000 (0x000003E8)
	movwf ZAHL_B0
	movlw 0x03				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_3			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife

	movfw DISP_2			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x09				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x64				;Lade ZAHL_B mit 100 (0x00000064)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_2			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife

	movfw DISP_1			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x09				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x0A				;Lade ZAHL_B mit 10 (0x0000000A)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_1			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife

	movfw DISP_0			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x06				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movwf ZAHL_B0			;Speichere Digit im Additionsregister
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C

	return

menue
	bsf PCLATH,3
	goto menue_1

;*******************************************************************************************
;	PAGE 1, 	bsf PCLATH,3				;Select Page 1
;				bcf PCLATH,4
;*******************************************************************************************
	ORG 0x800					;PAGE 1

menue_1
	bcf INTCON,GIE			;Interrupts verbieten
	bsf DDS_CONTROL_H,0		;setzen des Reset Bits im Control Register (DDS stoppen)
	call write_dds_control	;Aufruf des Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	bsf OUT_VT_ONOFF		;Vorteiler Eingang deaktivieren
	btfss TTL_ON_OFF,0		;prfen, ob TTL Ausgang eingeschaltet werden soll
	bcf TTL_ENABLE			;nein, TTL Ausgang abschalten
	btfsc TTL_ON_OFF,0		;prfen, ob TTL Ausgang ausgeschaltet werden soll
	bsf TTL_ENABLE			;nein, TTL Ausgang einschalten
	movlw d'4'				;laden der maximalen Anzahl Menpunkte
	movwf MENU_COUNT_MAX
	btfsc MENU_STAT,1		;prfen der Bits fr den direkten Sprung in den zuletzt genutzten Menpunkt  
	goto menu_mess_set
	btfsc MENU_STAT,2		;
	goto menu_dds_set
	btfsc MENU_STAT,3		;
	goto menu_vco_set

	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call disp_menue_1		;kein Bit gesetzt fr direkten Sprung, Aufruf des Unterprogramms zur Anzeige des Hauptmens
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
menue1_loop
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	btfss MENU_STAT,7		;prfen des Update Bits
	call disp_menue_1		;Bit ist nicht gesetzt, Update der Anzeige fr Hauptmen
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,4			;Prfen ob Pfeiltaste "down" nicht gedrckt ist (low ist)
	call menue_1dn			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;Prfen ob Pfeiltaste "up" nicht gedrckt ist (low ist)
	call menue_1up			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,1			;Prfen ob "Men" Taste gedrckt ist (high ist)
	goto menue_end
	btfss BEFEHL,2			;Prfen ob "OK" Taste gedrckt ist (high ist)
	goto menue1_loop		;wenn ja berspringe diesen Befehl

	movlw d'1'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_mess_set		; Aufruf des Programmteils menu_mess_set
	movlw d'2'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_dds_set		; Aufruf des Programmteils menu_dds_set
	movlw d'3'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_vco_set		; Aufruf des Programmteils menu_vco_set
	movlw d'4'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_einstell_set	; Aufruf des Programmteils menu_einstell_set
	bcf PCLATH,3			;Select Page 0
	goto menue_end

menu_mess_set
	btfsc MENU_STAT,1		;prfen ob Bit nicht mehr gesetzt ist
	goto $+4				;Bit ist gesetzt, weiter mit letztem Wert von TEMP_DIG_CNT
	movlw d'1'				;Bit ist nicht gesetzt, Rcksetzen des Zhlers auf 1 und Lschen des Programm Registers
	movwf TEMP_DIG_CNT
	clrf PROGRAM
	movlw d'2'				;Anzeige hat 2 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movfw PROGRAM			;sichern der aktuellen Messprogramms (F,P,Z) in Temp Register
	movwf TEMP_11
	movfw EINGANG			;sichern des aktuellen Messeingangs in Temp Register
	movwf TEMP_12
	bsf TEMP_DIG_STAT,0		;setze Statusbit
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call dig_mess_end		;
loop_mess_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_mess1			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_mess			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,5			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,6			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call cursor_mess		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, weiter bei nchster Prfung
	bsf TEMP_DIG_STAT,1		;setze Statusbit fr "up"
	call dig_mess1			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,4			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_mess1			;wenn ja berspringe diesen Befehl
	goto loop_mess_set		;zurck zur Schleife 

copy_mess
	bcf ME_DB0				;setze die Ausgnge ME_DB0,ME_DB1 fr den gewhlten Messeingang
	bcf ME_DB1
	btfsc EINGANG,0
	bsf ME_DB0
	btfsc EINGANG,1
	bsf ME_DB1

	call writeepr_mess		;Aufruf des Unterprogramms zum Schreiben von Einstellungswerten in das interne EEPROM

	clrf MENU_STAT			;Lsche Register und setze Bit fr Men "Messen"
	bsf MENU_STAT,1
	goto menue_end1			;weiter bei Standardroutine fr Beendigung des Mens in den Menpunkten 1-4
copy_mess1
	movfw TEMP_11			;rcksichern des aktuellen Programms
	movwf PROGRAM
	movfw TEMP_12			;rcksichern des aktuellen Messeingangs
	movwf EINGANG
	clrf MENU_STAT			;Lsche Register und setze Bit fr Men
	goto menue_end1			;weiter bei Standardroutine fr Beendigung des Mens in den Menpunkten 1-4


menu_dds_set
	btfsc MENU_STAT,2		;prfen ob Bit nicht mehr gesetzt ist
	goto $+3				;Bit ist gesetzt, weiter mit letztem Wert von TEMP_DIG_CNT
	movlw d'1'				;Bit ist nicht gesetzt, Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'19'				;Anzeige hat 19 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movfw TTL_ON_OFF		;sichern des aktuellen Status TTL On/Off
	movwf TEMP_10
	movfw DDS_WAVEFORM		;sichern der aktuellen Wellenform in Temp Register
	movwf TEMP_11
	movfw DDS_VOL			;sichern des aktuellen Ausgangspegels in Temp Register
	movwf TEMP_12

	movlw A'-'				;Hole "-" nach W
	btfss OFFSET_VAL,7		;prfen, ob Vorzeichen = "-" (Bit 7 = 1)
	movlw A'+'				;Vorzeichen = "+" (Bit 7 = 1)
	movwf AD_DA_REST		;Vorzeichen = "-" (Bit 7 = 0), 
	movfw OFFSET_VAL		;kopiere Offset Wert nach Temp Register
	movwf TEMP_15
	bcf TEMP_15,7			;Lsche Vorzeichen Bit
	movfw TEMP_15
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call convert_dig		;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	movwf AD_DA_L
	movfw TEMP_14
	movwf AD_DA_H
	movlw d'10'
	btfsc TEMP_13,0
	movwf AD_DA_H

	bcf PCLATH,3			;Select Page 0	
	call clear_ZAHL_C		;lsche ZAHL_C0-ZAHL_C7
	bsf PCLATH,3			;Select Page 1
	movfw DDS_PHASE_L		;Kopieren der 2 8-Bit DDS_PHASE-Register in die Rechenregister fr die Zerlegung in einzelne Digits
	movwf ZAHL_C0	
	movfw DDS_PHASE_H
	movwf ZAHL_C1
	bcf PCLATH,3			;Select Page 0	
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen Einer 32-Bit Zahl in einzelne Digits (max.999.999.999)
	bsf PCLATH,3			;Select Page 1
	movfw DISP_0			;kopieren der DISP Register nach ZAHL_A Register, diese werden zur Anzeige des Winkels verwendet: 000,0
	movwf ZAHL_A0
	movfw DISP_1
	movwf ZAHL_A1
	movfw DISP_2
	movwf ZAHL_A2
	movfw DISP_3
	movwf ZAHL_A3

	movfw DDS_FREQ_0		;Kopieren der 4 8-Bit DDS_FREQ-Register in die Rechenregister fr die Zerlegung in einzelne Digits
	movwf ZAHL_C0
	movfw DDS_FREQ_1		;
	movwf ZAHL_C1
	movfw DDS_FREQ_2		;
	movwf ZAHL_C2
	movfw DDS_FREQ_3		;
	movwf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	bcf PCLATH,3			;Select Page 0	
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen Einer 32-Bit Zahl in einzelne Digits (max.999.999.999)
	bsf PCLATH,3			;Select Page 1
	bsf TEMP_DIG_STAT,0		;setze Statusbit
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call dig_dds_end		;
loop_dds_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_dds1			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_dds			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,5			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,6			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call dig_dds_end		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, weiter bei nchster Prfung
	bsf TEMP_DIG_STAT,1		;setze Statusbit fr "up"
	call dig_dds1			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,4			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_dds1			;wenn ja berspringe diesen Befehl
	goto loop_dds_set		;zurck zur Schleife 

copy_dds					;
	movfw AD_DA_H
	movwf TEMP_13
	bcf STATUS,C
	rlf TEMP_13
	rlf TEMP_13
	rlf TEMP_13
	movfw AD_DA_H
	addwf TEMP_13
	addwf TEMP_13
	movfw AD_DA_L
	addwf TEMP_13
	movlw d'100'			;wenn Offset Wert >100%, rcksetzen auf 100
	subwf TEMP_13,w
	movlw d'100'
	btfsc STATUS,C
	movwf TEMP_13
	movlw A'+'
	subwf AD_DA_REST,w
	btfss STATUS,Z	
	bsf TEMP_13,7
	movfw TEMP_13
	movwf OFFSET_VAL

	bcf PCLATH,3			;Select Page 0		
	call convert_dig_hex	;Konvertierung der 9 Zahlen des Displays in die 32-Bit Zahl fr die Frequenz
	bsf PCLATH,3			;Select Page 1	
	movfw ZAHL_C0			;Kopieren der 32-Bit Zahl von den Rechenregistern in die Frequenzregister
	movwf DDS_FREQ_0
	movfw ZAHL_C1
	movwf DDS_FREQ_1
	movfw ZAHL_C2
	movwf DDS_FREQ_2
	movfw ZAHL_C3
	movwf DDS_FREQ_3

	bsf PCLATH,4			;select Page_3
	call convert_dds_winkel	;Unterprogramm zur Konvertierung der 4 Digits des Displays in 2 Phasenwinkel Register (hexadezimal)
	bcf PCLATH,4			;select Page_1	

	call writeepr_dds		;Aufruf des Unterprogramms zum Schreiben von Einstellungswerten in das interne EEPROM
	call wait_250_p1
	bsf PCLATH,4			;select Page_3
	call dds_settings		;Unterprogramm fr alle DDS Einstellungen
	bcf PCLATH,4			;select Page_1	
	clrf MENU_STAT			;Lsche Register und setze Bit fr DDS Generator
	bsf MENU_STAT,2
	goto menue_end1			;weiter bei Standardroutine fr Beendigung des Mens in den Menpunkten 1-4
copy_dds1
	movfw TEMP_10			;rcksichern des aktuellen Status TTL On/Off
	movwf TTL_ON_OFF	
	movfw TEMP_11			;rcksichern der aktuellen Wellenform
	movwf DDS_WAVEFORM
	movfw TEMP_12			;rcksichern des aktuellen Ausgangspegels
	movwf DDS_VOL
	clrf MENU_STAT			;Lsche Register und setze Bit fr Men
	goto menue_end1			;weiter bei Standardroutine fr Beendigung des Mens in den Menpunkten 1-4


menu_vco_set
	btfsc MENU_STAT,3		;prfen ob Bit nicht mehr gesetzt ist
	goto $+3				;Bit ist gesetzt, weiter mit letztem Wert von TEMP_DIG_CNT
	movlw d'1'				;Bit ist nicht gesetzt, Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'12'				;Anzeige hat 12 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movfw PROGRAM			;sichern des aktuellen Programms in Temp Register
	movwf TEMP_10
	movfw KENNLIN			;sichern der aktuellen Kennlinie (Lin/Log) in Temp Register
	movwf TEMP_11
	movfw DEKADE			;sichern des aktuellen Anzahl Dekaden in Temp Register
	movwf TEMP_12

	bsf STATUS,RP0			;Select BANK_1
	movfw WOB_STEP			;sichern der aktuellen Schritte/Durchlauf in Temp Register
	movwf TEMP_0
	movfw WOB_M_A			;sichern der aktuellen Einstellung (Manuell=1/Auto=0) in Temp Register
	movwf TEMP_1
	movfw WOB_ZEIT			;sichern der aktuellen Zeit/Schritt in Temp Register
	movwf TEMP_2
	bcf STATUS,RP0			;Select BANK_0

	movlw d'4'				;prfen, ob Programm 4 (Wobbel) eingestellt ist: 
	subwf PROGRAM,w			;subtrahiere vom aktuellen Programm den Wert 4.
	btfsc STATUS,Z			;prfen ob >0<
	goto $+0x06				;aktuelles Programm ist 4
	movlw d'3'				;aktuelles Programm ist >4<, lade Programm 3
	movwf PROGRAM			;
	movlw d'10'				;Anzeige hat 10 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	goto $+0x07
	movlw 0x00
	bsf STATUS,RP0			;Select BANK_1	
	btfss WOB_M_A,0			;
	movlw 0x01
	bcf STATUS,RP0			;Select BANK_0
	addwf TEMP_CNT_MAX	

	bcf PCLATH,3			;Select Page 0
	call clear_ZAHL_C		;Lsche ZAHL_C Register

	movfw FREQREG_L			;kopiere Startfrequenz (x10) L/M/H-Bytes in ZAHL_C Register fr die Anzeige auf dem Display
	movwf ZAHL_C0
	movfw FREQREG_M			;
	movwf ZAHL_C1
	movfw FREQREG_H			;
	movwf ZAHL_C2
	call convert_hex_dig	;
	clrf DISP_0				;Lsche Wert fr Nachkommastelle

	bsf TEMP_DIG_STAT,0		;setze Statusbit
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call dig_vco_end		;Unterprogramm: Clear Display, Disp VCO/WOB, Set Cursor
loop_vco_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_vco1			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_vco			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,5			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,6			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call dig_vco_end		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, weiter bei nchster Prfung
	bsf TEMP_DIG_STAT,1		;setze Statusbit fr "up"
	call dig_vco1			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,4			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_vco1			;wenn ja berspringe diesen Befehl
	goto loop_vco_set		;zurck zur Schleife 

copy_vco
	bcf PCLATH,3			;Select Page 0
	call convert_dig_hex	;Aufruf des Unterprogramms zum Umwandeln der einzelnen Digits (max.999.999.999) in eine 32-Bit Zahl
	bsf PCLATH,3			;Select Page 1

	call writeepr_vco_wob	;Aufruf des Unterprogramms zum Schreiben von Einstellungswerten in das interne EEPROM

	clrf MENU_STAT			;Lsche Register und setze Bit fr Men "VCO"
	bsf MENU_STAT,3
	goto menue_end1			;weiter bei Standardroutine fr Beendigung des Mens in den Menpunkten 1-4
copy_vco1
	movfw TEMP_10			;rcksichern des aktuellen Programms
	movwf PROGRAM
	movfw TEMP_11			;rcksichern der aktuellen Kennlinie
	movwf KENNLIN
	movfw TEMP_12			;rcksichern des aktuellen Anzahl Dekaden
	movwf DEKADE
	bsf STATUS,RP0			;Select BANK_1
	movfw TEMP_0			;rcksichern des aktuellen Schritte/Durchlauf
	movwf WOB_STEP
	movfw TEMP_1			;rcksichern der aktuellen Einstellung (Manuell/Auto)
	movwf WOB_M_A
	movfw TEMP_2			;rcksichern des aktuellen Zeit/Schritt
	movwf WOB_ZEIT
	bcf STATUS,RP0			;Select BANK_0
	bcf DDS_CONTROL_H,0		;lschen des Reset Bits im Control Register (DDS starten)
	call write_dds_control	;Aufruf des Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	clrf MENU_STAT			;Lsche Register und setze Bit fr Men
	goto menue_end1			;weiter bei Standardroutine fr Beendigung des Mens in den Menpunkten 1-4


menu_einstell_set
	movlw d'6'				;laden der maximalen Anzahl Menpunkte in den Menpunktzhler
	movwf MENU_COUNT_MAX
	movfw MENU_COUNT		;zwischenspeichern des Programmpunktes von Men1 in Temp Register
	movwf TEMP_11
	movlw d'1'				;lade Menpunktzhler mit Wert 1
	movwf MENU_COUNT
	bcf MENU_STAT,6			;lsche Status Bit
menu_einstell_loop
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	btfss MENU_STAT,6
	call disp_menu_einstell	;	
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,4			;Prfen ob Pfeiltaste "down" nicht gedrckt ist (low ist)
	call menue_1dn			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;Prfen ob Pfeiltaste "up" nicht gedrckt ist (low ist)
	call menue_1up			;wenn ja berspringe diesen Befehl
	btfss BEFEHL,1			;Prfen ob "Men" Taste gedrckt ist (high ist)
	goto $+4
	movfw TEMP_11			;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register
	movwf MENU_COUNT	
	goto menue_1
	btfss BEFEHL,2			;Prfen ob "OK" Taste gedrckt ist (high ist)
	goto menu_einstell_loop	;wenn ja berspringe diesen Befehl, wenn nein zurck zur Schleife

	movlw d'1'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_quhr_set		; Aufruf des Programmteils menu_quhr_set
	movlw d'2'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_q1hz_set		; Aufruf des Programmteils menu_q1hz_set
	movlw d'3'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_ddstaktm_set	; Aufruf des Programmteils menu_ddstaktm_set
	movlw d'4'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_vorteil_set	; Aufruf des Programmteils menu_vorteil_set
	movlw d'5'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_offset_set	; Aufruf des Programmteils menu_offset_set
	movlw d'6'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_dacgain_set	; Aufruf des Programmteils menu_dacgain_set
	goto menue_1

menu_quhr_set
	clrf INTCON				;Interrupts verbieten
	movfw TEMP_11			;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register
	movwf MENU_COUNT	

	clrf ZAHL_A0			;wird fr Sekunden verwendet
	clrf ZAHL_A1			;wird fr Minuten verwendet
	clrf ZAHL_A2			;wird fr Stunden verwendet


	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call cursor_off
	call clear_display
	call disp_uhr
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call wait_250_p1		;Wartezeit von 250ms Page1

	clrf STAT_INT
	bsf STAT_INT,0
	clrf MESS_END
	bsf INTCON,PEIE			;periphere Interrupts erlauben
	bsf STATUS,RP0			;Select BANK_1
	movlw b'00000001'		;Timer 1 Interrupt erlauben
	movwf PIE1 
	bcf STATUS,RP0			;Select BANK_0
	movlw b'00000000'		;TMR1 Off (Bit0), Prescaler 1:1
	movwf T1CON

	movlw HIGH_BYTE_S		;Zhler auf 1 sek einstellen
	movwf COUNT_H_S
	clrf TMR1L
	movlw d'247'
	movwf TMR1H	

	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_quhr			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfss BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto $-4				;wenn ja berspringe diesen Befehl

	bsf T1CON,TMR1ON		;Mit dem OK Befehl wird die Uhr gestartet, Timer 1 starten
	bcf PIR1, TMR1IF		;Timer 1 Interrupt flag lschen
	bsf INTCON,GIE			;Globale Interrupts erlauben
	call wait_250_p1		;Wartezeit von 250ms Page1
inc_uz_loop
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_quhr			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_quhr			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife

	btfss MESS_END,0
	goto inc_uz_loop	
	bcf MESS_END,0

	incf	ZAHL_A0,f		; 1 Sekunden erhhen
	movlw	D'60'
	subwf	ZAHL_A0,w
	btfss	STATUS,Z	
	goto	inc_uz_end		; 1 Sekunden <> 10	

	clrf	ZAHL_A0
	incf	ZAHL_A1,f		; 1 Minuten erhhen
	movlw	D'60'
	subwf	ZAHL_A1,w
	btfss	STATUS,Z	
	goto	inc_uz_end		;  1 Minuten <> 10

	clrf	ZAHL_A1
	incf	ZAHL_A2,f		; 1 Stunden erhhen
	movlw	D'24'
	subwf	ZAHL_A2, w
	btfss	STATUS,Z	
	goto	inc_uz_end		; 1 Stunden <> 10

	clrf	ZAHL_A2

inc_uz_end
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call disp_uhr
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	goto inc_uz_loop

copy_quhr
	goto menue_end			;weiter bei Standardroutine fr Beendigung des Mens


menu_q1hz_set
	movfw TEMP_11			;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register
	movwf MENU_COUNT	

	clrf STAT_INT
	bsf STAT_INT,3
	bcf PCLATH,3			;Select Page 0
	call clear_ZAHL_C		;lsche Register ZAHL_C0 - ZAHL_C7
	bsf PCLATH,3			;Select Page 1
	clrf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3

	clrf DISP_0
	clrf DISP_1
	clrf DISP_2
	clrf DISP_3

	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call clear_display		;lsche Display
	call disp_q1hz			;sende an Display -----ppm-----s
	movlw 0x81				;Displayadresse
	call disp_q1hz_dat		;sende an Display -0000, Abweichung in ppm
	movlw 0x89				;Displayadresse
	call disp_q1hz_dat		;sende an Display ---------0000, abgelaufene Sekunden
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4

	clrf MESS_END
	clrf INTCON
	bsf INTCON,PEIE			;periphere Interrupts erlauben
	bsf STATUS,RP0			;Select BANK_1
	clrf PIE1
	bsf PIE1,TMR1IE			;Timer 1 Interrupt erlauben
	bcf STATUS,RP0			;Select BANK_0
	clrf T1CON				;Timer 1 stoppen

	movlw HIGH_BYTE_S		;Zhler auf 1 sek einstellen
	movwf COUNT_H_S
	bsf STATUS,RP0			;Select BANK_1
	movwf WOB_ZEIT_H
	bcf STATUS,RP0			;Select BANK_0
	movlw LOW_BYTE_S
	movwf TMR1H	
	bsf STATUS,RP0			;Select BANK_1
	movwf WOB_ZEIT_LN
	movwf WOB_ZEIT_LN1
	movlw 0xFF
	movwf WOB_LOOP
	bcf STATUS,RP0			;Select BANK_0
	movwf COUNT_ADJ
	clrf TMR1L

	movlw b'00000101'		;Bit 3-0: Capture mode: every rising edge
	movwf CCP1CON

	bcf PIR1, TMR1IF		;Timer 1 Interrupt flag lschen
	bcf PIR1,CCP1IF			;Capture Interrupt Flag lschen
							;Warten auf die positive Flanke von RB0
	btfss IN_MENUE			;Abfrage des Tasters zum Wechsel des Programms (Abbruch)
	goto menue_end
	btfss PIR1,CCP1IF		;Abfrage der positiven Flanke von RC2
	goto $-3				;noch keine positive Flanke, zurck zur Warteschleife
	bsf T1CON,TMR1ON		;positive Flanke liegt an, Beginn der Messung, Timer 1 starten
	bcf PIR1,CCP1IF			;lsche CCP1 Flag
	bsf INTCON,GIE			;Globale Interrups erlauben	

q1hz_loop
	btfss IN_MENUE			;Abfrage des Men Tasters
	goto menue_end
	btfss MESS_END,7		;Prft ob eine Sekunde abgelaufen ist 
	goto q1hz_loop1
	bcf MESS_END,7
	goto int_first
q1hz_loop1
	btfss PIR1,CCP1IF		;Prft die positive Flanke von RB0 
	goto q1hz_loop
	bcf PIR1,CCP1IF
	goto ext_first

int_first
	btfss IN_MENUE			;Abfrage des Men Tasters
	goto menue_end
	btfsc PIR1,CCP1IF		;Prft die positive Flanke von RC2 
	goto int_first1
	movlw d'48'				; => 0,213sek, Abbruch wenn nach 0,213sek (z.B. DCF 59ste Sek) keine positive Flanke
	subwf COUNT_H_S,w		;
	btfss STATUS,Z
	goto int_first
int_first1
	bcf PIR1,CCP1IF			;lsche CCP1 Flag
	movfw COUNT_H_S			;sichern der Timer 1 Werte im ZAHL_C Register
	movwf ZAHL_C2
	movfw CCPR1L
	movwf ZAHL_C0
	movfw CCPR1H
	movwf ZAHL_C1

	movlw LOW_BYTE_S		;zahl_c1 - low_byte_s
	subwf ZAHL_C1,f
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto $+2
	incf ZAHL_C2
	movfw ZAHL_C2			;high_byte_s - zahl_c2
	sublw HIGH_BYTE_S
	movwf ZAHL_C2

	movlw A'+'				;Vorzeichen fr Abweichung
	goto auswert_q1hz

ext_first
	movfw COUNT_H_S			;sichern der Timer 1 Werte im ZAHL_C Register
	movwf ZAHL_C2
	movfw CCPR1L
	movwf ZAHL_C0
	movfw CCPR1H
	movwf ZAHL_C1
ext_first1
	btfss IN_MENUE			;Abfrage des Men Tasters
	goto menue_end
	btfss MESS_END,7		;Prft ob eine Sekunde abgelaufen ist 
	goto ext_first1
	bcf MESS_END,7			;Lsche Sekunden berlauf bit

	movfw ZAHL_C0			;0 - zahl_c0
	sublw d'0'
	movwf ZAHL_C0
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto $+4				;Ergebnis positiv, weiter
	incfsz ZAHL_C1			;Ergebnis negativ
	goto $+2	
	decf ZAHL_C2

	movfw ZAHL_C1			;0 - zahl_c1
	sublw d'0'
	movwf ZAHL_C1
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto $+2				;Ergebnis positiv, weiter
	decf ZAHL_C2

	movlw A'-'				;Vorzeichen fr Abweichung	

auswert_q1hz
	movwf TEMP_10			;Vorzeichen fr Abweichung
	incfsz ZAHL_A0
	goto $+2
	incf ZAHL_A1
	movfw ZAHL_A0			;sichern der Register in Temp register
	movwf TEMP_11
	movfw ZAHL_A1			;sichern der Register in Temp register
	movwf TEMP_12
	bcf PCLATH,3			;Select Page 0
	call div32
	bsf PCLATH,3			;Select Page 1
	bcf STATUS,C
	rrf ZAHL_C1
	rrf ZAHL_C0
	bcf STATUS,C
	rrf ZAHL_C1
	rrf ZAHL_C0
	bcf PCLATH,3			;Select Page 0	
	call convert_hex_dig	;Zerlegen einer 32 Bit Zahl (ZAHL_C) in die einzelnen Digits (max 999.999.999)
	bsf PCLATH,4			;Select Page 2
	call disp_q1hz_vz		;sende an Display Vorzeichen der Abweichung +/-
	movlw 0x81				;Displayadresse
	call disp_q1hz_dat		;sende an Display -0000, Abweichung in ppm

	bcf PCLATH,4			;Select Page 0
	movfw TEMP_11			;kopiere die abgelaufenen Sekunden nach ZAHL_C
	movwf ZAHL_C0
	movfw TEMP_12
	movwf ZAHL_C1
	clrf ZAHL_C2
	clrf ZAHL_C3
	call convert_hex_dig	;Zerlegen einer 32 Bit Zahl (ZAHL_C) in die einzelnen Digits (max 999.999.999)
	movlw 0x89				;Displayadresse
	bsf PCLATH,4			;Select Page 2
	call disp_q1hz_dat		;sende an Display ---------0000, abgelaufene Sekunden
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4			;
	movfw TEMP_11			;Rcksichern der Temp Register
	movwf ZAHL_A0
	movfw TEMP_12			;Rcksichern der Temp Register
	movwf ZAHL_A1
	bcf PCLATH,3			;Select Page 0
	call clear_ZAHL_C		;lsche Register
	bsf PCLATH,3			;Select Page 1
	goto q1hz_loop			;Zurck zur Schleife


menu_ddstaktm_set			;Menpunkt zur Anpassung des DDS Taktes MCLK
	movfw TEMP_11			;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register
	movwf MENU_COUNT

	movfw DDS_MCLK_0		;Zerlegen des 32-Bit Wertes fr MCLK in die 8 Zahlen (1er,10er,...,Millionen10er)
	movwf ZAHL_C0			;Kopieren der 4 8-Bit MCLK-Register in die Zhler Register fr die Division
	movfw DDS_MCLK_1		;
	movwf ZAHL_C1
	movfw DDS_MCLK_2		;
	movwf ZAHL_C2
	movfw DDS_MCLK_3		;
	movwf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	bcf PCLATH,3			;Select Page 0
	call convert_hex_dig	;Aufruf des Unterprogramms zum Zerlegen Einer 32-Bit Zahl in einzelne Digits (max.999.999.999)
	bsf PCLATH,3			;Select Page 1
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'8'				;Anzeige hat 8 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	bsf TEMP_DIG_STAT,0		;setze Statusbit
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call dig_ddstaktm_end	;
loop_ddstaktm_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_ddstaktm1		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_ddstaktm		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,5			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,6			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call cursor_ddstaktm	;wenn ja berspringe diesen Befehl
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, weiter bei nchster Prfung
	bsf TEMP_DIG_STAT,1		;setze Statusbit fr "up"
	call dig_ddstaktm1		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,4			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_ddstaktm1		;wenn ja berspringe diesen Befehl
	goto loop_ddstaktm_set	;zurck zur Schleife 

copy_ddstaktm
	bcf PCLATH,3			;Select Page 0
	call convert_dig_hex	;Aufruf des Unterprogramms zum Konvertieren der einzelne Digits (max.999.999.999)in eine 32-Bit Zahl
	bsf PCLATH,3			;Select Page 1
	movfw ZAHL_C0			;hole das Low Byte der 32-Bit Zahl ins W-Reg und kopiere es in das DDS_MCLK_0 Register
	movwf DDS_MCLK_0
	movwf TEMP_13			;  und in das Schreibregister fr das EEPROM
	movlw 0x00				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	movfw ZAHL_C1			;hole das Low Byte +1 der 32-Bit Zahl ins W-Reg und kopiere es in das DDS_MCLK_1 Register
	movwf DDS_MCLK_1
	movwf TEMP_13			;  und in das Schreibregister fr das EEPROM
	movlw 0x01				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	movfw ZAHL_C2			;hole das Low Byte +2 der 32-Bit Zahl ins W-Reg und kopiere es in das DDS_MCLK_2 Register
	movwf DDS_MCLK_2
	movwf TEMP_13			;  und in das Schreibregister fr das EEPROM
	movlw 0x02				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	movfw ZAHL_C3			;hole das High Byte der 32-Bit Zahl ins W-Reg und kopiere es in das DDS_MCLK_3 Register
	movwf DDS_MCLK_3
	movwf TEMP_13			;  und in das Schreibregister fr das EEPROM
	movlw 0x03				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
copy_ddstaktm1
	goto menue_end			;weiter bei Standardroutine fr Beendigung des Mens


menu_vorteil_set
	movfw TEMP_11			;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register
	movwf MENU_COUNT	
	movfw MUL_VT			;kopieren des aktuellen Vorteilerfaktors in Temp Register
	movwf TEMP_11
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_set_vort_t	;Aufruf des Unterprogramms zur Anzeige des Textes
	call disp_set_vort_w	;Aufruf des Unterprogramms zur Anzeige des Wertes
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call wait_250_p1		;Wartezeit von 250ms Page1
loop_vorteil_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_vorteil1		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_vorteil		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "up" gesetzt ist
	goto $+7				;Bit ist nicht gesetzt, weiter bei nchster Prfung
	incf TEMP_11			;Erhhe Vorteiler Wert um 1
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call disp_set_vort_w	;Aufruf des Unterprogramms zur Anzeige des Wertes
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	btfss BEFEHL,4			;prfen ob Bit fr Befehl "down"  gesetzt ist
	goto loop_vorteil_set	;zurck zur Schleife 
	decf TEMP_11			;Reduziere Vorteiler Wert um 1
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call disp_set_vort_w	;Aufruf des Unterprogramms zur Anzeige des Wertes
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	goto loop_vorteil_set	;zurck zur Schleife 

copy_vorteil
	movfw TEMP_11			;Kopiere den genderten Wert ins Vorteiler Register
	movwf MUL_VT
	movwf TEMP_13			;  und in das Schreibregister fr das EEPROM
	movlw 0x04				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
copy_vorteil1
	goto menue_end			;weiter bei Standardroutine fr Beendigung des Mens


menu_offset_set				;Menpunkt zur Kompensation des Offset fr Sinus/Dreieck und Rechteck des DDS Generators
	movfw TEMP_11			;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register
	movwf MENU_COUNT
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'2'				;Anzeige hat 2 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;

	call read_offset		;Unterprogramm zum Lesen der Offsetwerte aus dem internen EEPROM

	movfw OFFSET_NULL_L		;sichern der Offset Null Wert (L/H Byte) in Temp Register
	movwf TEMP_14
	movfw OFFSET_NULL_H
	movwf TEMP_15	
	movfw OFFSET_100_L		;sichern der Offset -100% Wert (L/H Byte) in Temp Register
	movwf TEMP_10
	movfw OFFSET_100_H
	movwf TEMP_12

	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4			;
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call cursor_on			;Cursor Einschalten
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call dig_offset_end		;Unterprogramm zum setzen des DAC Ausgangs, Schreiben des 
							;  Menpunktes, und setzen des Cursor
loop_offset_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_offset1		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_offset		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,5			;prfen ob Bit fr Befehl "right" nicht gesetzt ist
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,6			;prfen ob Bit fr Befehl "left" nicht gesetzt ist
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call cursor_offset		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,3			;prfen ob Bit fr Befehl "up" nicht gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, weiter bei nchster Prfung
	bsf TEMP_DIG_STAT,1		;setze Statusbit fr "up"
	call dig_offset1		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,4			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_offset1		;wenn ja berspringe diesen Befehl
	goto loop_offset_set	;zurck zur Schleife 

copy_offset
	movfw OFFSET_NULL_L		;kopiere das Low Byte des Offset Wertes in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x07				;Lade die Adresse des EEPROM Fr Offset Reckteck, Low Byte 
	btfss DDS_WAVEFORM,1	;prfen ob Wellenform = Rechteck (= 2 => 00000010)
	movlw 0x05				;Wellenform ist nicht Rechteck, Lade die Adresse des EEPROM fr Offset Sinus/Dreieck, Low Byte
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	movfw OFFSET_NULL_H		;kopiere das High Byte des Offset Wertes in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x08				;Lade die Adresse des EEPROM Fr Offset Reckteck, High Byte 
	btfss DDS_WAVEFORM,1	;prfen ob Wellenform = Rechteck (= 2 => 00000010)
	movlw 0x06				;Wellenform ist nicht Rechteck, Lade die Adresse des EEPROM fr Offset Sinus/Dreieck, High Byte
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw OFFSET_100_L		;kopiere das Low Byte des Offset Wertes in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x1F				;Lade die Adresse des EEPROM Fr Offset Reckteck, Low Byte 
	btfss DDS_WAVEFORM,1	;prfen ob Wellenform = Rechteck (= 2 => 00000010)
	movlw 0x1D				;Wellenform ist nicht Rechteck, Lade die Adresse des EEPROM fr Offset Sinus/Dreieck, Low Byte
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	movfw OFFSET_100_H		;kopiere das High Byte des Offset Wertes in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x20				;Lade die Adresse des EEPROM Fr Offset Reckteck, High Byte 
	btfss DDS_WAVEFORM,1	;prfen ob Wellenform = Rechteck (= 2 => 00000010)
	movlw 0x1E				;Wellenform ist nicht Rechteck, Lade die Adresse des EEPROM fr Offset Sinus/Dreieck, High Byte
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	call wait_250_p1		;Wartezeit von 250ms Page1
	bsf PCLATH,4			;Select Page 3
	call offset_korr		;Setzen der Offsetkorrektur ber DAC
	bcf PCLATH,4			;Select Page 1
copy_offset1
	movfw TEMP_14		;rcksichern der Offset Null Wert (L/H Byte)
	movwf OFFSET_NULL_L
	movfw TEMP_15
	movwf OFFSET_NULL_H	
	movfw TEMP_10		;sichern der Offset -100% Wert (L/H Byte) in Temp Register
	movwf OFFSET_100_L
	movfw TEMP_12
	movwf OFFSET_100_H
	goto menue_end			;weiter bei Standardroutine fr Beendigung des Mens

menu_dacgain_set
	movfw TEMP_11			;rcksichern des Programmpunktes von Men1 von Temp Register in MENU_COUNT Register
	movwf MENU_COUNT	
	bcf INTCON,GIE			;Interrupts verbieten
	movfw DAC_GAIN			;Sichern der aktuellen DAC Verstrkung in Temp Register
	movwf TEMP_11
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_set_dacgain	;Aufruf des Unterprogramms zur Anzeige auf dem Display
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call wait_250_p1		;Wartezeit von 250ms Page1
loop_dacgain_set
	call menu_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto copy_dacgain1		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,2			;prfen ob Bit fr Befehl OK nicht gesetzt ist
	goto copy_dacgain		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;prfen ob Bit fr Befehl "up" nicht gesetzt ist
	call dig_dacgain		;Bit ist gesetzt, Aufruf des Unterprogramms
	btfsc BEFEHL,4			;prfen ob Bit fr Befehl "down" nicht gesetzt ist
	call dig_dacgain		;Bit ist gesetzt, Aufruf des Unterprogramms
	goto loop_dacgain_set	;zurck zur Schleife 

copy_dacgain
	movfw DAC_GAIN			;Kopiere den genderten Wert ins Schreibregister fr das EEPROM
	movwf TEMP_13			; 
	movlw 0x09				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	goto menue_end			;weiter bei Standardroutine fr Beendigung des Mens
copy_dacgain1
	movfw TEMP_11			;Rcksichern der aktuellen DAC Verstrkung
	movwf DAC_GAIN
	goto menue_end			;weiter bei Standardroutine fr Beendigung des Mens


menue_end					;Standardroutine fr Beendigung des Mens
	clrf MENU_STAT			;Lsche Register
menue_end1					;Standardroutine fr Beendigung des Mens in den Menpunkten 1-4
	call wait_250_p1		;Wartezeit von 250ms Page1
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call cursor_off
	call clear_display
	bcf PCLATH,4			;Select Page 0
	goto main

menu_befehl						;Unterprogramm zum Laden und invertieren der Tastaturbefehle
	movlw 0xFF					;lade Register Befehl mit Wert 255
	movwf BEFEHL
	movfw PORTB					;hole PORTB ins W-Register und subtrahiere es von 255 im Register Befehl
	subwf BEFEHL				;255-PORTB=BEFEHL
	return

menue_1up
	incf MENU_COUNT			;erhhe Menpunkt Zhler um 1
	movfw MENU_COUNT_MAX	;hole Maximalwert der Menpunkte ins W-Register und erhhe um 1 
	incf MENU_COUNT_MAX,w
	subwf MENU_COUNT,w		;subtrahiere aktuellen Menpunkt nach Erhhung vom Maximalwert und prfe ob 0
	btfss STATUS, Z		
	goto $+3				;Ergebnis ist nicht 0
	movlw d'1'				;Ergebnis ist 0, Rcksetzen des Zhlers mit Wert 1
	movwf MENU_COUNT
	call wait_250_p1		;Wartezeit von 250ms Page1
	bcf MENU_STAT,6			;lsche das Statusbit
	bcf MENU_STAT,7			;lsche das Statusbit
	return

menue_1dn
	decfsz MENU_COUNT		;reduziere Menpunkt Zhler um 1 und prfe ob 0
	goto $+3				;Zhler ist nicht 0
	movfw MENU_COUNT_MAX	;Zhler ist 0, Rcksetzen des Zhlers mit Maximalwert
	movwf MENU_COUNT
	call wait_250_p1		;Wartezeit von 250ms Page1
	bcf MENU_STAT,6			;lsche das Statusbit
	bcf MENU_STAT,7			;lsche das Statusbit
	return


inc_dig_temp_cnt				;Unterprogramm zur Erhhung des Menpunktes horizontal
	movfw TEMP_CNT_MAX			;
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z				;prfen ob Zhler gleich dem Maximalwert ist
	goto $+4					;wenn ja, berspringe diesen Befehl	
	movlw d'1'
	movwf TEMP_DIG_CNT
	goto inc_temp_cnt_end
	incf TEMP_DIG_CNT
inc_temp_cnt_end
	bsf TEMP_DIG_STAT,0			;setze das Status bit
	return		

dec_dig_temp_cnt				;Unterprogramm zur Reduzierung des Menpunktes horizontal
	decfsz TEMP_DIG_CNT			;reduziere Zhler um 1 und prfe ob der der Zhler 0 ist
	goto dec_temp_cnt_end		;wenn ja, berspringe diesen Befehl	
	movfw TEMP_CNT_MAX			;
	movwf TEMP_DIG_CNT
dec_temp_cnt_end
	bsf TEMP_DIG_STAT,0			;setze das Status bit
	return

wait_250_p1					;Wartezeit von 250ms Page1
	movlw d'250'
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call wait 
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	return

;*******************************************************************************************
; Weitere Unterprogramme zur Mensteuerung
;*******************************************************************************************

dig_mess1					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt Messen
							;1:Programm
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_mess2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_mess1			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf PROGRAM
	movlw d'3'
	subwf PROGRAM,w
	btfsc STATUS,Z		
	clrf PROGRAM
	goto dig_mess_end		;Springe zum Ende des Unterprogramms
dec_mess1					;1:Programm
	movfw PROGRAM			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf PROGRAM			;reduziere Zhler um 1
	goto dig_mess_end		;Springe zum Ende des Unterprogramms
	movlw d'2'
	movwf PROGRAM
	goto dig_mess_end		;Springe zum Ende des Unterprogramms
dig_mess2					;2:Messeingang
	movlw d'2'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_mess_end			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_mess2			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf EINGANG
	movlw d'4'
	subwf EINGANG,w
	btfsc STATUS,Z		
	clrf EINGANG
	goto dig_mess_end		;springe zum Ende des Unterprogramms
dec_mess2					;2:Messeingang
	movfw EINGANG			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf EINGANG			;reduziere Zhler um 1
	goto dig_mess_end		;springe zum Ende des Unterprogramms
	movlw d'3'
	movwf EINGANG
	goto dig_mess_end		;springe zum Ende des Unterprogramms
dig_mess_end
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call disp_set_mess		;Unterprogramm fr Anzeige Menpunkt 1.Messen
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
cursor_mess
	movlw 0xC9				;lade Adresse ins W-Register
	btfss TEMP_DIG_CNT,1
	movlw 0x86				;lade Adresse ins W-Register
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call wait_250_p1		;Wartezeit von 250ms Page1
	clrf TEMP_DIG_STAT		;lsche Statusbit Register
	return


dig_dds1					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt dds
							;1:Wellenform
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_dds2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_dds1			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DDS_WAVEFORM
	movlw d'3'
	subwf DDS_WAVEFORM,w
	btfsc STATUS,Z		
	clrf DDS_WAVEFORM
	goto dig_dds_end
dec_dds1					;1:Wellenform
	movfw DDS_WAVEFORM		;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf DDS_WAVEFORM		;reduziere Zhler um 1
	goto dig_dds_end		;springe zum Ende des Unterprogramms
	movlw d'2'
	movwf DDS_WAVEFORM
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds2					;2:Ausgangspegel
	movlw d'2'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_dds3			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfsc TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes NICHT gesetzt ist
	goto dec_dds2			;Bit ist gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DDS_VOL
	movlw d'64'
	subwf DDS_VOL,w
	btfsc STATUS,Z		
	clrf DDS_VOL
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dec_dds2					;2:Ausgangspegel
	movfw DDS_VOL			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf DDS_VOL			;reduziere Zhler um 1
	goto dig_dds_end		;springe zum Ende des Unterprogramms
	movlw d'63'
	movwf DDS_VOL
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds3					;3:Winkel 100er
	movlw d'3'				;prfen ob der 3. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_dds4			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_dds3			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf ZAHL_A3
	movlw d'4'
	subwf ZAHL_A3,w
	btfsc STATUS,Z		
	clrf ZAHL_A3
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dec_dds3					;3:Winkel 100er
	movfw ZAHL_A3			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf ZAHL_A3			;reduziere Zhler um 1
	goto dig_dds_end		;springe zum Ende des Unterprogramms
	movlw d'3'
	movwf ZAHL_A3
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds4					;4:Winkel 10er
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds5
	movfw ZAHL_A2			;Wert fr Winkel 10er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf ZAHL_A2
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds5					;5:Winkel 1er
	movlw d'5'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds6
	movfw ZAHL_A1			;Wert fr Winkel 1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf ZAHL_A1
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds6					;6:Winkel 0,1er
	movlw d'6'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds7
	movfw ZAHL_A0			;Wert fr Winkel 0,1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf ZAHL_A0
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds7					;7:Frequenz Millionen 10er
	movlw d'7'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds8
	movfw DISP_8			;Wert fr Millionen 10er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_8
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds8					;8:Frequenz Millionen 1er
	movlw d'8'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds9
	movfw DISP_7			;Wert fr Millionen 1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_7
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds9					;9:Frequenz 100.000er
	movlw d'9'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds10
	movfw DISP_6			;Wert fr Millionen 1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_6
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds10					;10:Frequenz 10.000er
	movlw d'10'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds11
	movfw DISP_5			;Wert fr 10.000er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_5
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds11					;11:Frequenz 1.000er
	movlw d'11'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds12
	movfw DISP_4			;Wert fr 1.000er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_4
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds12					;12:Frequenz 100er
	movlw d'12'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds13
	movfw DISP_3			;Wert fr 100er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_3
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds13					;13:Frequenz 10er
	movlw d'13'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds14
	movfw DISP_2			;Wert fr 10er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_2
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds14					;14:Frequenz 1er
	movlw d'14'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds15
	movfw DISP_1			;Wert fr 1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_1
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds15					;15:Frequenz 0,1er
	movlw d'15'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds16
	movfw DISP_0			;Wert fr 0,1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_0
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds16					;16:Offset Vorzeichen +/-
	movlw d'16'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds17
	movlw A'+'
	subwf AD_DA_REST,w
	btfss STATUS,Z
	goto $+4
	movlw A'-'
	movwf AD_DA_REST
	goto dig_dds_end		;springe zum Ende des Unterprogramms	
	movlw A'+'
	movwf AD_DA_REST
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds17					;17:Offset 10er
	movlw d'17'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds18
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_dds17			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf AD_DA_H
	movlw d'10'
	subwf AD_DA_H,w
	btfsc STATUS,Z		
	clrf AD_DA_L
	movlw d'11'
	subwf AD_DA_H,w
	btfsc STATUS,Z		
	clrf AD_DA_H
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dec_dds17					;17:Offset 10er
	movfw AD_DA_H			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf AD_DA_H			;reduziere Zhler um 1
	goto dig_dds_end		;springe zum Ende des Unterprogramms
	movlw d'10'
	movwf AD_DA_H
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds18					;18:Offset 1er
	movlw d'18'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds19
	movfw AD_DA_L
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf AD_DA_L
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds19					;19:TTL Ausgang On/Off
	movlw d'19'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_dds_end
	btfss TTL_ON_OFF,0		;prfen ob TTL Ausgang auf EIN steht
	goto $+3
	clrf TTL_ON_OFF			;TTL Ausgang steht auf EIN => ausschalten
	goto dig_dds_end		;springe zum Ende des Unterprogramms
	movlw 0x01
	movwf TTL_ON_OFF		;TTL Ausgang steht auf AUS => einschalten
	goto dig_dds_end		;springe zum Ende des Unterprogramms
dig_dds_end
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call clear_display
	call disp_set_dds		;Unterprogramm fr Anzeige Menpunkt 2.DDS
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
cursor_dds
	movlw 0x1E				;vorbereiten fuer Log-Tabelle im Bereich 0x1D00..0x1DFF
	movwf PCLATH			;
	movfw TEMP_DIG_CNT
	addlw 0xFF
	bsf PCLATH,4			;Select Page 3
	call cursor_tab
	bcf PCLATH,3			;Select Page 2
	movwf LOOP_1			;Cursorposition in Temp Register zwischenspeichern
	movlw d'16'
	subwf TEMP_DIG_CNT,w
	movfw LOOP_1
	btfsc STATUS,C
	addlw 0xC0
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call wait_250_p1		;Wartezeit von 250ms Page1
	clrf TEMP_DIG_STAT		;lsche Statusbit Register
	return

		;V	C	O		K	L	:	L	o	g		D	e	k	:	3
		;F	u	:		1	.	0	0	0	.	0	0	0		H	z
		;S	:	0	2	5	6		A	B		t	:	0	,	2	s



dig_vco1					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt vco
							;1:VCO/Wobbel
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_vco2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw d'3'
	btfss PROGRAM,2			;Prfen, ob aktuelles Programm =4 (Wobbel, 0100)
	movlw d'4'				;aktuelles Program =3 (VCO, 0011), ndern auf 4
	movwf PROGRAM			;aktuelles Programm =4 (Wobbel, 0100), ndern auf 3
	movlw d'12'				;setzen der maximalen Stellen: VCO = 10, WOB = 12/13
	btfss PROGRAM,2			;Prfen, ob aktuelles Programm =4 (Wobbel, 0100)
	movlw d'10'				;aktuelles Program =3 (VCO, 0011), 10 Stellen max
	movwf TEMP_CNT_MAX		;aktuelles Programm =4 (Wobbel, 0100), 12/13 Stellen max

	btfss PROGRAM,2			;Prfen, ob aktuelles Programm =4 (Wobbel, 0100)
	goto dig_vco_end		;aktuelles Programm =3, springe zum Ende des Unterprogramms	
	movlw 0x00				;wenn Programm = 4 und Auto: 13 Stellen max
	bsf STATUS,RP0			;Select BANK_1	
	btfss WOB_M_A,0			;Prfen, ob Manuell gewhlt ist 
	movlw 0x01				;Automatisch ist gewhlt, 
	bcf STATUS,RP0			;Select BANK_0
	addwf TEMP_CNT_MAX		;addiere 0 oder 1 zu temp_cnt_max
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco2					;2:Kennlinie
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_vco3			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_vco2			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf KENNLIN
	movlw d'2'
	subwf KENNLIN,w
	btfsc STATUS,Z		
	clrf KENNLIN
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dec_vco2					;2:Kennlinie
	movfw KENNLIN			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf KENNLIN			;reduziere Zhler um 1
	goto dig_vco_end		;springe zum Ende des Unterprogramms
	movlw d'1'
	movwf KENNLIN
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco3					;3:Anzahl Dekaden
	movlw d'3'				;prfen ob der 3. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_vco4			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_vco3			;Bit ist gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DEKADE
	movlw d'4'
	subwf DEKADE,w
	btfss STATUS,Z		
	goto dig_vco_end		;springe zum Ende des Unterprogramms
	movlw d'1'
	movwf DEKADE
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dec_vco3					;3:Anzahl Dekaden
	decfsz DEKADE			;reduziere Zhler um 1
	goto $+3
	movlw d'3'
	movwf DEKADE
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco4					;4:Frequenz Millionen 1er
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco5
	movfw DISP_7			;Wert fr Millionen 1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_7
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco5					;5:Frequenz 100.000er
	movlw d'5'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco6
	movfw DISP_6			;Wert fr 100.000er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_6
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco6					;6:Frequenz 10.000er
	movlw d'6'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco7
	movfw DISP_5			;Wert fr 10.000er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_5
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco7					;7:Frequenz 1.000er
	movlw d'7'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco8
	movfw DISP_4			;Wert fr 1.000er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_4
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco8					;8:Frequenz 100er
	movlw d'8'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco9
	movfw DISP_3			;Wert fr 100er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_3
	goto dig_vco_end		;springe zum Ende des Unterprogramms
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco9					;9:Frequenz 10er
	movlw d'9'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco10
	movfw DISP_2			;Wert fr 10er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_2
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco10					;10:Frequenz 1er
	movlw d'10'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco11
	movfw DISP_1			;Wert fr 1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_1
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco11					;11:Schritte/Durchlauf
	movlw d'11'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco12
	btfsc TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes NICHT (btfsc) gesetzt ist
	goto dec_vco11			;Bit ist gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	bsf STATUS,RP0			;Select BANK_1
	incf WOB_STEP			;Erhhe Register um 1 (Register hat Werte von 1 bis 6)
	movlw d'7'				;wenn nach Erhhung Registerwert = 7: berlauf, rcksetzen des Registers mit Wert 1 
	subwf WOB_STEP,w
	btfss STATUS,Z		
	goto dig_vco_end		;Wert <7, springe zum Ende des Unterprogramms
	movlw d'1'				;Wert =7, Rcksetzen des Registers mit Wert 1 
	movwf WOB_STEP
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dec_vco11					;11:Schritte/Durchlauf
	bsf STATUS,RP0			;Select BANK_1
	decfsz WOB_STEP			;reduziere Register um 1 (Register hat Werte von 1 bis 6) und prfe ob 0
	goto dig_vco_end		;Wert >0, springe zum Ende des Unterprogramms
	movlw d'6'				;Wert =0, Rcksetzen des Registers mit Wert 6 
	movwf WOB_STEP
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco12					;12:Manuell/Auto + Kontinuierlich/Burst
	movlw d'12'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco13
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_vco12			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	bsf STATUS,RP0			;Select BANK_1
	incf WOB_M_A			;erhhe Wert (Register hat Werte von 0 bis 3)
	movlw d'4'				;wenn nach Erhhung Registerwert = 4: berlauf, rcksetzen des Registers mit Wert 0 
	subwf WOB_M_A,w
	btfss STATUS,Z		
	goto vco_set_max		;Wert <4
	clrf WOB_M_A			;Wert =4, Rcksetzen des Registers mit Wert 0 
	goto vco_set_max
dec_vco12					;12:Manuell/Auto + Kontinuierlich/Burst
	bsf STATUS,RP0			;Select BANK_1
	movfw WOB_M_A			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;Wert ist 0, Rcksetzen des Registers mit Wert 3
	decf WOB_M_A			;Wert <0>, reduziere Zhler um 1
	goto vco_set_max		;
	movlw d'3'				;Wert ist 0, Rcksetzen des Registers mit Wert 3
	movwf WOB_M_A
vco_set_max
	movlw d'12'
	btfss WOB_M_A,0
	movlw d'13'
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_CNT_MAX	
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dig_vco13					;13:Zeit/Schritt
	movlw d'13'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_vco_end
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_vco13			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	bsf STATUS,RP0			;Select BANK_1
	incf WOB_ZEIT			;erhhe Wert (Register hat Werte von 0 bis 9)
	movlw d'11'				;wenn nach Erhhung Registerwert = 11: berlauf, rcksetzen des Registers mit Wert 0 
	subwf WOB_ZEIT,w
	btfss STATUS,Z		
	goto dig_vco_end		;Wert <11, springe zum Ende des Unterprogramms
	clrf WOB_ZEIT			;Wert =11, Rcksetzen des Registers mit Wert 0 
	goto dig_vco_end		;springe zum Ende des Unterprogramms
dec_vco13					;13:Zeit/Schritt
	bsf STATUS,RP0			;Select BANK_1
	movfw WOB_ZEIT			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;Wert ist 0, Rcksetzen des Registers mit Wert 10
	decf WOB_ZEIT			;Wert <0>, reduziere Zhler um 1
	goto dig_vco_end		;springe zum Ende des Unterprogramms
	movlw d'10'				;Wert ist 0, Rcksetzen des Registers mit Wert 10
	movwf WOB_ZEIT
dig_vco_end
	bcf STATUS,RP0			;Select BANK_0
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_set_vco		;Unterprogramm fr Anzeige Menpunkt 3.VCO/Wobbel
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
cursor_vco
	movlw 0x1E				;vorbereiten fuer Log-Tabelle im Bereich 0x1D00..0x1DFF
	movwf PCLATH			;
	movfw TEMP_DIG_CNT
	addlw 0x12
	bsf PCLATH,4			;Select Page 3
	call cursor_tab
	bcf PCLATH,3			;Select Page 2

	movwf LOOP_1			;Cursorposition in Temp Register zwischenspeichern
	movlw d'11'
	subwf TEMP_DIG_CNT,w
	movfw LOOP_1
	btfsc STATUS,C
	addlw 0xC0
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call wait_250_p1		;Wartezeit von 250ms Page1
	clrf TEMP_DIG_STAT		;lsche Statusbit Register
	return


dig_ddstaktm1				;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt DDS Takt
	movlw d'1'				;Millionen 10er
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_ddstaktm2
	movfw DISP_7			;Wert fr Millionen 10er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_7
	goto dig_ddstaktm_end
dig_ddstaktm2				;Millionen 1er
	movlw d'2'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_ddstaktm3
	movfw DISP_6			;Wert fr Millionen 1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_6
	goto dig_ddstaktm_end
dig_ddstaktm3				;100.000er
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_ddstaktm4
	movfw DISP_5			;Wert fr 100.000er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_5
	goto dig_ddstaktm_end
dig_ddstaktm4				;10.000er
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_ddstaktm5
	movfw DISP_4			;Wert fr 10.000er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_4
	goto dig_ddstaktm_end
dig_ddstaktm5				;1.000er
	movlw d'5'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_ddstaktm6
	movfw DISP_3			;Wert fr 1.000er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_3
	goto dig_ddstaktm_end
dig_ddstaktm6				;100er
	movlw d'6'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_ddstaktm7
	movfw DISP_2			;Wert fr 100er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_2
	goto dig_ddstaktm_end
dig_ddstaktm7				;10er
	movlw d'7'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_ddstaktm8
	movfw DISP_1			;Wert fr 10er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_1
	goto dig_ddstaktm_end
dig_ddstaktm8				;1er
	movlw d'8'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_ddstaktm_end
	movfw DISP_0			;Wert fr 1er, untere 4 Bit
	movwf TEMP_13
	call inc_dec_dds
	movfw TEMP_13
	movwf DISP_0
	goto dig_ddstaktm_end
dig_ddstaktm_end
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_set_ddstaktm	;Unterprogramm fr Anzeige Menpunkt 5.3.DDS Takt
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
cursor_ddstaktm
	movlw 0x1E				;vorbereiten fuer Log-Tabelle im Bereich 0x1D00..0x1DFF
	movwf PCLATH			;
	movfw TEMP_DIG_CNT
	addlw 0x1F
	bsf PCLATH,4			;Select Page 3
	call cursor_tab
	bcf PCLATH,3			;Select Page 2
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call wait_250_p1		;Wartezeit von 250ms Page1
	clrf TEMP_DIG_STAT		;lsche Statusbit Register
	return


dig_offset1					;Unterprogramm zur Erhhung/Reduzierung (10 Bit: 0=>1023) der Werte im Menuepunkt Einstellungen=>Offset
	movlw d'1'				;Prfen, ob der Offset Null Wert verndert werden soll
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_offset2		;nein, weiter 
	movfw OFFSET_NULL_L		;kopiere Offset Null Werte in Temp Register
	movwf TEMP_13
	movfw OFFSET_NULL_H
	movwf TEMP_16
	call incdec_offset		;Unterprogramm zur Erhhung/Reduzierung des Wertes
	movfw TEMP_13
	movwf OFFSET_NULL_L
	movfw TEMP_16
	movwf OFFSET_NULL_H
	goto dig_offset_end
dig_offset2					;
	movlw d'2'				;Prfen, ob der Offset -100% Wert verndert werden soll
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_offset_end		;nein, weiter 
	movfw OFFSET_100_L		;kopiere Offset Null Werte in Temp Register
	movwf TEMP_13
	movfw OFFSET_100_H
	movwf TEMP_16
	call incdec_offset		;Unterprogramm zur Erhhung/Reduzierung des Wertes
	movfw TEMP_13
	movwf OFFSET_100_L
	movfw TEMP_16
	movwf OFFSET_100_H
dig_offset_end
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4			;
	call disp_set_offset
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
cursor_offset
	movfw OFFSET_100_L		;Hole Offset -100% Low Byte nach W
	btfss TEMP_DIG_CNT,1	;prfen, ob der Cursor auf 2er Stelle steht
	movfw OFFSET_NULL_L		;Cursor steht auf 1er Stelle, hole Offset Null Low Byte nach W
	movwf DAC_L				;kopiere Low Byte ins DAC_L Register
	movfw OFFSET_100_H		;Hole Offset -100% High Byte nach W
	btfss TEMP_DIG_CNT,1	;prfen, ob der Cursor auf 2er Stelle steht
	movfw OFFSET_NULL_H		;Cursor steht auf 1er Stelle, hole Offset Null High Byte nach W
	movwf DAC_H				;kopiere High Byte ins DAC_H Register
	bsf DAC_H,5				;Ausgang B (Offset korr.) wird angesprochen (Bit 5 wird spter noch 2x nach links verschoben)
	bsf DAC_H,3				;Gain = 1, d.h. 0-1023 entsprechen 2-2048 mV
	call send_dac_data		;Aufruf des Unterprogramms zum senden der Daten an den DAC
	movlw 0xCD
	btfss TEMP_DIG_CNT,1	;prfen, ob der Cursor auf 2er Stelle steht
	movlw 0x8D
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4			;
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call wait_250_p1		;Wartezeit von 250ms Page1
	clrf TEMP_DIG_STAT		;lsche Statusbit Register
	return

dig_dacgain
	movfw DAC_GAIN			;prfe ob DAC_GAIN = 0, wenn ja setze DAC_GAIN auf 1, wenn nein auf 0
	btfss STATUS,Z
	goto $+4				;DAC_GAIN >0< => setze DAC_GAIN auf 0
	movlw 0x01				;DAC_GAIN = 0 => setze DAC_GAIN auf 1
	movwf DAC_GAIN	
	goto $+2
	clrf DAC_GAIN			;setze DAC_GAIN auf 0
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4			;
	call disp_set_dacgain	;Aufruf des Unterprogramms
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	call wait_250_p1		;Wartezeit von 250ms Page1
	return	

incdec_offset
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_offset			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incfsz TEMP_13			;erhhe Low Byte um 1 und prfe den berlauf
	return					;kein berlauf des Low Bytes
	incf TEMP_16			;berlauf des Low Bytes, erhhe High Byte um 1 und prfe ob High Byte = 4 (1024=> berlauf)
	btfss TEMP_16,2
	return					;High Byte noch nicht 4 (kein berlauf)
	clrf TEMP_13			;High Byte ist 4 (berlauf), Rcksetzen des High/Low Bytes auf 0
	clrf TEMP_16
	return
dec_offset					;Unterprogramm zur Reduzierung (10 Bit: 1023=>0) der Werte im Menuepunkt Einstellungen=>Offset
	movlw d'1'				
	subwf TEMP_13,f			;subtrahiere 1 vom Low Byte und prfe den Unterlauf (Carry Bit)
	btfsc STATUS,C
	return					;kein Unterlauf des Low Bytes
	subwf TEMP_16,f			;Unterlauf des Low Bytes, subtrahiere 1 vom High Byte und prfe den Unterlauf (Carry Bit)
	btfsc STATUS,C
	return					;kein Unterlauf des High Bytes
	movlw 0xFF				;Unterlauf des High Bytes, Rcksetzen des High/Low Bytes auf Wert 1023 (0x03FF)
	movwf TEMP_13
	movlw 0x03
	movwf TEMP_16
	return

inc_dec_dds
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_dds			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf TEMP_13
	movlw d'10'
	subwf TEMP_13,w
	btfsc STATUS,Z		
	clrf TEMP_13
	return
dec_dds
	movfw TEMP_13			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf TEMP_13			;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'9'
	movwf TEMP_13
	return



dds_freqformat
	rlf DDS_FREQREG_1,f
	rlf DDS_FREQREG_2,f
	rlf DDS_FREQREG_3,f
	rlf DDS_FREQREG_1,f
	rlf DDS_FREQREG_2,f
	rlf DDS_FREQREG_3,f
	rrf DDS_FREQREG_1,f
	rrf DDS_FREQREG_1,f
	bsf DDS_FREQREG_1,6			;F-Reg 0,
	bcf DDS_FREQREG_1,7			;F-Reg 0,
	bsf DDS_FREQREG_3,6			;F-Reg 0,
	bcf DDS_FREQREG_3,7			;F-Reg 0,
	return

write_dds_control				;Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	bsf OUT_CLK					;Clock auf High setzen
	nop							;kurze Wartezeit
	bcf DDS_CS					;setze Chip select Funktionsgenerator auf low (enable)
	movfw DDS_CONTROL_H			;sende Controlregister Daten, High Byte
	call send_dds_data
	movfw DDS_CONTROL_L			;sende Controlregister Daten, Low Byte
	call send_dds_data
	bsf DDS_CS					;setze Chip select Funktionsgenerator auf high (disable)
	return

write_dds_freq					;Unterprogramms zum Schreiben des Frequenz Registers, DDS Chips AD9833
	bsf OUT_CLK					;Clock auf High setzen
	nop
	bcf DDS_CS					;setze Chip select Funktionsgenerator auf low (enable)
	movfw DDS_FREQREG_1			;sende Frequenzregister Daten (LSB)
	call send_dds_data
	movfw DDS_FREQREG_0
	call send_dds_data
	bsf DDS_CS					;setze Chip select Funktionsgenerator auf high (disable)
	nop							;kurze Wartezeit
	nop
	bcf DDS_CS					;setze Chip select Funktionsgenerator auf low (enable)
	movfw DDS_FREQREG_3			;sende Frequenzregister Daten (MSB)
	call send_dds_data
	movfw DDS_FREQREG_2
	call send_dds_data
	bsf DDS_CS					;setze Chip select Funktionsgenerator auf high (disable)
write_dds_phase					;Unterprogramms zum Schreiben des Phasen Registers, DDS Chips AD9833
	nop							;kurze Wartezeit
	nop
	bcf DDS_CS					;setze Chip select Funktionsgenerator auf low (enable)
	movfw DDS_PHASEREG_H		;Phasenregister: 1100 XXXX XXXX XXXX
	call send_dds_data
	movfw DDS_PHASEREG_L
	call send_dds_data
	bsf DDS_CS					;setze Chip select Funktionsgenerator auf high (disable)
	return


send_dds_data
	movwf SHIFT_REG				;Das W-Register wird in das Reg. SHIFT_REG kopiert
	bsf OUT_CLK					;Takt auf high setzen, damit die Daten bei der fallenden
									;Flanke bernommen werden knnen
	movlw d'8'					;es sollen 8 Bits bertragen werden
	movwf SHIFT_CNT				;Zhler fr die bertragenen bits
send_dds_loop
	rlf SHIFT_REG,F				;Das Zeichen wird nach links verschoben. Das Carryflag wird
									;an den Ausgang weitergegeben
	btfsc STATUS,C				;Prfen, welchen Wert das Carryflag hat
	bsf OUT_DAT					;Carryflag = 1 -> Pin SI auf Highpegel setzen
	btfss STATUS,C				;Nochmal prfen, da immer nur ein Befehl bersprungen wird
	bcf OUT_DAT					;Carryflag = 0 -> Pin SI auf Lowpegel setzen
	nop							;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	bcf OUT_CLK					;mit der fallenden Flanke werden die Daten bernommen
	nop							;kurze Wartezeit
	bsf OUT_CLK					;positive Flanke des Taktes
	decfsz SHIFT_CNT,F			;ein bit wurde bertragen -> Zhler um 1 verringern
	goto send_dds_loop			;nchstes bit bertragen
	bcf OUT_DAT					;Pin SI auf Low zurcksetzen
	return						;Rcksprung aus dem Unterprogramm



dds_pegel					;Unterprogramms zum setzen der Ausgnge fr den DDS Pegel
	movfw DDS_VOL
	sublw d'63'
	movwf SHIFT_REG
	btfss STATUS,C
	clrf SHIFT_REG
	btfsc DDS_WAVEFORM,1	;prfen ob Bit 1 von DDS_WAVEFORM low ist (Sinus/Dreieck)
	bsf SHIFT_REG,6			;Bit 1 ist high (Rechteck), setze Ausgang auf high (Rechteck)
	movfw SHIFT_REG
	call write_data
	bsf VOL_CS				;Chip Select (Strobe) des Schieberegisters auf High setzen
	nop
	nop
	nop
	bcf VOL_CS				;Chip Select (Strobe) des Schieberegisters auf Low setzen
	return

send_dac_data				; 
	rlf DAC_L				;Low und High Byte 2x nach links schieben (siehe MCP4812 Datenblatt)
	rlf DAC_H
	rlf DAC_L
	rlf DAC_H
	bsf DAC_H,4				;Shutdown Bit auf High setzen
	bcf DAC_CS				;Chip select auf Low setzen
	movfw DAC_H				;senden des High Bytes
	call write_data
	movfw DAC_L				;senden des low Bytes
	call write_data
	nop
	bsf DAC_CS				;Chip select auf High setzen, Daten werden bernommen
	bcf OUT_DAT				;Pin auf Low zurcksetzen
	return

write_data
	movwf SHIFT_REG			;Das W-Register wird in das Reg. SHIFT_REG kopiert
	bcf OUT_CLK				;Takt auf low setzen, damit die Daten bei der steigenden Flanke bernommen werden
	movlw d'8'				;es sollen 8 Bits bertragen werden
	movwf SHIFT_CNT			;Zhler fr die bertragenen bits
write_data_loop
	rlf SHIFT_REG,F			;Das Zeichen wird nach links verschoben. Das Carryflag wird an den Ausgang weitergegeben
	btfsc STATUS,C			;Prfen, welchen Wert das Carryflag hat
	bsf OUT_DAT				;Carryflag = 1 -> Pin SI auf Highpegel setzen
	btfss STATUS,C			;Nochmal prfen, da immer nur ein Befehl bersprungen wird
	bcf OUT_DAT				;Carryflag = 0 -> Pin SI auf Lowpegel setzen
	nop						;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	bsf OUT_CLK				;mit der steigenden Flanke werden die Daten bernommen
	nop						;kurze Wartezeit
	bcf OUT_CLK				;Takt wieder auf low setzen
	decfsz SHIFT_CNT,F		;ein bit wurde bertragen -> Zhler um 1 verringern
	goto write_data_loop	;nchstes bit bertragen
	return					;Rcksprung aus dem Unterprogramm

;*******************************************************************************************
;Internes EEPROM: Unterprogramme zum Lesen/Schreiben von Einstellungswerten 
;*******************************************************************************************

read_parameter				;laden der Konstanten aus dem internen EEPROM in die F-Register
	movlw 0x00				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_MCLK_0		;Kopieren der Daten von W nach F, DDS Oszillator Frequenz 25MHz => 0x017D7840
	movlw 0x01				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_MCLK_1		;Kopieren der Daten von W nach F, DDS Oszillator Frequenz 25MHz => 0x017D7840
	movlw 0x02				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_MCLK_2		;Kopieren der Daten von W nach F, DDS Oszillator Frequenz 25MHz => 0x017D7840
	movlw 0x03				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_MCLK_3		;Kopieren der Daten von W nach F, DDS Oszillator Frequenz 25MHz => 0x017D7840
	movlw 0x04				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf MUL_VT			;Kopieren der Daten von W nach F, Vorteiler Faktor (0 bis 255)+1 = 1 bis 256
	movlw 0x09				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DAC_GAIN			;Kopieren der Daten von W nach F, DAC Verstrkung: 1 = 0-2048mV, 0 = 0-4096mV
	movlw 0x0A				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf PROGRAM			;Kopieren der Daten von W nach F, 0=Frequenz,1=Periode,2=Zhler,3=VCO,4=Wobbel
	movlw 0x0B				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf EINGANG			;Kopieren der Daten von W nach F, 0 = TTL, 1 = Analog, 2 = DDS, 3 = Vorteiler
	movlw 0x0C				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf FREQREG_L			;Kopieren der Daten von W nach F, VCO/Wobbel Fu L-Byte
	movlw 0x0D				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf FREQREG_M			;Kopieren der Daten von W nach F, VCO/Wobbel Fu M-Byte
	movlw 0x0E				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf FREQREG_H			;Kopieren der Daten von W nach F, VCO/Wobbel Fu H-Byte
	movlw 0x0F				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf KENNLIN			;Kopieren der Daten von W nach F, VCO/Wobbel Kennlinie: 0 = Linear, 1 = Logarithmisch
	movlw 0x10				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DEKADE			;Kopieren der Daten von W nach F, VCO/Wobbel Dekaden: 1-3
	movlw 0x11				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0			;Select BANK_1
	movwf WOB_STEP			;Kopieren der Daten von W nach F, Wobbel Steps: 1=1024, 2=512, 3=256, 4=128, 5=64, 6=32
	bcf STATUS,RP0			;Select BANK_0
	movlw 0x12				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0			;Select BANK_1
	movwf WOB_M_A			;Kopieren der Daten von W nach F, Wobbel Man/Aut: 0 = Automatisch, 1 = manuell
	bcf STATUS,RP0			;Select BANK_0
	movlw 0x13				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0			;Select BANK_1
	movwf WOB_ZEIT			;Kopieren der Daten von W nach F, Wobbel delta t: 	0=10ms, 1=20ms, 2=50ms, 3=100ms, 4=200ms,
	bcf STATUS,RP0			;Select BANK_0,										5=500ms, 6=1000ms, 7=2000ms
	movlw 0x14				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_WAVEFORM		;Kopieren der Daten von W nach F, DDS Wellenform: 0=Sinus, 1=Dreieck, 2=Rechteck
	movlw 0x15				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_VOL			;Kopieren der Daten von W nach F, DDS Ausgangspegel: 0-7 entspricht 0dB bis -42dB
	movlw 0x16				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_FREQ_0		;Kopieren der Daten von W nach F, DDS Frequenz (x10 fr eine Nachkommastelle) 
	movlw 0x17				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_FREQ_1		;Kopieren der Daten von W nach F, DDS Frequenz (x10 fr eine Nachkommastelle) 
	movlw 0x18				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_FREQ_2		;Kopieren der Daten von W nach F, DDS Frequenz (x10 fr eine Nachkommastelle) 
	movlw 0x19				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_FREQ_3		;Kopieren der Daten von W nach F, DDS Frequenz (x10 fr eine Nachkommastelle) 
	movlw 0x1A				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_PHASE_L		;Kopieren der Daten von W nach F, DDS Phasenwinkel (x10 fr eine Nachkommastelle) 
	movlw 0x1B				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DDS_PHASE_H		;Kopieren der Daten von W nach F, DDS Phasenwinkel (x10 fr eine Nachkommastelle)
	movlw 0x1C				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf TTL_ON_OFF		;Kopieren der Daten von W nach F, TTL Ausgang
read_offset
	movlw 0x05				;EEPROM Adresse Offset Null SIN, Low Byte
	btfsc DDS_WAVEFORM,1
	movlw 0x07				;EEPROM Adresse Offset Null SQR, Low Byte
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf OFFSET_NULL_L		;Kopieren der Daten von W nach F, Offset Wert fr Sinus/Dreieck, Low Byte
	movlw 0x06				;EEPROM Adresse Offset Null SIN, High Byte
	btfsc DDS_WAVEFORM,1
	movlw 0x08				;EEPROM Adresse Offset Null SQR, High Byte
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf OFFSET_NULL_H		;Kopieren der Daten von W nach F, Offset Wert fr Sinus/Dreieck, High Byte
	movlw 0x1D				;EEPROM Adresse Offset -100% SIN, Low Byte
	btfsc DDS_WAVEFORM,1
	movlw 0x1F				;EEPROM Adresse Offset -100% SQR, Low Byte
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf OFFSET_100_L		;Kopieren der Daten von W nach F, Offset Wert fr Rechteck, Low Byte
	movlw 0x1E				;EEPROM Adresse Offset -100% SIN, High Byte
	btfsc DDS_WAVEFORM,1
	movlw 0x20				;EEPROM Adresse Offset -100% SQR, High Byte
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf OFFSET_100_H		;Kopieren der Daten von W nach F, Offset Wert fr Rechteck, High Byte
	movlw 0x21				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf OFFSET_VAL		;Kopieren der Daten von W nach F, Offset Wert
	return

writeepr_mess				;Unterprogramms zum Schreiben von Einstellungswerten in das interne EEPROM
	movfw PROGRAM			;kopiere Wert fr Programm in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x0A				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw EINGANG			;kopiere Wert fr Eingang in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x0B				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	return

writeepr_dds
	movfw DDS_WAVEFORM		;kopiere Wert fr DDS Wellenform in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x14				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw DDS_VOL			;kopiere Wert fr DDS Signalpegel in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x15				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw DDS_FREQ_0		;kopiere Wert fr DDS Frequenz (x10) in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x16				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw DDS_FREQ_1		;kopiere Wert fr DDS Frequenz (x10) in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x17				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw DDS_FREQ_2		;kopiere Wert fr DDS Frequenz (x10) in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x18				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw DDS_FREQ_3		;kopiere Wert fr DDS Frequenz (x10) in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x19				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw DDS_PHASE_L		;kopiere Wert fr DDS Phasenwinkel (x10) in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x1A				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw DDS_PHASE_H		;kopiere Wert fr DDS Phasenwinkel (x10) in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x1B				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw TTL_ON_OFF		;kopiere Wert fr Ausgang TTL ON/OFF in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x1C				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw OFFSET_VAL		;kopiere Wert fr Ausgang TTL ON/OFF in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x21				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	return

writeepr_vco_wob			;Unterprogramms zum Schreiben von Einstellungswerten in das interne EEPROM
	movfw ZAHL_C0			;Kopiere Fu (L-Byte) von den Rechenregistern in die Frequenz- und Schreibregister fr das EEPROM
	movwf FREQREG_L			;
	movwf TEMP_13			;
	movlw 0x0C				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw ZAHL_C1			;Kopiere Fu (M-Byte) von den Rechenregistern in die Frequenz- und Schreibregister fr das EEPROM
	movwf FREQREG_M
	movwf TEMP_13			;
	movlw 0x0D				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw ZAHL_C2			;Kopiere Fu (H-Byte) von den Rechenregistern in die Frequenz- und Schreibregister fr das EEPROM
	movwf FREQREG_H
	movwf TEMP_13			;
	movlw 0x0E				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw PROGRAM			;Kopiere Programm in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x0A				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw KENNLIN			;Kopiere Kennlinie (Lin/Log) in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x0F				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	movfw DEKADE			;Kopiere Anzahl Dekaden in das Schreibregister fr das EEPROM
	movwf TEMP_13			;
	movlw 0x10				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	
	bsf STATUS,RP0			;Select BANK_1	
	movfw WOB_STEP			;Kopiere Programm in das Schreibregister fr das EEPROM
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13			;
	movlw 0x11				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	bsf STATUS,RP0			;Select BANK_1
	movfw WOB_M_A			;Kopiere Kennlinie (Lin/Log) in das Schreibregister fr das EEPROM
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_13			;
	movlw 0x12				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM

	bsf STATUS,RP0			;Select BANK_1
	movfw WOB_ZEIT			;Kopiere Anzahl Dekaden in das Schreibregister fr das EEPROM
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_13			;
	movlw 0x13				;Lade die Adresse des EEPROM in die der Wert geschrieben werden soll
	call writeEE			;Aufruf des Unterprogramms zum Schreiben in das interne EEPROM
	return

readEE							;Unterprogramm zum lesen der Zelle deren Adresse in W steht (internes EEPROM)
	bsf STATUS, RP1 			;EEADR liegt in der Bank 2 
	bcf STATUS, RP0
 	movwf EEADR 				;schreibe die Adresse in EEADR 
	bsf STATUS, RP0				;Bank 3
	bcf EECON1, EEPGD			;Access data memory
	bsf EECON1, RD 				;EEPROM Leseproze starten 
	bcf STATUS, RP0 			;Bank 2 
	movf EEDATA, W 				;Die Daten der EEPROM Zelle nach W kopieren 
	bcf STATUS, RP1 			;Bank 0 
	return

writeEE							;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht (internes EEPROM)
	bsf STATUS, RP0				;Bank 3 (EECON1 ist in Bank 3)
	bsf STATUS, RP1
	btfsc EECON1, WR			;prfen, ob das EEPROM beschrieben werden kann
	goto $-1					;warten, bis das bit WR von der Hardware zurckgesetzt wurde
	bcf STATUS, RP0				;Bank 2 (EEADR und EEDATA sind in Bank 2)
	movwf EEADR					;Die Adresse an die die Daten geschrieben werden sollen ist im W-Register
	bcf STATUS, RP1				;Bank 0
	movfw TEMP_13				;hole zu speichernden Wert ins W-Register
	bsf STATUS, RP1				;Bank 2 (EEADR und EEDATA sind in Bank 2)
	movwf EEDATA				;Wert wird in EEADR geschrieben.
	bsf STATUS, RP0				;Bank 3
	bcf EECON1, EEPGD			;Das Bit EEPGD lschen, um das interne Daten-EEPROM auszuwhlen. Wird das Bit gesetzt, wird der Progamm-Flash ausgewhlt
	bsf EECON1, WREN			;erlaubt das Beschreiben des Daten-EEPROM
	movlw 0x55					;Um das EEPROM zu beschreiben, mssen die folgenden 5 Befehle
	movwf EECON2				;genau in dieser Reihenfolge aufgerufen werden.
	movlw 0xAA					;
	movwf EECON2				;
	bsf EECON1, WR				;Jetzt beginnt der eigentliche Schreibvorgang.
	bcf EECON1, WREN			;Das Beschreiben des EEPROM verbieten.
	bcf STATUS, RP0				;Bank 0
	bcf STATUS, RP1
	return						;Zurck ins Hauptprogramm



;*******************************************************************************************
;	PAGE 2, 	bcf PCLATH,3				;Select Page 2
;				bsf PCLATH,4
;*******************************************************************************************
	ORG 0x1000					;PAGE 2

;*******************************************************************************************
; Unterprogramme zur Anzeige auf dem Display
;*******************************************************************************************

disp_menue_1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw d'1'				;
	subwf MENU_COUNT,w	;
	btfss STATUS,Z		
	goto disp_menue_2
	call dispm1_mess
	goto disp_menue_end
disp_menue_2
	movlw d'2'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_3
	call dispm1_dds
	goto disp_menue_end
disp_menue_3
	movlw d'3'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_4
	call dispm1_vco
	goto disp_menue_end
disp_menue_4
	movlw d'4'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_end
	call dispm1_einstell
	goto disp_menue_end
disp_menue_end
	movlw d'250'				;Wartezeit von 250ms
	call wait 	
	bsf MENU_STAT,7				;setze das Status Bit
	return

dispm1_mess
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'M'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm1_dds
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movlw A'P'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm1_vco
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'3'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'C'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'/'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'W'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'b'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'b'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return


dispm1_einstell
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

Init_display					;Unterprogramm zur Initialisierung des Displays
	movlw d'250'				;geforderte Wartezeit des Displays von 200ms
	call wait 		
	movlw 0x38					;Befehl: Function Set
	call disp_befehl	;
	movlw 0x39					;Befehl: Function Set, (muss zweimal bertragen werden)
	call disp_befehl	;
	movlw 0x14					;Befehl: Bias Set
	call disp_befehl	;
	movlw 0x78					;Befehl: Contrast Set
	call disp_befehl	;
	movlw 0x52					;Befehl: Power/Icon/Contrast Contr.
	call disp_befehl	;
	movlw 0x69					;Befehl: Follower Control
	call disp_befehl	;
	movlw d'250'				;geforderte Wartezeit des Displays von 200ms
	call wait 		
	movlw 0x0C					;Befehl: Display On/Off Control (00001100)
	call disp_befehl	;
	call clear_display			;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x06					;Befehl: Entry Mode Set (00000110)
	call disp_befehl	;
	return

clear_display					;Unterprogramm zum lschen des Displays					
	movlw 0x01					;Befehl: Clear Display
	call disp_befehl	;
	movlw d'2'					;geforderte Wartezeit des Displays von 1,08ms
	call wait 		
	return

cursor_off
	movlw 0x0C					;Befehl: Display On/Off Control, Cursor OFF (00001100)
	call disp_befehl			;
	movlw d'20'					;Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return

cursor_on
	movlw 0x0E					;Befehl: Display On/Off Control, Cursor ON (00001110)
	call disp_befehl			;
	movlw d'20'					;geforderte Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return

disp_main
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'1'
	subwf PROGRAM,w
	btfss STATUS,Z
	goto disp_prog_freq
	movlw A'P'					;ASCII:P fr Periode
	call disp_daten				;bertrage das Zeichen
	goto disp_wert
disp_prog_freq
	movlw A'F'					;ASCII:F fr Frequenz
	call disp_daten				;bertrage das Zeichen
disp_wert
	movlw A':'					;ASCII: ":"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movfw DISP_6				;Daten fr Millionen 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	btfss KOMMA,6				;prfen ob ein Komma gesetzt werden soll
	goto $+3					;nein, weiter
	movlw A','					;ja, Daten fr ","
	call disp_daten				;bertrage das Zeichen
	movfw DISP_5				;Daten fr 100.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	btfss KOMMA,5				;prfen ob ein Komma gesetzt werden soll
	goto $+3					;nein, weiter
	movlw A','					;ja, Daten fr ","
	call disp_daten				;bertrage das Zeichen
	movfw DISP_4				;Daten fr 10.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	btfss KOMMA,4				;prfen ob ein Komma gesetzt werden soll
	goto $+3					;nein, weiter
	movlw A','					;ja, Daten fr ","
	call disp_daten				;bertrage das Zeichen
	movfw DISP_3				;Daten fr 1.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_2				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_1				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_0				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movfw UNIT					;Zeichen fr die Einheit, z.B. k fr kilo, m fr milli, u fr mikro, n fr nano
	call disp_daten				;bertrage das Zeichen

	movlw d'1'
	subwf PROGRAM,w
	btfss STATUS,Z
	goto $+4
	movlw A's'					;ASCII:s fr Sekunde
	call disp_daten				;bertrage das Zeichen
	goto disp_main_z2
	movlw A'H'					;ASCII:H fr Herz
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:z
	call disp_daten				;bertrage das Zeichen
disp_main_z2
	movlw 0xC0					;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'					;ASCII:P fr Periodenmessung
	subwf DISP_PRESC,w			;prfen ob Periodenmessung verwendet wird
	btfss STATUS,Z
	goto disp_main_z2f			;nein, Frequenzmessung
	movlw A'P'					;ASCII:P fr Periodenmessung
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	goto disp_main_z2fe
disp_main_z2f
	movfw DISP_PRESC			;Daten fr Prescaler
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
disp_main_z2fe	
	btfss MENU_STAT,5			;Prfe das Display Update Bit fr Anzeige Eingang und DDS Einstellungen
	call disp_eing_ddsset		;Bit ist nicht gesetzt, Aufruf des Unterprogramms
	return						;Bit ist gesetzt, kein Update erforderlich

disp_zaehl
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'Z'					;ASCII:Z
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII: ":"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movfw DISP_8				;Daten fr Millionen  100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_7				;Daten fr Millionen  10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_6				;Daten fr Millionen 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII: "."
	call disp_daten				;bertrage das Zeichen
	movfw DISP_5				;Daten fr 100.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_4				;Daten fr 10.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_3				;Daten fr 1.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII: "."
	call disp_daten				;bertrage das Zeichen
	movfw DISP_2				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_1				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_0				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:n fr Anzahl
	call disp_daten				;bertrage das Zeichen
	btfsc MENU_STAT,5			;Prfe das Display Update Bit fr Anzeige Eingang und DDS Einstellungen
	return						;Bit ist gesetzt, kein Update erforderlich
	movlw 0xC0					;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_eing_ddsset		;Bit ist nicht gesetzt, Aufruf des Unterprogramms
	return

disp_vco
	movlw 0xC0					;Zeichen 1 in der 2. Zeile 
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_vco_wob
	movlw 0x20					;ASCII: "blank"
	call disp_daten				;bertrage das Zeichen	
	call disp_vco_kl			;Aufruf des Unterprogramms zum Schreiben der Kennlinie (LIN/LOG)
	movlw 0x20					;ASCII: "blank"
	call disp_daten				;bertrage das Zeichen	
	movfw DEKADE				;Daten fr Anzahl Dekaden
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII: "D"
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII: "e"
	call disp_daten				;bertrage das Zeichen
	movlw A'k'					;ASCII: "k"
	call disp_daten				;bertrage das Zeichen
	btfss PROGRAM,2				;Prfen, ob Wobbel gewhlt ist (Wobbel =4, Bit 2 prfen)
	return						;Wobbel ist nicht gewhlt, Rcksprung aus dem Unterprogramm
	movlw 0x20					;ASCII: "blank"
	call disp_daten				;bertrage das Zeichen	
	movlw A'M'					;ASCII: hole "M" ins W-Register	
	bsf STATUS,RP0				;Select BANK_1
	btfss WOB_M_A,0				;prfen ob "Manuell" (1) gewhlt ist
	movlw A'A'					;ASCII: "Auto" (0) ist gewhlt, hole "A" ins W-Register
	bcf STATUS,RP0				;Select BANK_0
	call disp_daten				;bertrage das Zeichen
	bsf STATUS,RP0				;Select BANK_1
	btfss WOB_M_A,1				;prfen ob "Burst" (1) gewhlt ist
	goto $+4					;"Burst" (1) ist nicht gewhlt,
	bcf STATUS,RP0				;Select BANK_0, "Burst" (1) ist gewhlt
	movlw A'B'					;ASCII: hole "B" ins W-Register	
	call disp_daten				;bertrage das Zeichen
	bcf STATUS,RP0				;Select BANK_0
	return


disp_vco_freq
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'F'					;ASCII: "F"
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII: ":"
	call disp_daten				;bertrage das Zeichen
	movfw DISP_8
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_7
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DISP_6
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_5
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_4
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DISP_3
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_2
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_1
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DISP_0
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII: "H"
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII: "z"
	call disp_daten				;bertrage das Zeichen
	return

disp_wob_graph
	bsf STATUS,RP0				;Select BANK_1
	movfw WOB_COUNT_L			;kopiere WOB_COUNT Reg. nach AD_DA Reg., der AD_DA Wert (0-1023) wird durch 64 geteilt (=> 0-15)
	bcf STATUS,RP0				;Select BANK_0	
	movwf AD_DA_L
	bsf STATUS,RP0				;Select BANK_1
	movfw WOB_COUNT_H
	bcf STATUS,RP0				;Select BANK_0	
	movwf AD_DA_H
	movlw d'6'					;Die Temp Register werden 6 mal nach rechts verschoben (2^6 = 64), d.h. durch 64 geteilt 
	movwf LOOP_1
	bcf STATUS,C
	rrf AD_DA_H
	rrf AD_DA_L
	decfsz LOOP_1				;
	goto $-4					;noch nicht 6 mal geschoben, zurck zur Schleife
	movfw AD_DA_L				;6 mal geschoben, hole Low Byte nach W und addiere die 1.Adresse in der 1.Reihe
	btfsc STATUS,Z
	goto $+8					;Wert ist 0, lsche letztes Zeichen in Reihe 1 und schreibe Zeichen an erster Position
	addlw 0x7F
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x20					;Zeichen fr "blank"
	call disp_daten				;bertrage das Zeichen	
	movlw 0x7C					;Zeichen fr "I"
	call disp_daten				;bertrage das Zeichen	
	return
	movlw 0x8F					;Adresse fr letztes Zeichen in Reihe 1
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x20					;Zeichen fr "blank"
	call disp_daten				;bertrage das Zeichen	
	movlw 0x80					;Adresse fr erstes Zeichen in Reihe 1
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x7C					;Zeichen fr "I"
	call disp_daten				;bertrage das Zeichen	
	return

disp_eing_ddsset
	bsf MENU_STAT,5				;Setze das Display Update Bit fr Eingang und DDS Einstellungen
	movlw A' '					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen	
	call disp_freq_eingang		;Aufruf des Unterprogramms zur Anzeige des Messeingangs
	movlw A' '					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movlw 0x02					;Prfen ob Eingang DDS gewhlt ist
	subwf EINGANG,w
	btfss STATUS,Z	
	return						;Eingang ist nicht DDS
	call disp_dds_waveform		;Eingang ist DDS, Anzeige von Wellenform und Signalabschwchung
	movlw A' '					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	call disp_dds_vol			;Anzeige der Signalabschwchung
	return

disp_freq_eingang
	movlw 0x03
	subwf EINGANG,w
	btfsc STATUS,Z	
	goto disp_freq_vt
	movlw 0x02
	subwf EINGANG,w
	btfsc STATUS,Z	
	goto disp_freq_dds
	movlw 0x01
	subwf EINGANG,w
	btfsc STATUS,Z	
	goto disp_freq_analog
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return	
disp_freq_vt
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return	
disp_freq_dds
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return	
disp_freq_analog
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'N'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A' '					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return	


disp_uhr
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'					;ASCII:U
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:h
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:r
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movfw ZAHL_A2				;Daten fr Stunde
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw ZAHL_A1				;Daten fr Minute
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw ZAHL_A0				;Daten fr Sekunde
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	return

disp_set_mess
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_mess_prog
;2.Reihe
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_mess_eingang
	return

disp_mess_prog
	movlw 0x86					;Zeichen 7 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x02
	subwf PROGRAM,w
	btfsc STATUS,Z	
	goto disp_mess_z
	movlw 0x01
	subwf PROGRAM,w
	btfsc STATUS,Z	
	goto disp_mess_p
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return	
disp_mess_z
	movlw A'Z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return	
disp_mess_p
	movlw A'P'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return	

disp_mess_eingang
	movlw 0xC9					;Zeichen 10 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x03
	subwf EINGANG,w
	btfsc STATUS,Z	
	goto disp_mess_vt
	movlw 0x02
	subwf EINGANG,w
	btfsc STATUS,Z	
	goto disp_mess_dds
	movlw 0x01
	subwf EINGANG,w
	btfsc STATUS,Z	
	goto disp_mess_analog
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return	
disp_mess_vt
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return	
disp_mess_dds
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return	
disp_mess_analog
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return	

disp_set_dds
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'16'					;wenn TEMP_DIG_CNT <16 wird die normale Anzeige verwendet
	subwf TEMP_DIG_CNT,w		;wenn TEMP_DIG_CNT >=16 wird die 2.Reihe in der 1.Reihe
	btfsc STATUS,C				;geschoben und die 3.Reihe in der 2.Reihe angezeigt
	goto disp_set_dds1

	call disp_dds_waveform		;Aufruf des Unterprogramms zum Schreiben der Wellenform (SIN, TRI, SQR)
	movlw 0x84					;Zeichen 5 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_dds_vol			;Aufruf des Unterprogramms zum Schreiben des Ausgangspegels (-00dB bis -63dB)
	movlw 0x8A					;Zeichen 11 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw ZAHL_A3				;Daten fr Winkel 100er
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw ZAHL_A2				;Daten fr Winkel 10er
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw ZAHL_A1				;Daten fr Winkel 1er
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw ZAHL_A0				;Daten fr Winkel 0,1er
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw d'223'				;Zeichen fr 
	call disp_daten				;bertrage das Zeichen
;2.Reihe
	movlw 0xC0					;Zeichen 2 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
disp_set_dds1
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_frequenz			;Anzeige Frequenz: 00.000.000
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DISP_0				;Daten fr 0,1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen

	movlw d'16'					;wenn TEMP_DIG_CNT <16 wird die normale Anzeige verwendet
	subwf TEMP_DIG_CNT,w		;wenn TEMP_DIG_CNT >=16 wird die 2.Reihe in der 1.Reihe
	btfsc STATUS,C				;geschoben und die 3.Reihe in der 2.Reihe angezeigt
	goto $+2
	return
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'f'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'f'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw AD_DA_REST			;Offset Vorzeichen
	call disp_daten				;bertrage das Zeichen
	movfw AD_DA_H				;Offset 100er/10er
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movfw AD_DA_L				;Offset 1er
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'%'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw TTL_ON_OFF
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen	
	return

disp_frequenz
	movfw DISP_8				;Daten fr Millionen 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
disp_freq_fu
	movfw DISP_7				;Daten fr Millionen 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DISP_6				;Daten fr 100.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_5				;Daten fr 10.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_4				;Daten fr 1.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DISP_3				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_2				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_1				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

disp_dds_waveform
	movlw 0x02
	subwf DDS_WAVEFORM,w
	btfsc STATUS,Z	
	goto disp_dds_sqr
	movlw 0x01
	subwf DDS_WAVEFORM,w
	btfsc STATUS,Z	
	goto disp_dds_tri
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'I'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'N'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
disp_dds_tri
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'R'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'I'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
disp_dds_sqr
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'R'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return

disp_dds_vol
	movlw A'-'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DDS_VOL				;
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'B'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return

disp_set_vco
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'11'					;wenn TEMP_DIG_CNT <11 wird die normale Anzeige verwendet
	subwf TEMP_DIG_CNT,w		;wenn TEMP_DIG_CNT >=11 wird die 2.Reihe in der 1.Reihe
	btfsc STATUS,C				;geschoben und die 3.Reihe in der 2.Reihe angezeigt
	goto disp_set_vco1

	call disp_vco_wob			;Aufruf des Unterprogramms zum schreiben von "VCO" oder "WOB"
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A'K'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_vco_kl			;Aufruf des Unterprogramms zum Schreiben der Kennlinie (LIN/LOG)
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'k'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DEKADE
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
;2.Reihe
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
disp_set_vco1
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	call disp_freq_fu			;Anzeige Frequenz: 0.000.000
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen

	movlw d'11'					;wenn TEMP_DIG_CNT <11 wird die normale Anzeige verwendet
	subwf TEMP_DIG_CNT,w		;wenn TEMP_DIG_CNT >=11 wird die 2.Reihe in der 1.Reihe
	btfsc STATUS,C				;geschoben und die 3.Reihe in der 2.Reihe angezeigt
	goto $+2
	return
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_vco_step			;Unterprogramm zum Schreiben der Steps (0032 - 1024)
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A'M'					;Hole 'M' fr Manuell nach W
	bsf STATUS,RP0				;Select BANK_1	
	btfss WOB_M_A,0				;prfen ob Manuell x1 oder Automatisch x0
	movlw A'A'					;Automatisch, Hole 'A' nach W
	bcf STATUS,RP0				;Select BANK_0	
	call disp_daten				;bertrage das Zeichen
	movlw A'B'					;Hole 'B' fr Burst nach W
	bsf STATUS,RP0				;Select BANK_1	
	btfss WOB_M_A,1				;prfen ob Burst 1x oder kontinuierlich 0x
	movlw 0x20					;kontinuierlich, Hole ' ' nach W
	bcf STATUS,RP0				;Select BANK_0	
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	bsf STATUS,RP0				;Select BANK_1	
	btfss WOB_M_A,0				;prfen ob Manuell x1 oder Automatisch x0
	call disp_vco_dt
	bcf STATUS,RP0				;Select BANK_0	
	return

disp_vco_wob
	btfsc PROGRAM,2				;Prfen ob Programm VCO (=3 => 00000011) oder Wobbel (=4 => 00000100) gewhlt ist (Bit 2 prfen) 
	goto $+8					;Bit 2 ist gesetzt, d.h. Wobbel ist gewhlt	
	movlw A'V'					;Bit 2 ist nicht gesetzt, d.h. VCO ist gewhlt,	ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'C'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return
	movlw A'W'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'B'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_vco_kl						;Unterprogramms zum Schreiben der Kennlinie (LIN/LOG)
	movfw KENNLIN				;hole Kennline nach W und prfe ob 0
	btfss STATUS,Z
	goto disp_vco_kl_log		;Kennlinie ist nicht 0, weiter beim Unterprogramms zum Schreiben der Kennlinie (LOG)
	goto disp_vco_kl_lin		;Kennlinie ist 0, weiter beim Unterprogramms zum Schreiben der Kennlinie (LIN)
disp_vco_kl_log
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return
disp_vco_kl_lin
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_vco_step
	bsf STATUS,RP0				;Select BANK_1	
	movlw d'1'					;prfen, ob 1024 Schritte (WOB_STEP =1 ) gewhlt sind
	subwf WOB_STEP,w
	btfss STATUS,Z
	goto disp_vco_step2			;1024 Schritte (WOB_STEP =1 ) sind nicht gewhlt, weiter
	bcf STATUS,RP0				;Select BANK_0
	movlw d'10'					;1024 Schritte (WOB_STEP =1 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	movlw d'24'
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	return
disp_vco_step2
	movlw d'2'					;prfen, ob 512 Schritte (WOB_STEP =2 ) gewhlt sind
	subwf WOB_STEP,w
	btfss STATUS,Z
	goto disp_vco_step3			;512 Schritte (WOB_STEP =2 ) sind nicht gewhlt, weiter
	bcf STATUS,RP0				;Select BANK_0
	movlw d'05'					;512 Schritte (WOB_STEP =2 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	movlw d'12'
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	return
disp_vco_step3
	movlw d'3'					;prfen, ob 256 Schritte (WOB_STEP =3 ) gewhlt sind
	subwf WOB_STEP,w
	btfss STATUS,Z
	goto disp_vco_step4			;256 Schritte (WOB_STEP =3 ) sind nicht gewhlt, weiter
	bcf STATUS,RP0				;Select BANK_0
	movlw d'02'					;256 Schritte (WOB_STEP =3 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	movlw d'56'
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	return
disp_vco_step4
	movlw d'4'					;prfen, ob 128 Schritte (WOB_STEP =4 ) gewhlt sind
	subwf WOB_STEP,w
	btfss STATUS,Z
	goto disp_vco_step5			;128 Schritte (WOB_STEP =4 ) sind nicht gewhlt, weiter
	bcf STATUS,RP0				;Select BANK_0
	movlw d'01'					;128 Schritte (WOB_STEP =4 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	movlw d'28'
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	return
disp_vco_step5
	movlw d'5'					;prfen, ob 64 Schritte (WOB_STEP =5 ) gewhlt sind
	subwf WOB_STEP,w
	btfss STATUS,Z
	goto disp_vco_step6			;64 Schritte (WOB_STEP =5 ) sind nicht gewhlt, weiter
	bcf STATUS,RP0				;Select BANK_0
	movlw d'00'					;64 Schritte (WOB_STEP =5 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	movlw d'64'
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	return
disp_vco_step6
	bcf STATUS,RP0				;Select BANK_0
	movlw d'00'					;32 Schritte (WOB_STEP =6 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	movlw d'32'
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	return

disp_vco_dt
	bsf STATUS,RP0				;Select BANK_1	
	movlw d'0'					;prfen, ob 1ms (WOB_Zeit =0 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt1			;1ms (WOB_Zeit =0 ) sind nicht gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw d'1'					;1ms (WOB_Zeit =0 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	call disp_vco_ms			;Unterprogramm zum Schreiben von ms
	return
disp_vco_dt1
	movlw d'1'					;prfen, ob 2ms (WOB_Zeit =1 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt2			;2ms (WOB_Zeit =1 ) sind nicht gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw d'2'					;2ms (WOB_Zeit =1 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	call disp_vco_ms			;Unterprogramm zum Schreiben von ms
	return
disp_vco_dt2
	movlw d'2'					;prfen, ob 5ms (WOB_Zeit =2 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt3			;5ms (WOB_Zeit =2 ) sind nicht gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw d'5'					;5ms (WOB_Zeit =2 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	call disp_vco_ms			;Unterprogramm zum Schreiben von ms
	return
disp_vco_dt3
	movlw d'3'					;prfen, ob 10ms (WOB_Zeit =3 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt4			;10ms (WOB_Zeit =3 ) sind nicht gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw d'10'					;10ms (WOB_Zeit =3 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	call disp_vco_ms			;Unterprogramm zum Schreiben von ms
	return
disp_vco_dt4
	movlw d'4'					;prfen, ob 20ms (WOB_Zeit =4 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt5			;20ms (WOB_Zeit =4 ) sind nicht gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw d'20'					;20ms (WOB_Zeit =4 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	call disp_vco_ms			;Unterprogramm zum Schreiben von ms
	return
disp_vco_dt5
	movlw d'5'					;prfen, ob 50ms (WOB_Zeit =5 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt6			;50ms (WOB_Zeit =5 ) sind nicht gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw d'50'					;50ms (WOB_Zeit =5 ) sind gewhlt
	call convdisp_hex_2dig		;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige		
	call disp_vco_ms			;Unterprogramm zum Schreiben von ms
	return
disp_vco_dt6
	movlw d'6'					;prfen, ob 100ms (WOB_Zeit =6 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt7			;100ms (WOB_Zeit =6 ) sind nicht gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw A'0'					;ASCII:  100ms (WOB_Zeit =6 ) sind gewhlt
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
disp_vco_dt7
	movlw d'7'					;prfen, ob 200ms (WOB_Zeit =7 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt8			;200ms (WOB_Zeit =7 ) sind nicht gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw A'0'					;ASCII:  200ms (WOB_Zeit =7 ) sind gewhlt
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
disp_vco_dt8
	movlw d'8'					;prfen, ob 500ms (WOB_Zeit =8 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt9			;500ms (WOB_Zeit =8 ) sind nicht gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw A'0'					;ASCII:  500ms (WOB_Zeit =8 ) sind gewhlt
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
disp_vco_dt9
	movlw d'9'					;prfen, ob 1000ms (WOB_Zeit =9 ) gewhlt sind
	subwf WOB_ZEIT,w
	btfss STATUS,Z
	goto disp_vco_dt10			;1000ms (WOB_Zeit =9 ) sind nicht gewhlt, weiter
	bcf STATUS,RP0				;Select BANK_0
	movlw A'1'					;ASCII: 1000ms (WOB_Zeit =9 ) sind gewhlt
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return
disp_vco_dt10					;2000ms (WOB_Zeit =10 ) sind gewhlt
	bcf STATUS,RP0				;Select BANK_0
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A','					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return

disp_vco_ms
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	return


disp_fehler1
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'/'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'k'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'c'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_fehler2
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'/'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'k'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return


disp_menu_einstell
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw d'1'				;
	subwf MENU_COUNT,w	;
	btfss STATUS,Z		
	goto disp_menu_einstell2
	call dispm_quhr
	goto disp_menu_einst_end
disp_menu_einstell2
	movlw d'2'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menu_einstell3
	call dispm_q1hz
	goto disp_menu_einst_end
disp_menu_einstell3
	movlw d'3'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menu_einstell4
	call dispm_ddstakt
	goto disp_menu_einst_end
disp_menu_einstell4
	movlw d'4'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menu_einstell5
	call dispm_vorteil
	goto disp_menu_einst_end
disp_menu_einstell5
	movlw d'5'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menu_einstell6
	call dispm_offset
	goto disp_menu_einst_end
disp_menu_einstell6
	movlw d'6'				;
	subwf MENU_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menu_einst_end
	call dispm_dacgain
	goto disp_menu_einst_end
disp_menu_einst_end
	movlw d'250'				;Wartezeit von 250ms
	call wait 	
	bsf MENU_STAT,6				;setze das Status Bit
	return

dispm_quhr
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'('					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'U'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A')'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm_q1hz
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'('					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A')'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm_ddstakt
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'3'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'k'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm_vorteil
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm_offset
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'f'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'f'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

dispm_dacgain
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'6'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'C'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A'G'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return


disp_q1hz
	movlw 0x85					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x8D					;Zeichen 14 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_q1hz_vz
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movf TEMP_10,w				;Vorzeichen der Abweichung
	call disp_daten				;bertrage das Zeichen
	return

disp_q1hz_dat
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movf DISP_3,w				;Digit 4
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_2,w				;Digit 3
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_1,w				;Digit 2
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movf DISP_0,w				;Digit 1
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

disp_set_ddstaktm
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A' '					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'k'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0xC1					;Zeichen 2 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw DISP_7				;Daten fr Millionen 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_6				;Daten fr Millionen 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DISP_5				;Daten fr 100.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_4				;Daten fr 10.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_3				;Daten fr 1.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DISP_2				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_1				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_0				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A' '					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_set_vort_t					;Unterprogramms zur Anzeige des Textes
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'-'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'W'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_set_vort_w					;Unterprogramms zur Anzeige des Wertes
	movlw 0x88					;Zeichen 9 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_11				;hole Wert des Vorteilerfaktors nach W und prfe ob 0
	btfss STATUS,Z
	goto $+8					;Wert >0<, weiter bei Auftrennung und Anzeige des Wertes
	movlw A'2'					;Wert ist 0, => 256, Schreibe direct in ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'6'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	goto disp_set_vtw_end
	movfw TEMP_11				;hole Wert des Vorteilerfaktors nach W	
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er,10er und 1er
	movfw TEMP_13				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
disp_set_vtw_end
	movlw d'250'				;Wartezeit von 250ms
	call wait 			
	return

disp_set_offset
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_dds_waveform		;Anzeige SIN, TRI oder SQR
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	movlw A'N'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	bcf PCLATH,4				;select Page_0		
	call clear_ZAHL_C			;Lsche ZAHL_C Register
	movfw OFFSET_NULL_L			;Kopiere Offset Werte L/H in ZAHL_C Register
	movwf ZAHL_C0
	movfw OFFSET_NULL_H
	movwf ZAHL_C1
	call convert_hex_dig		;Unterprogramm zum Zerlegen einer 32-Bit Zahl in die einzelnen Digits (max.999.999.999)zur Anzeige auf dem Display
	bsf PCLATH,4				;select Page_2		
	movfw DISP_3				;Zeichen fr 1.000er, untere 4 Bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_2				;Zeichen fr 100er, untere 4 Bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_1				;Zeichen fr 10er, untere 4 Bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_0				;Zeichen fr 1er, untere 4 Bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen

	movlw 0xC3					;Zeichen 4 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'-'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'%'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII: blank
	call disp_daten				;bertrage das Zeichen
	bcf PCLATH,4				;select Page_0		
	call clear_ZAHL_C			;Lsche ZAHL_C Register
	movfw OFFSET_100_L			;Kopiere Offset Werte L/H in ZAHL_C Register
	movwf ZAHL_C0
	movfw OFFSET_100_H
	movwf ZAHL_C1
	call convert_hex_dig		;Unterprogramm zum Zerlegen einer 32-Bit Zahl in die einzelnen Digits (max.999.999.999)zur Anzeige auf dem Display
	bsf PCLATH,4				;select Page_2		
	movfw DISP_3				;Zeichen fr 1.000er, untere 4 Bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_2				;Zeichen fr 100er, untere 4 Bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_1				;Zeichen fr 10er, untere 4 Bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw DISP_0				;Zeichen fr 1er, untere 4 Bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

disp_set_dacgain
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'-'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw DAC_GAIN				;prfe ob DAC_GAIN = 0, wenn ja schreibe 4 Volt, wenn nein schreibe 2 Volt
	btfss STATUS,Z
	goto $+4					;DAC_GAIN >0< => schreibe 2 Volt
	movlw A'4'					;DAC_GAIN = 0 => schreibe 4 Volt
	call disp_daten				;bertrage das Zeichen
	goto $+3
	movlw A'2'					;DAC_GAIN >0< => schreibe 2 Volt
	call disp_daten				;bertrage das Zeichen
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x88					;Zeichen 9 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call cursor_on
	return

convdisp_hex_2dig				;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	bcf STATUS,RP0				;Select BANK_0	
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return


convert_dig					;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf TEMP_15			;Temp Register fr Zwischenspeicherung des zu zerlegenden Wertes
	clrf TEMP_13			;lsche Temp Register
	clrf TEMP_14			;lsche Temp Register
dig_100er
	movlw d'100'			;schreibe Wert 100 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 100 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_10er			;Ergebnis negativ, weiter bei 10er
	incf TEMP_13			;Ergebnis noch positiv, Erhhe 100er um 1 
	movlw d'100'			; und reduziere den zu zerlegenden Werte um 100 im F-Register
	subwf TEMP_15,f			;
	goto dig_100er			;Zurck zur Schleife
dig_10er
	movlw d'10'				;schreibe Wert 10 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 10 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_1er			;Ergebnis negativ, weiter bei 1er
	incf TEMP_14			;Ergebnis noch positiv, Erhhe 10er um 1 
	movlw d'10'				; und reduziere den zu zerlegenden Werte um 10 im F-Register
	subwf TEMP_15,f			;
	goto dig_10er			;Zurck zur Schleife
dig_1er
	movfw TEMP_15
	return




disp_befehl
	movwf SHIFT_REG				;Register mit den Daten, die ausgegeben werden sollen
	bsf DISP_EN					;Enable auf High setzen
	bcf DISP_RS					;RS auf Low setzen, da es sich um ein Kommando handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_daten
	movwf SHIFT_REG				;Das W-Register wird in das Reg. SHIFT_REG kopiert
	bsf DISP_EN					;Enable auf High setzen
	bsf DISP_RS					;RS auf High setzen, da es sich um ein Zeichen handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_routine
	bcf SI_CLK					;Takt auf Low setzen, damit die Daten bei der steigenden
									;Flanke bernommen werden knnen
	movlw d'8'					;es sollen 8 Bits bertragen werden
	movwf SHIFT_CNT				;Zhler fr die bertragenen bits
shiftDaten
	rlf SHIFT_REG,F				;Das Zeichen wird nach links verschoben. Das Carryflag wird
									;an den Ausgang weitergegeben
	btfsc STATUS,C				;Prfen, welchen Wert das Carryflag hat
	bsf SI_DAT					;Carryflag = 1 -> Pin SI_DAT auf Highpegel setzen
	btfss STATUS,C				;Nochmal prfen, da immer nur ein Befehl bersprungen wird
	bcf SI_DAT					;Carryflag = 0 -> Pin SI_DAT auf Lowpegel setzen
	nop							;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	nop
	bsf SI_CLK					;mit dieser steigenden Flanke werden die Daten bernommen
	nop							;kurze Wartezeit
	nop
	bcf SI_CLK					;negative Flanke des Taktes
	decfsz SHIFT_CNT,F			;ein bit wurde bertragen -> Zhler um 1 verringern
	goto shiftDaten				;nchstes bit bertragen
	bcf DISP_EN					;Enable auf low setzen, mit der negativen Flanke werden Daten bertragen 
	bcf DISP_RS					;bertragung beendet Pin RS auf Low zurcksetzen
	bcf SI_DAT					;Pin SI_DAT auf Low zurcksetzen
	movlw d'150'				;geforderte Wartezeit (min. 26,3 us)
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1
	return						;Rcksprung aus dem Unterprogramm

wait
	movwf LOOP_1			;
wait_loop
	movlw d'250'
	movwf TEMP_13
loop_16
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	decfsz TEMP_13
	goto loop_16
	decfsz LOOP_1
	goto wait_loop
	return

;*******************************************************************************************
;	PAGE 3, 	bsf PCLATH,3				;Select Page 3
;				bsf PCLATH,4
;*******************************************************************************************
	ORG 0x1800					;PAGE 3

log_tabelle
	movlw 0x1D				;vorbereiten fuer Log-Tabelle im Bereich 0x1D00..0x1DFF
	movwf PCLATH			;
	movfw AD_DA_L 
	call log_tab_L
	movwf ZAHL_B0			;L-byte n
	movlw 0x1F				;vorbereiten fuer Log-Tabelle im Bereich 0x1F00..0x1FFF
	movwf PCLATH			;
	movfw AD_DA_L 
	call log_tab_H
	movwf ZAHL_B1			;H-byte n
	incfsz AD_DA_L
	goto $+6
	movlw 0x00
	movwf TEMP_12			;L-byte n+1
	movlw 0x00
	movwf TEMP_13			;H-byte n+1
	return
	movlw 0x1D				;vorbereiten fuer Log-Tabelle im Bereich 0x1D00..0x1DFF
	movwf PCLATH			;
	movfw AD_DA_L 
	call log_tab_L
	movwf TEMP_12			;L-byte n+1
	movlw 0x1F				;vorbereiten fuer Log-Tabelle im Bereich 0x1F00..0x1FFF
	movwf PCLATH			;
	movfw AD_DA_L 
	call log_tab_H
	movwf TEMP_13			;H-byte n+1
	return



prep_vco_wob
	clrf INTCON					;Interups verbieten	
	clrf T1CON	
	clrf MESS_END				;lsche Register, Bit 0: berlauf 10-Bit Zhler inc/dec wob, Bit 1: dt<0,2s, Bit 3: Fehler allgemein
								; Bit 4: 1=Fehler Fu/Dek zu hoch, 0=Fehler Fu/dt zu klein, Bit 5-6 frei, Bit 7: dt abgelaufen

	movfw FREQREG_L				;lade Zahl_A mit Fu (Wobbel/VCO)
	movwf ZAHL_A0
	movfw FREQREG_M
	movwf ZAHL_A1
	movfw FREQREG_H
	movwf ZAHL_A2
	clrf ZAHL_A3

	movlw 0x01
	subwf DEKADE,w
	btfss STATUS,Z
	goto $+3
	movlw 0x01					;1 Dekade, lade Zahl_B mit Wert 1
	goto $+8	
	movlw 0x02
	subwf DEKADE,w
	btfss STATUS,Z
	goto $+3
	movlw 0x0A					;2 Dekaden, lade Zahl_B mit Wert 10
	goto $+2	
	movlw 0x64					;3 Dekaden, lade Zahl_B mit Wert 100

	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call mul32					;Multipliziere ZAHL_A mit ZAHL_B, das Ergebnis steht in den Registern ZAHL_C0 bis ZAHL_C7
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	clrf ZAHL_A0				;lade Zahl_A mit Wert 2^28 = 0x10000000
	clrf ZAHL_A1
	clrf ZAHL_A2
	movlw 0x10
	movwf ZAHL_A3
	movfw ZAHL_C0				;lade Zahl_B mit Fu x10 (Wobbel/VCO), steht noch in ZAHL_C
	movwf ZAHL_B0
	movfw ZAHL_C1
	movwf ZAHL_B1
	movfw ZAHL_C2
	movwf ZAHL_B2
	movfw ZAHL_C3
	movwf ZAHL_B3

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call mul32					;Multipliziere ZAHL_A mit ZAHL_B, das Ergebnis steht in den Registern ZAHL_C0 bis ZAHL_C7
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	movfw DDS_MCLK_0			;Lade DDS_MCLK in ZAHL_A Register
	movwf ZAHL_A0
	movfw DDS_MCLK_1
	movwf ZAHL_A1
	movfw DDS_MCLK_2
	movwf ZAHL_A2
	movfw DDS_MCLK_3
	movwf ZAHL_A3
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, der Rest in ZAHL_B
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	bsf MESS_END,3				;setze vorsichtshalber Fehler Bits allgemein und fr zu hohe Fu bzw. zu viele Dekaden
	bsf MESS_END,4

	movlw 0x00					;prfen ob Wert fr obere Frequenz < Maximalwert von 2hoch28/2 ist (0x08000000)
	movwf TEMP_10				;  Wert fr obere Frequenz Fo steht noch in ZAHL_C
	movlw 0x00
	movwf TEMP_11
	movlw 0x00
	movwf TEMP_12
	movlw 0x08
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call test_kleiner			;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4
	btfsc STATUS,C
	return						;obere Frequenz >= Maximalwert von 2hoch28/2, zurck zur Schleife mit Fehlermeldung

	bcf MESS_END,3				;obere Frequenz < Maximalwert von 2hoch28/2, lsche Fehler Bit fr zu hohe Fu bzw. zu viele Dekaden
	bcf MESS_END,4				;

	movfw KENNLIN				;prfen, ob Kennlinie Linear oder Logarithmisch gewhlt ist
	btfsc STATUS,Z
	goto calc_delta_lin			;Kennlinie =0, d.h. linear, weiter bei Berechnungen fr lineare Kennlinie

	movlw 0x01					;Kennlinie >0<, d.h. logarithmisch, prfen ob 1, 2 oder 3 Dekaden gewhlt sind
	subwf DEKADE,w
	btfss STATUS,Z
	goto $+3
	call copy_fd1dds			;1 Dekade ist gewhlt, kopiere ZAHL_C Register (Fo Wert DDS) in FD1DDS Register
	goto prep_wobbel			;weiter bei prep_wobbel (war return => zurck zur Hauptschleife)
	movlw 0x02					;
	subwf DEKADE,w
	btfss STATUS,Z
	goto $+3
	call copy_fd2dds			;2 Dekaden sind gewhlt, kopiere ZAHL_C Register (Fo Wert DDS) in FD2DDS Register
	goto $+2
	call copy_fd3dds			;3 Dekaden sind gewhlt, kopiere ZAHL_C Register (Fo Wert DDS) in FD3DDS Register

	movlw d'10'					;durch 10 teilen: Lade Wert 10 in ZAHL_A Register
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, die Nachkommastelle in ZAHL_B0
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	movlw d'5'					;prfen, ob die Nachkommastelle grer oder gleich 5 ist
	subwf ZAHL_B0,w
	btfsc STATUS,C
	incfsz ZAHL_C0				;Nachkommastelle ist grer oder gleich 5 => erhhe Frequenzregister_0 um 1
	goto $+6					;
	incfsz ZAHL_C1				;es ist ein berlauf aufgetreten, erhhe Frequenzregister_1
	goto $+4					;
	incfsz ZAHL_C2				;es ist ein berlauf aufgetreten, erhhe Frequenzregister_2
	goto $+2					;
	incf ZAHL_C3				;es ist ein berlauf aufgetreten, erhhe Frequenzregister_3	

	movlw 0x02					;prfen ob 2 oder 3 Dekaden gewhlt sind
	subwf DEKADE,w
	btfss STATUS,Z
	goto $+3
	call copy_fd1dds			;2 Dekaden sind gewhlt, kopiere ZAHL_C Register (Fo Wert DDS / 10) in FD1DDS Register
	goto prep_wobbel			;weiter bei prep_wobbel (war return => zurck zur Hauptschleife)
	call copy_fd2dds			;3 Dekaden sind gewhlt, kopiere ZAHL_C Register (Fo Wert DDS / 10) in FD2DDS Register

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, die Nachkommastelle in ZAHL_B0
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	movlw d'5'					;prfen, ob die Nachkommastelle grer oder gleich 5 ist
	subwf ZAHL_B0,w
	btfsc STATUS,C
	incfsz ZAHL_C0				;Nachkommastelle ist grer oder gleich 5 => erhhe Frequenzregister_0 um 1
	goto $+6					;
	incfsz ZAHL_C1				;es ist ein berlauf aufgetreten, erhhe Frequenzregister_1
	goto $+4					;
	incfsz ZAHL_C2				;es ist ein berlauf aufgetreten, erhhe Frequenzregister_2
	goto $+2					;
	incf ZAHL_C3				;es ist ein berlauf aufgetreten, erhhe Frequenzregister_3	

	call copy_fd1dds			;3 Dekaden sind gewhlt, kopiere ZAHL_C Register (Fo Wert DDS / 100) in FD1DDS Register

	goto prep_wobbel			;weiter bei prep_wobbel
	

calc_delta_lin					;ZAHL_C - ZAHL_B = ZAHL_C
	call copy_fd3dds			;speichern des DDS Wertes fr Fo im FD3DDS Register

	clrf ZAHL_A1				;Berechnung DDS Wert fr Fu: 1 Dek.: Fo / 10,  2 Dek.: Fo / 100,  3 Dek.: Fo / 1000
	clrf ZAHL_A2				;lsche ZAHL_A Register
	clrf ZAHL_A3

	movlw d'1'					;prfen, ob 1 Dekade gewhlt ist
	subwf DEKADE,w
	btfss STATUS,Z
	goto $+4					;Anzahl Dekaden >1<, weiter bei nchster Prfung
	movlw 0x0A					;Anzahl Dekaden =1, Lade Wert 10 (0x0A) in Nenner Register fr die Division
	movwf ZAHL_A0
	goto calc_fu_value
	movlw d'2'					;prfen, ob 2 Dekaden gewhlt sind
	subwf DEKADE,w
	btfss STATUS,Z
	goto $+4					;Anzahl Dekaden >2<
	movlw 0x64					;Anzahl Dekaden =2, Lade Wert 100 (0x64) in Nenner Register fr die Division
	movwf ZAHL_A0
	goto calc_fu_value
	movlw 0xE8					;Anzahl Dekaden =3, Lade Wert 1000 (0x03E8) in Nenner Register fr die Division
	movwf ZAHL_A0
	movlw 0x03
	movwf ZAHL_A1

calc_fu_value					;Teile DDS Wert fr Fo (ZAHL_C) durch 10, 100 oder 1000 (ZAHL_A)
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call div32					;Teile ZAHL_C durch ZAHL_A, das ganzzahlige Ergebnis steht in ZAHL_C, der Rest in ZAHL_B
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	call copy_fd1dds			;speichern des DDS Wertes fr Fu (ZAHL_C) im FD1DDS Register

								;Berechnung der Differenz zwischen den DDS Werten fr Fo und Fu: Fo - Fu = Fd
	movfw ZAHL_C0				;ZAHL_C - ZAHL_B = ZAHL_C
	movwf ZAHL_B0				;Kopiere ZAHL_C nach ZAHL_B
	movfw ZAHL_C1
	movwf ZAHL_B1
	movfw ZAHL_C2
	movwf ZAHL_B2
	movfw ZAHL_C3
	movwf ZAHL_B3				

	bsf STATUS,RP0				;Select BANK_1, Kopiere DDS Wert fr Fo von FD3DDS Registern nach ZAHL_C
	movfw FD3DDS_0
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_C0
	bsf STATUS,RP0				;Select BANK_1
	movfw FD3DDS_1
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_C1
	bsf STATUS,RP0				;Select BANK_1
	movfw FD3DDS_2
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_C2
	bsf STATUS,RP0				;Select BANK_1
	movfw FD3DDS_3
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_C3
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call sub32					;Subtrahiere ZAHL_B von ZAHL_C, das Ergebnis steht in ZAHL_C (Fd)
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	movfw ZAHL_C0				;( Fo - Fu ) mit 4096 (0x1000) multiplizieren,
	movwf ZAHL_A0				;kopiere ( Fo - Fu ) in Multiplikationsregister ZAHL_A
	movfw ZAHL_C1
	movwf ZAHL_A1
	movfw ZAHL_C2
	movwf ZAHL_A2
	movfw ZAHL_C3
	movwf ZAHL_A3
	clrf ZAHL_B0				;lade Wert 4096 (0x1000) in zweites Multiplikationsregister ZAHL_B
	movlw 0x10
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call mul32					;Multipliziere ZAHL_A mit ZAHL_B, das Ergebnis steht in den Registern ZAHL_C0 bis ZAHL_C7
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4				;Ergebnis ( Fo - Fu ) x 4096 steht in ZAHL_C

								;Ergbnis ( Fo - Fu ) x 4096 durch 1023 teilen
	movlw 0xFF					;lade Zahl_A mit Wert 1023 = 0x000003FF
	movwf ZAHL_A0
	movlw 0x03
	movwf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, der Rest in ZAHL_B
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	call copy_fd2dds			;kopiere Ergebnis der Berechnungen ( Fo - Fu ) x 4096 /1023 in FD2DDS Register


prep_wobbel					;Unterprogramm zur Vorbereitung der bentigten Parameter
	btfss PROGRAM,2			;Prfen ob Programm VCO (=3 => 00000011) oder Wobbel (=4 => 00000100) gewhlt ist (Bit 2 prfen) 
	return					;Bit 2 ist nicht gesetzt, d.h. VCO ist gewhlt =>zurck zur Hauptschleife

	clrf AD_DA_L			;lschen der Register
	clrf AD_DA_H
	bsf STATUS,RP0			;Select BANK_1
	clrf WOB_COUNT_L		;lschen der Register, diese werden als Zhler fr die Schrittweite benutzt
	clrf WOB_COUNT_H

	movfw WOB_STEP			;kopiere Wert in Temp Register: Schritte/Durchlauf: 1=1024, 2=512, 3=256, 4=128, 5=64, 6=32
	movwf TEMP_0
	clrf WOB_STEPW			;lsche Schrittweiten Register, setze Carry Bit und schiebe Schrittweiten Register nach links bis Temp Register =0
	bsf STATUS,C			;setze Carry Bit
	rlf WOB_STEPW			;1x schieben: 00000001 =1, 2x schieben: 00000010 =2, 3x schieben: 00000100 =4,
	decfsz TEMP_0			;4x schieben: 00001000 =8, 5x schieben: 00010000 =16, 6x schieben: 00100000 =32,
	goto $-2				;Zhler noch nicht 0, zurck zur Schleife


							;dt = Zeit/Schritt: 0=10ms, 1=20ms, 2=50ms, 3=100ms, 4=200ms, 5=500ms, 6=1000ms, 7=2000ms
							;fr das gewhlte dt werden die Parameter aus der Tabelle im Bereich 0x1E01..0x1E3C geholt.
							;die erforderlichen Takte Ndt (Timer1, Prescaler 1:1) ergeben sich zu:   
para_delta_t				; Ndt = 65536 x (WOB_ZEIT_H - 1) + 256 x (256 - WOB_ZEIT_LN + (256 - WOB_ZEIT_LN1) x (WOB_LOOP - 1)) / WOB_LOOP
	movlw 0x1E				;vorbereiten fuer delta t Tabelle im Bereich 0x1E01..0x1E3C
	movwf PCLATH			;
	movfw WOB_ZEIT			;hole WOB_ZEIT nach W (0-7)
	addlw d'0'				;addiere Konstante zu W, der entsprechende Wert steht in der Tabelle in Zeile (WOB_ZEIT + Konstante)
	call delta_t_tab		;Aufruf des Unterprogramms
	movwf WOB_ZEIT_H
	movfw WOB_ZEIT
	addlw d'11'
	call delta_t_tab
	movwf WOB_ZEIT_LN1
	movwf WOB_ZEIT_LN
	movfw WOB_ZEIT
	addlw d'22'
	call delta_t_tab
	movwf WOB_LOOP
	addlw 0xFF				;wenn WOB_LOOP = 1, muss WOB_ZEIT_LN1 um 1 erhht werden.
	btfss STATUS,Z
	incf WOB_ZEIT_LN1

	movfw WOB_ZEIT
	addlw d'33'
	call delta_t_tab
	movwf DELTAP_H
	movwf TEMP_2
	movfw WOB_ZEIT
	addlw d'44'
	call delta_t_tab
	movwf DELTAP_M
	movwf TEMP_1
	movfw WOB_ZEIT
	addlw d'55'
	call delta_t_tab
	movwf DELTAP_L
	movwf TEMP_0

	movfw WOB_STEP
	movwf TEMP_3
	bcf STATUS,C
	rrf TEMP_2
	rrf TEMP_1
	rrf TEMP_0
	decfsz TEMP_3
	goto $-5
							;prfen, ob TEMP_0-2 < Wert 782 (0x00030E) (entspricht ca.100ms fr einen Balken in graphischer Anzeige)
	movlw 0x00				;hole 0x00 ins W-Reg.								= DELTAP / 2 hoch WOB_STEP 
	subwf TEMP_2,w			;subtrahiere W-Reg. von TEMP_2
	btfss STATUS,C			;prfen, ob Ergebnis negativ, d.h. <0
	goto set_messend2		;Ergebnis negativ, setze Bit 2 in MESS_END Register
	btfss STATUS,Z			;prfen, ob Ergebnis = 0
	goto test_messend1		;Ergebnis positiv, weiter bei test_messend1
	movlw 0x03				;Ergebnis = 0, nchst kleineres Byte prfen, hole 0x03 ins W-Reg.
	subwf TEMP_1,w			;subtrahiere W-Reg. von TEMP_1
	btfss STATUS,C			;prfen, ob Ergebnis negativ, d.h. <0
	goto set_messend2		;Ergebnis negativ, setze Bit 2 in MESS_END Register
	btfss STATUS,Z			;prfen, ob Ergebnis = 0
	goto test_messend1		;Ergebnis positiv, weiter bei test_messend1
	movlw 0x0E				;Ergebnis = 0, nchst kleineres Byte prfen, hole 0x0E ins W-Reg.
	subwf TEMP_0,w			;subtrahiere W-Reg. von TEMP_0
	btfsc STATUS,C			;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	goto test_messend1		;TEMP >= Wert 782,  weiter bei test_messend1
set_messend2
	bcf STATUS,RP0			;Select BANK_0	
	bsf MESS_END,1			;TEMP < Wert 782, setze Bit 1+2 in MESS_END Register
	bsf MESS_END,2
	goto set_delta_t

test_messend1				;wenn Wobbel Auto und Schrittweite <200ms gewhlt ist, wird andere Frequenzanzeige genutzt
							;   hierfr wird Bit 1 in Register MESS_END gesetzt
	btfsc WOB_M_A,0			;prfen ob Wobbel Manuell (1) oder Automatisch (0) gewhlt ist
	goto set_delta_t		;Manuell ist gewhlt, weiter im Programm
	movlw d'6'				;Auto ist gewhlt, prfen ob Zeitschrittweite < 6 (<200ms)
	subwf WOB_ZEIT,w
	btfsc STATUS,C
	goto set_delta_t		;Zeitschrittweite >= 6 (>=200ms), weiter im Programm
	bcf STATUS,RP0			;Select BANK_0	
	bsf MESS_END,1			;setze Statusbit fr Zeitschrittweite <200ms und Wobbel Auto

set_delta_t
	bsf STATUS,RP0			;Select BANK_1
	movfw WOB_ZEIT_H		;Zhler auf gewhltes delta t einstellen
	bcf STATUS,RP0			;Select BANK_0	
	movwf COUNT_H_S
	clrf T1CON
	bsf STATUS,RP0			;Select BANK_1	
	movfw WOB_LOOP
	bcf STATUS,RP0			;Select BANK_0	
	movwf COUNT_ADJ
	clrf TMR1L
	bsf STATUS,RP0			;Select BANK_1	
	movfw WOB_ZEIT_LN1
	bcf STATUS,RP0			;Select BANK_0	
	movwf TMR1H	

	clrf STAT_INT
	bsf STAT_INT,3
	bsf STATUS,RP0			;Select BANK_1
	movlw b'01000000'		;Pull Up Port B enabled, RB0 Interrupt bei steigender Flanke, TMR0 intern, Prescaler 1:2
	movwf OPTION_REG		;  prescaler 
	movlw b'00000001'		;Timer 1 Interrupt erlauben
	movwf PIE1 
	bcf STATUS,RP0			;Select BANK_0
	clrf T1CON				;TMR1 Off (Bit0), Prescaler 1:1
	bsf INTCON,PEIE			;periphere Interrupts erlauben
	bsf INTCON,GIE			;globale Interrupts erlauben


	bsf STATUS,RP0			;Select BANK_1	
	btfss WOB_M_A,1			;prfen, ob Burst gewhlt ist
	return					;Burst ist nicht gewhlt, zurck zur Hauptprogrammschleife
	bcf STATUS,RP0			;Select BANK_0

					;Programmteil zum Berechnen der Basisdaten fr Wobbel Burst
					;prfen, ob delta t mindestens 2x Periodenlnge von Fu
	movlw 0x00				;lade Wert 20.000.000 (=> 0x01312D00) in ZAHL_C Register und teile durch Fu (x10)
	movwf ZAHL_C0			;	
	movlw 0x2D				;
	movwf ZAHL_C1
	movlw 0x31
	movwf ZAHL_C2
	movlw 0x01
	movwf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7

	movfw FREQREG_L			;kopiere Fu (x10) in ZAHL_A Register
	movwf ZAHL_A0
	movfw FREQREG_M
	movwf ZAHL_A1
	movfw FREQREG_H
	movwf ZAHL_A2
	clrf ZAHL_A3

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, der Rest in ZAHL_B
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	bsf MESS_END,3				;setze vorsichtshalber Fehler Bits allgemein und fr zu kleine Fu bzw. zu kleines delta t
	bcf MESS_END,4

								;prfen ob Periode von Fu (Tfu) < delta t /2 ist: subtrahiere delta t /2 von Tfu
	bsf STATUS,RP0				;Select BANK_1 
	movfw DELTAP_L
	bcf STATUS,RP0				;Select BANK_0 
	movwf TEMP_10				;
	bsf STATUS,RP0				;Select BANK_1 
	movfw DELTAP_M
	bcf STATUS,RP0				;Select BANK_0 
	movwf TEMP_11
	bsf STATUS,RP0				;Select BANK_1 
	movfw DELTAP_H
	bcf STATUS,RP0				;Select BANK_0 
	movwf TEMP_12
	movlw 0x00
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call test_kleiner			;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4
	btfss STATUS,C
	goto $+2					;Periode von Fu (Tfu) < delta t /2
	return						;Periode von Fu (Tfu) >= delta t /2, zurck zur Schleife mit Fehlermeldung

	bcf MESS_END,3				;delta t /2 >= Periode von Fu, lsche Fehler Bit allgemein und fr zu kleine Fu bzw. zu kleines delta t

	clrf ZAHL_C0				;lade Wert 0x3D090000000000 in ZAHL_C Register (= 2.000.000 x 2^28 x 32)
	clrf ZAHL_C1				;2.000.000 = Takte/sek Timer 0;  2^28 = Konstante;  32 = max Anzahl Perioden
	clrf ZAHL_C2				;
	clrf ZAHL_C3				;
	movlw 0x00
	movwf ZAHL_C4
	movlw 0x09
	movwf ZAHL_C5
	movlw 0x3D
	movwf ZAHL_C6
	clrf ZAHL_C7

	movfw DDS_MCLK_0			;Lade DDS_MCLK in ZAHL_A Register
	movwf ZAHL_A0
	movfw DDS_MCLK_1
	movwf ZAHL_A1
	movfw DDS_MCLK_2
	movwf ZAHL_A2
	movfw DDS_MCLK_3
	movwf ZAHL_A3
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, der Rest in ZAHL_B
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	movfw ZAHL_C0				;kopieren des Ergebnisses in die BU_VAL Register
	bsf STATUS,RP0				;Select BANK_1
	movwf BU_VAL_0
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C1
	bsf STATUS,RP0				;Select BANK_1
	movwf BU_VAL_1
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C2
	bsf STATUS,RP0				;Select BANK_1
	movwf BU_VAL_2
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C3
	bsf STATUS,RP0				;Select BANK_1
	movwf BU_VAL_3
	bcf STATUS,RP0				;Select BANK_0

	return

vco_wob_lin					;Berechnung: A/D-Wert * Fd (DDS-Format)/ 4096
	movfw AD_DA_L			;kopiere Low Byte des A/D-Wertes in Multiplikationsregister ZAHL_A0
	movwf ZAHL_A0
	movfw AD_DA_H			;kopiere High Byte des A/D-Wertes in Multiplikationsregister ZAHL_A1
	movwf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
							;kopiere Fd (DDS-Format) von FD2DDS Registern nach Multiplikationsregister ZAHL_B
	bsf STATUS,RP0			;Select BANK_1
	movfw FD2DDS_0
	bcf STATUS,RP0			;Select BANK_0
	movwf ZAHL_B0
	bsf STATUS,RP0			;Select BANK_1
	movfw FD2DDS_1
	bcf STATUS,RP0			;Select BANK_0
	movwf ZAHL_B1
	bsf STATUS,RP0			;Select BANK_1
	movfw FD2DDS_2
	bcf STATUS,RP0			;Select BANK_0
	movwf ZAHL_B2
	bsf STATUS,RP0			;Select BANK_1
	movfw FD2DDS_3
	bcf STATUS,RP0			;Select BANK_0
	movwf ZAHL_B3
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4
	call mul32				;ZAHL_A * ZAHL_B = ZAHL_C, (A/D-Wert) * (Fd) = (A/D-Wert * Fd), Ergebnis steht in ZAHL_C
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4

	call div_2h12			;teilen durch 2^12, das Ergebnis A/D-Wert * Fd / 2^12 steht in Registern ZAHL_C0 - ZAHL_C3

							;Berechnung: A/D-Wert * Fd / 2^16 + Fu
							;kopiere DDS Wert fr Fu in Additionsregister ZAHL_B
	bsf STATUS,RP0			;Select BANK_1
	movfw FD1DDS_0
	bcf STATUS,RP0			;Select BANK_0
	movwf ZAHL_B0
	bsf STATUS,RP0			;Select BANK_1
	movfw FD1DDS_1
	bcf STATUS,RP0			;Select BANK_0
	movwf ZAHL_B1
	bsf STATUS,RP0			;Select BANK_1
	movfw FD1DDS_2
	bcf STATUS,RP0			;Select BANK_0
	movwf ZAHL_B2
	bsf STATUS,RP0			;Select BANK_1
	movfw FD1DDS_3
	bcf STATUS,RP0			;Select BANK_0
	movwf ZAHL_B3
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4
	call add32				;ZAHL_C + ZAHL_B = ZAHL_C, (A/D-Wert * Fd / 1023) + (Fu) = (A/D-Wert * Fd / 1023 + Fu)
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	return

calc_frequenz					;Berechnung der Frequenz aus dem DDS Frequenzwert: F = DDS Frequenzwert x 10 x MCLK / 2^28
	movlw 0x0A					;Lade Wert 10 in ZAHL_A Register
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3

	movfw ZAHL_C0				;Lade DDS Wert fr obere Frequenz in ZAHL_B Register
	movwf ZAHL_B0
	movfw ZAHL_C1
	movwf ZAHL_B1
	movfw ZAHL_C2
	movwf ZAHL_B2
	movfw ZAHL_C3
	movwf ZAHL_B3

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call mul32					;Multipliziere ZAHL_A mit ZAHL_B, das Ergebnis steht in den Registern ZAHL_C0 bis ZAHL_C7
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4


	movfw DDS_MCLK_0			;Lade DDS_MCLK in ZAHL_A Register
	movwf ZAHL_A0
	movfw DDS_MCLK_1
	movwf ZAHL_A1
	movfw DDS_MCLK_2
	movwf ZAHL_A2
	movfw DDS_MCLK_3
	movwf ZAHL_A3

	movfw ZAHL_C0				;Lade DDS Frequenzwert x 10 in ZAHL_B Register
	movwf ZAHL_B0
	movfw ZAHL_C1
	movwf ZAHL_B1
	movfw ZAHL_C2
	movwf ZAHL_B2
	movfw ZAHL_C3
	movwf ZAHL_B3

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call mul32					;Multipliziere ZAHL_A mit ZAHL_B, das Ergebnis steht in den Registern ZAHL_C0 bis ZAHL_C7
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4				;Ergebnis: DDS Frequenzwert x 10 x MCLK

	movlw d'4'					;Teilen von ZAHL_C durch 2^28 in 2 Schritten: 1.: teilen durch 16, 2.: teilen durch 2^24
	movwf LOOP_1
div_16_loop
	bcf STATUS,C				;Lsche Carry Bit, damit es beim Rotieren nicht strt
	rrf ZAHL_C6					;teilen durch 16; 4 x nach rechts verschieben
	rrf ZAHL_C5
	rrf ZAHL_C4
	rrf ZAHL_C3
	rrf ZAHL_C2
	decfsz LOOP_1				;reduziere Zhler um 1 und prfe ob 0
	goto div_16_loop			;noch nicht 4x geschoben, zurck zur Schleife
								;das durch 2^28 geteilte Ergebnis steht in ZAHL_C3 (L) bis ZAHL_C6 (H), Rest in ZAHL_C2

	btfss ZAHL_C2,7				;wenn Rest >=128 ist, d.h. >=0,5 ist, erhhe ZAHL_C3 um 1
	goto $+8					;Rest (ZAHL_C2) < 128, weiter bei kopieren des Ergebnis
	incfsz ZAHL_C3				;Rest (ZAHL_C2) >=128 erhhe Ergebnis (ZAHL_C3) um 1
	goto $+6
	incfsz ZAHL_C4				;es ist ein berlauf aufgetreten, erhhe um 1 und prfe ob 0
	goto $+4
	incfsz ZAHL_C5				;es ist ein berlauf aufgetreten, erhhe um 1 und prfe ob 0
	goto $+2
	incf ZAHL_C6				;es ist ein berlauf aufgetreten, erhhe um 1

	movfw ZAHL_C3				;kopieren der Register
	movwf ZAHL_C0
	movfw ZAHL_C4
	movwf ZAHL_C1
	movfw ZAHL_C5
	movwf ZAHL_C2
	movfw ZAHL_C6
	movwf ZAHL_C3

	return

calc_burst_length				;Unterprogramm zur Berechnung der Burst Lnge in Takten von TMR0 (prescaler 2:1)

	bsf STATUS,RP0				;Select BANK_1
	movfw BU_VAL_0				;kopiere BU_VAL Register in die Zhlerregister fr die Division (ZAHL_C)
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_C0
	bsf STATUS,RP0				;Select BANK_1
	movfw BU_VAL_1
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_C1
	bsf STATUS,RP0				;Select BANK_1
	movfw BU_VAL_2
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_C2
	bsf STATUS,RP0				;Select BANK_1
	movfw BU_VAL_3
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7

	movfw DDS_FREQREG_0			;kopiere den DDS Wert fr die Frequenz in das Nennerregister fr die Division (ZAHL_A)
	movwf ZAHL_A0
	movfw DDS_FREQREG_1
	movwf ZAHL_A1
	movfw DDS_FREQREG_2
	movwf ZAHL_A2
	movfw DDS_FREQREG_3
	movwf ZAHL_A3

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, der Rest in ZAHL_B
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	bsf STATUS,RP0				;Select BANK_1	
	btfsc WOB_M_A,0				;prfen, ob Manuell gewhlt ist
	goto werte_timer0			;Manuell ist gewhlt, es wird der Wert fr 16 Perioden der Frequenz verwendet

t_burst
	bsf STATUS,RP0				;Select BANK_1
	movfw DELTAP_L				;wenn ZAHL_C >= delta t /2 ist, d.h. >=0,5 ist, teile ZAHL_C durch 2
	bcf STATUS,RP0				;Select BANK_0
	movwf TEMP_10
	bsf STATUS,RP0				;Select BANK_1
	movfw DELTAP_M
	bcf STATUS,RP0				;Select BANK_0
	movwf TEMP_11
	bsf STATUS,RP0				;Select BANK_1
	movfw DELTAP_H
	bcf STATUS,RP0				;Select BANK_0
	movwf TEMP_12
	movlw 0x00
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call test_kleiner			;Unterprogramm zum prfen, ob ZAHL_C < Wert in TEMP/W-Reg.
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4
	btfss STATUS,C
	goto werte_timer0			;ZAHL_C < delta t /2, weiter im Unterprogramm
	bcf STATUS,C				;ZAHL_C >= delta t /2, teile ZAHL_C durch 2 (nach rechts verschieben)
	rrf ZAHL_C2
	rrf ZAHL_C1
	rrf ZAHL_C0
	goto t_burst
	
werte_timer0
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C0				;Berechnete Takte Timer0 um 7 reduzieren (Zeitdifferenz): 7 - Zahl C0
	sublw 0x07
	movwf ZAHL_C0				;kopiere Wert zurck nach Zahl C0
	btfsc STATUS,C				;prfe ob Ergebnis negativ
	goto $+4					;positiv, weiter
	incfsz ZAHL_C1,f			;negativ, erhhe Zahl C1 um 1 und prfe auf berlauf
	goto $+2					;kein berlauf, weiter
	incf ZAHL_C2,f	

	movfw ZAHL_C1				;
	sublw 0x00
	movwf ZAHL_C1				;kopiere Wert zurck nach Zahl C1
	btfsc STATUS,C				;prfe ob Ergebnis negativ
	goto $+2					;positiv, weiter
	incf ZAHL_C2,f				;negativ, erhhe Zahl C2 um 1

	movfw ZAHL_C2				;
	sublw 0x00
	movwf ZAHL_C2				;kopiere Wert zurck nach Zahl C2

	movfw ZAHL_C0				;kopieren der Register
	bsf STATUS,RP0				;Select BANK_1
	movwf TBURST_L
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C1
	bsf STATUS,RP0				;Select BANK_1
	movwf TBURST_M
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C2
	bsf STATUS,RP0				;Select BANK_1
	movwf TBURST_H
	bcf STATUS,RP0				;Select BANK_0
	return


copy_fd1dds
	movfw ZAHL_C0
	bsf STATUS,RP0				;Select BANK_1
	movwf FD1DDS_0
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C1
	bsf STATUS,RP0				;Select BANK_1
	movwf FD1DDS_1
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C2
	bsf STATUS,RP0				;Select BANK_1
	movwf FD1DDS_2
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C3
	bsf STATUS,RP0				;Select BANK_1
	movwf FD1DDS_3
	bcf STATUS,RP0				;Select BANK_0
	return

copy_fd2dds
	movfw ZAHL_C0
	bsf STATUS,RP0				;Select BANK_1
	movwf FD2DDS_0
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C1
	bsf STATUS,RP0				;Select BANK_1
	movwf FD2DDS_1
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C2
	bsf STATUS,RP0				;Select BANK_1
	movwf FD2DDS_2
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C3
	bsf STATUS,RP0				;Select BANK_1
	movwf FD2DDS_3
	bcf STATUS,RP0				;Select BANK_0
	return

copy_fd3dds
	movfw ZAHL_C0
	bsf STATUS,RP0				;Select BANK_1
	movwf FD3DDS_0
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C1
	bsf STATUS,RP0				;Select BANK_1
	movwf FD3DDS_1
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C2
	bsf STATUS,RP0				;Select BANK_1
	movwf FD3DDS_2
	bcf STATUS,RP0				;Select BANK_0
	movfw ZAHL_C3
	bsf STATUS,RP0				;Select BANK_1
	movwf FD3DDS_3
	bcf STATUS,RP0				;Select BANK_0
	return

log_mul_dek1				;Unterprogramm zum Kopieren des Multiplikators fr die aktuelle Dekade bei Logarithmischer Kennlinie 
	movlw d'1'					;prfen ob die aktuelle Dekade =1
	bsf STATUS,RP0				;Select BANK_1
	subwf DEK_AKTUELL,w
	bcf STATUS,RP0				;Select BANK_0
	btfss STATUS,Z
	goto log_mul_dek2			;aktuelle Dekade >1<, weiter bei nchster Prfung

	bsf STATUS,RP0				;Select BANK_1
	movfw FD1DDS_0
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A0
	bsf STATUS,RP0				;Select BANK_1
	movfw FD1DDS_1
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A1
	bsf STATUS,RP0				;Select BANK_1
	movfw FD1DDS_2
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A2
	bsf STATUS,RP0				;Select BANK_1
	movfw FD1DDS_3
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A3
	return

log_mul_dek2
	movlw d'2'					;prfen ob die aktuelle Dekade =2
	bsf STATUS,RP0				;Select BANK_1
	subwf DEK_AKTUELL,w
	bcf STATUS,RP0				;Select BANK_0
	btfss STATUS,Z
	goto log_mul_dek3			;aktuelle Dekade >2<, weiter bei nchster Prfung

	bsf STATUS,RP0				;Select BANK_1
	movfw FD2DDS_0
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A0
	bsf STATUS,RP0				;Select BANK_1
	movfw FD2DDS_1
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A1
	bsf STATUS,RP0				;Select BANK_1
	movfw FD2DDS_2
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A2
	bsf STATUS,RP0				;Select BANK_1
	movfw FD2DDS_3
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A3
	return

log_mul_dek3
	bsf STATUS,RP0				;Select BANK_1
	movfw FD3DDS_0
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A0
	bsf STATUS,RP0				;Select BANK_1
	movfw FD3DDS_1
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A1
	bsf STATUS,RP0				;Select BANK_1
	movfw FD3DDS_2
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A2
	bsf STATUS,RP0				;Select BANK_1
	movfw FD3DDS_3
	bcf STATUS,RP0				;Select BANK_0
	movwf ZAHL_A3
	return

div_2h16						;Teilen von ZAHL_C durch 2^16 (dez: 65536, hex: 0x010000)
								;das durch 2^16 geteilte Ergebnis steht in ZAHL_C2 (L) bis ZAHL_C5 (H), Rest in ZAHL_C1
	btfss ZAHL_C1,7				;  wenn Rest >=128 ist, d.h. >=0,5 ist, erhhe ZAHL_C2 um 1
	goto $+8					;Rest (ZAHL_C1) < 128, weiter bei kopieren des Ergebnis
	incfsz ZAHL_C2				;Rest (ZAHL_C1) >=128 erhhe Ergebnis (ZAHL_C2) um 1
	goto $+6
	incfsz ZAHL_C3				;es ist ein berlauf aufgetreten, erhhe um 1
	goto $+4
	incfsz ZAHL_C4				;es ist ein berlauf aufgetreten, erhhe um 1
	goto $+2
	incf ZAHL_C5				;es ist ein berlauf aufgetreten, erhhe um 1

	movfw ZAHL_C2				;kopieren der Register
	movwf ZAHL_C0
	movfw ZAHL_C3
	movwf ZAHL_C1
	movfw ZAHL_C4
	movwf ZAHL_C2
	movfw ZAHL_C5
	movwf ZAHL_C3
	return

div_2h12
	movlw d'4'					;Teilen von ZAHL_C durch 2^12 (=4096) in 2 Schritten: 1.: teilen durch 16, 
	movwf LOOP_1				;  2.: teilen durch 256
div_2h12_loop
	bcf STATUS,C				;Lsche Carry Bit, damit es beim Rotieren nicht strt
	rrf ZAHL_C4					;teilen durch 16: 4 x nach rechts verschieben
	rrf ZAHL_C3
	rrf ZAHL_C2
	rrf ZAHL_C1
	rrf ZAHL_C0
	decfsz LOOP_1				;reduziere Zhler um 1 und prfe ob 0
	goto div_2h12_loop			;noch nicht 4x geschoben, zurck zur Schleife
								;das durch 2^12 geteilte Ergebnis steht in ZAHL_C1 (L) bis ZAHL_C4 (H), Rest in ZAHL_C0

	btfss ZAHL_C0,7				;wenn Rest >=128 ist, d.h. >=0,5 ist, erhhe ZAHL_C1 um 1
	goto $+8					;Rest (ZAHL_C0) < 128, weiter bei kopieren des Ergebnis
	incfsz ZAHL_C1				;Rest (ZAHL_C0) >=128 erhhe Ergebnis (ZAHL_C1) um 1
	goto $+6
	incfsz ZAHL_C2				;es ist ein berlauf aufgetreten, erhhe um 1 und prfe ob 0
	goto $+4
	incfsz ZAHL_C3				;es ist ein berlauf aufgetreten, erhhe um 1 und prfe ob 0
	goto $+2
	incf ZAHL_C4				;es ist ein berlauf aufgetreten, erhhe um 1

	movfw ZAHL_C1				;kopieren der Register
	movwf ZAHL_C0
	movfw ZAHL_C2
	movwf ZAHL_C1
	movfw ZAHL_C3
	movwf ZAHL_C2
	movfw ZAHL_C4
	movwf ZAHL_C3

	return



runden_div32					;Unterprogramm zum ganzzahligen Runden des Ergebnisses der Division aus div32
	bcf STATUS,C				;lsche Carry Bit, damit es beim rotieren nicht strt
	rrf ZAHL_A3					;ZAHL_A 1x nach rechts schieben, d.h. durch 2 teilen 
	rrf ZAHL_A2
	rrf ZAHL_A1
	rrf ZAHL_A0

	movfw ZAHL_A3				;wenn Rest >= ZAHL_A/2 ist, d.h. >=0,5 ist, erhhe ZAHL_C um 1
	subwf ZAHL_B3,w				;subtrahiere ZAHL_A/2 von Rest
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Ergebnis negativ (STATUS,C=0), Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0  (STATUS,Z=1)
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movfw ZAHL_A2				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A2 ins W-Reg.
	subwf ZAHL_B2,w				;subtrahiere W-Reg. von ZAHL_B2
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movfw ZAHL_A1				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A1 ins W-Reg.
	subwf ZAHL_B1,w				;subtrahiere W-Reg. von ZAHL_B1
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movfw ZAHL_A0				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A0 ins W-Reg.
	subwf ZAHL_B0,w				;subtrahiere W-Reg. von ZAHL_B0
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Rest (ZAHL_B) < ZAHL_A/2, Rcksprung aus dem Unterprogramm
runden_div32_inc	
	incfsz ZAHL_C0				;Rest (ZAHL_B) >= ZAHL_A/2, erhhe ZAHL_C0 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incfsz ZAHL_C1				;berlauf vorhanden,  erhhe ZAHL_C1 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incfsz ZAHL_C2				;berlauf vorhanden,  erhhe ZAHL_C2 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incf ZAHL_C3				;berlauf vorhanden,  erhhe ZAHL_C3 um 1
	return						;Rcksprung aus dem Unterprogramm

convert_dds_winkel
	movfw ZAHL_A0
	movwf DISP_0
	movfw ZAHL_A1
	movwf DISP_1
	movfw ZAHL_A2
	movwf DISP_2
	movfw ZAHL_A3
	movwf DISP_3
	clrf DISP_4
	clrf DISP_5
	clrf DISP_6
	clrf DISP_7
	clrf DISP_8

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call convert_dig_hex		;Unterprogramm zum Umwandeln der einzelnen Digits (max.999.999.999) in eine 32-Bit Zahl
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	movfw ZAHL_C0				;kopieren der Register
	movwf DDS_PHASE_L
	movfw ZAHL_C1				;kopieren der Register
	movwf DDS_PHASE_H
	return

dds_settings				;Setzen aller DDS Einsstellungen
	call offset_korr		;Setzen der Offsetkorrektur ber DAC
	call dds_control_wave	;Setzen der Bits fr die Wellenform im dds_control_low Register
	call calc_dds_freq		;Unterprogramm zur Berechnung der Frequenzregisterdaten des DDS Chips AD9833
	call calc_dds_phase		;Unterprogramm zur Berechnung des DDS Phase Register aus dem Phasenwinkel
	bcf PCLATH,4			;select Page_1	
	call dds_freqformat
	call dds_pegel			;Schreiben des Signalpegels und setzen des Bit fr Sin/Sqr
	call write_dds_freq		;Unterprogramm zum Schreiben der Frequenz- u. Phasen Register, DDS Chips AD9833
	btfss TTL_ON_OFF,0		;Setze TTL Ausgang
	bcf TTL_ENABLE
	btfsc TTL_ON_OFF,0
	bsf TTL_ENABLE
	;bcf DDS_CONTROL_H,0		;lschen des Reset Bits	
	;call write_dds_control	;Aufruf des Unterprogramms zum Schreiben des Control Registers, DDS Chips AD9833
	bsf PCLATH,4			;select Page_3
	return

offset_korr
	bcf PCLATH,4			;Select Page 1	
	call read_offset		;Unterprogramm zum Lesen der Offsetwerte aus dem internen EEPROM
	bcf PCLATH,3			;Select Page 0
	call clear_ZAHL_C		;Lschen der Zahl C Register
	movfw OFFSET_100_L		;Kopiere die DAC Offset 100% Werte nach Zahl C
	movwf ZAHL_C0
	movfw OFFSET_100_H
	movwf ZAHL_C1
	movfw OFFSET_NULL_L		;Kopiere die DAC Offset 0% Werte nach Zahl B
	movwf ZAHL_B0
	movfw OFFSET_NULL_H
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call sub32				;Zahl_C - Zahl_B = Zahl_C, DAC Differenz zwischen 0 und 100%
	movfw OFFSET_VAL		;Kopiere Offset Wert (+/- 0-100%) nach Zahl A
	movwf ZAHL_A0
	bcf ZAHL_A0,7			;Lsche Bit 7 (Vorzeichen +/-)
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	movfw ZAHL_C0			;Kopiere DAC Differenz von Zahl C nach Zahl B
	movwf ZAHL_B0
	movfw ZAHL_C1
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32				;Zahl_A x Zahl_B = Zahl_C 
	movlw 0x64				;kopiere Wert 100 nach Zahl_A
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call div32				;Zahl_C / Zahl_A = Zahl_C, Rest in Zahl_B
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call runden_div32		;ganzzahliges Runden des Ergebnisses aus div32
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	movfw ZAHL_C0
	movwf ZAHL_B0
	movfw ZAHL_C1
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call clear_ZAHL_C		;Lschen der Zahl C Register
	movfw OFFSET_NULL_L
	movwf ZAHL_C0
	movfw OFFSET_NULL_H
	movwf ZAHL_C1
	btfss OFFSET_VAL,7
	call sub32
	btfsc OFFSET_VAL,7
	call add32
	bsf PCLATH,3			;Select Page 1
	
	movfw ZAHL_C0
	movwf DAC_L
	movfw ZAHL_C1
	movwf DAC_H
	bsf DAC_H,5				;Ausgang B (Offset korr.) wird angesprochen (Bit 5 wird spter noch 2x nach links verschoben)
	bsf DAC_H,3				;Gain = 1, d.h. 0-1023 entsprechen 2-2048 mV
	call send_dac_data		;Aufruf des Unterprogramms zum senden der Daten an den DAC
	bsf PCLATH,4			;Select Page 3
	return

dds_control_wave
	movlw 0x02				;prfen ob Wellenform 2 (Rechteck) gewhlt ist
	subwf DDS_WAVEFORM,W
	btfss STATUS,Z
	goto $+4				;nein, weiter bei nchster Prfung
	movlw b'00101000'		;Rechteck: Bit 1=0 (Mode), Bit 3=1 (DIV2), Bit 5=1 (OPBITEN)
	movwf DDS_CONTROL_L
	return

	movlw 0x01				;prfen ob Wellenform 1 (Dreieck) gewhlt ist
	subwf DDS_WAVEFORM,W
	btfss STATUS,Z
	goto $+4				;nein, weiter bei nchster Prfung
	movlw b'00000010'		;Dreieck: Bit 1=1 (Mode), Bit 3=0 (DIV2),  Bit 5=0 (OPBITEN)
	movwf DDS_CONTROL_L
	return

	clrf DDS_CONTROL_L		;Sinus: Bit 1=0 (Mode), Bit 3=0 (DIV2), Bit 5=0 (OPBITEN)
	return


calc_dds_freq					;Berechnung des Wertes fr das Frequenzregister, FREQREG = 2^28 * Fdds / MCLK
	clrf ZAHL_A0				;lade Zahl_A mit Wert 2^28 = 0x10000000
	clrf ZAHL_A1
	clrf ZAHL_A2
	movlw 0x10
	movwf ZAHL_A3
	movfw DDS_FREQ_0			;lade Zahl_B mit DDS_FREQ
	movwf ZAHL_B0
	movfw DDS_FREQ_1
	movwf ZAHL_B1
	movfw DDS_FREQ_2
	movwf ZAHL_B2
	movfw DDS_FREQ_3
	movwf ZAHL_B3
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call mul32					;Multipliziere ZAHL_A mit ZAHL_B, das Ergebnis steht in den Registern ZAHL_C0 bis ZAHL_C7

	movfw DDS_MCLK_0			;Lade DDS_MCLK in ZAHL_A Register
	movwf ZAHL_A0
	movfw DDS_MCLK_1
	movwf ZAHL_A1
	movfw DDS_MCLK_2
	movwf ZAHL_A2
	movfw DDS_MCLK_3
	movwf ZAHL_A3
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C

	movlw d'10'					;nochmal durch 10 teilen: Lade Wert 10 in ZAHL_A Register
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call div32					;Teile ZAHL_C durch ZAHL_A, das Ergebnis steht in ZAHL_C, die Nachkommastelle in ZAHL_B0
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4
	call runden_div32

	movfw ZAHL_C0				;Kopiere das Ergebnis in die DDS_FREQREG Register
	movwf DDS_FREQREG_0
	movfw ZAHL_C1
	movwf DDS_FREQREG_1
	movfw ZAHL_C2
	movwf DDS_FREQREG_2
	movfw ZAHL_C3
	movwf DDS_FREQREG_3
	return

calc_dds_phase
	movfw DDS_PHASE_L			;DDS Phasenregister: Phasenwinkel (grad x10) x 4096 / 3600 (grad x10)
	movwf ZAHL_C1				;4096 = 256 x 16 	=> x256 = 1 komplettes Register nach links: ZAHL_C1 => ZAHL_C2, ZAHL_C0 => ZAHL_C1
	movfw DDS_PHASE_H			;					=> x16 = 4x nach links verschieben => x2 x2 x2 x2 = x16
	movwf ZAHL_C2
	bcf STATUS,C				;lsche Carry Bit
	rlf ZAHL_C1
	rlf ZAHL_C2
	bcf STATUS,C				;lsche Carry Bit
	rlf ZAHL_C1
	rlf ZAHL_C2
	bcf STATUS,C				;lsche Carry Bit
	rlf ZAHL_C1
	rlf ZAHL_C2
	bcf STATUS,C				;lsche Carry Bit
	rlf ZAHL_C1
	rlf ZAHL_C2
	clrf ZAHL_C0
	clrf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7
	movlw 0x10					;lade Nennerregister mit Wert 3600 (0x0E10)
	movwf ZAHL_A0
	movlw 0x0E
	movwf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3

	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4
	call div32					;Teile ZAHL_C durch ZAHL_A, das ganzzahlige Ergebnis steht in ZAHL_C, der Rest in ZAHL_B
	bsf PCLATH,3				;Select Page 3
	bsf PCLATH,4

	call runden_div32			;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnisses der Division

	movfw ZAHL_C0				;kopieren des Ergebnisses in die DDS Phasen Register (Bits 0-11)
	movwf DDS_PHASEREG_L
	movfw ZAHL_C1
	movwf DDS_PHASEREG_H
	bsf DDS_PHASEREG_H,7		;Bits 12-15 = 1100
	bsf DDS_PHASEREG_H,6
	bcf DDS_PHASEREG_H,5
	bcf DDS_PHASEREG_H,4
	return


;*******************************************************************************************
;Logarithmentabellen L-Byte und H-byte, delta t Tabellen
;*******************************************************************************************

	ORG 0x1CFF

log_tab_L				;Logarithmentabelle (1.000-10.000) Low Byte
	addwf PCL,f
	retlw 0x9A		;Datenpunkt 0
	retlw 0xD5		;Datenpunkt 1
	retlw 0x11		;Datenpunkt 2
	retlw 0x4D		;Datenpunkt 3
	retlw 0x8A		;Datenpunkt 4
	retlw 0xC7		;Datenpunkt 5
	retlw 0x05		;Datenpunkt 6
	retlw 0x43		;Datenpunkt 7
	retlw 0x83		;Datenpunkt 8
	retlw 0xC2		;Datenpunkt 9
	retlw 0x02		;Datenpunkt 10
	retlw 0x43		;Datenpunkt 11
	retlw 0x85		;Datenpunkt 12
	retlw 0xC6		;Datenpunkt 13
	retlw 0x09		;Datenpunkt 14
	retlw 0x4C		;Datenpunkt 15
	retlw 0x90		;Datenpunkt 16
	retlw 0xD4		;Datenpunkt 17
	retlw 0x19		;Datenpunkt 18
	retlw 0x5F		;Datenpunkt 19
	retlw 0xA5		;Datenpunkt 20
	retlw 0xEC		;Datenpunkt 21
	retlw 0x34		;Datenpunkt 22
	retlw 0x7C		;Datenpunkt 23
	retlw 0xC5		;Datenpunkt 24
	retlw 0x0E		;Datenpunkt 25
	retlw 0x58		;Datenpunkt 26
	retlw 0xA3		;Datenpunkt 27
	retlw 0xEF		;Datenpunkt 28
	retlw 0x3B		;Datenpunkt 29
	retlw 0x88		;Datenpunkt 30
	retlw 0xD5		;Datenpunkt 31
	retlw 0x23		;Datenpunkt 32
	retlw 0x72		;Datenpunkt 33
	retlw 0xC2		;Datenpunkt 34
	retlw 0x12		;Datenpunkt 35
	retlw 0x64		;Datenpunkt 36
	retlw 0xB5		;Datenpunkt 37
	retlw 0x08		;Datenpunkt 38
	retlw 0x5B		;Datenpunkt 39
	retlw 0xAF		;Datenpunkt 40
	retlw 0x04		;Datenpunkt 41
	retlw 0x5A		;Datenpunkt 42
	retlw 0xB0		;Datenpunkt 43
	retlw 0x07		;Datenpunkt 44
	retlw 0x5F		;Datenpunkt 45
	retlw 0xB8		;Datenpunkt 46
	retlw 0x12		;Datenpunkt 47
	retlw 0x6C		;Datenpunkt 48
	retlw 0xC7		;Datenpunkt 49
	retlw 0x23		;Datenpunkt 50
	retlw 0x80		;Datenpunkt 51
	retlw 0xDE		;Datenpunkt 52
	retlw 0x3C		;Datenpunkt 53
	retlw 0x9C		;Datenpunkt 54
	retlw 0xFC		;Datenpunkt 55
	retlw 0x5D		;Datenpunkt 56
	retlw 0xBF		;Datenpunkt 57
	retlw 0x22		;Datenpunkt 58
	retlw 0x86		;Datenpunkt 59
	retlw 0xEA		;Datenpunkt 60
	retlw 0x50		;Datenpunkt 61
	retlw 0xB6		;Datenpunkt 62
	retlw 0x1E		;Datenpunkt 63
	retlw 0x86		;Datenpunkt 64
	retlw 0xEF		;Datenpunkt 65
	retlw 0x5A		;Datenpunkt 66
	retlw 0xC5		;Datenpunkt 67
	retlw 0x31		;Datenpunkt 68
	retlw 0x9E		;Datenpunkt 69
	retlw 0x0C		;Datenpunkt 70
	retlw 0x7B		;Datenpunkt 71
	retlw 0xEC		;Datenpunkt 72
	retlw 0x5D		;Datenpunkt 73
	retlw 0xCF		;Datenpunkt 74
	retlw 0x42		;Datenpunkt 75
	retlw 0xB6		;Datenpunkt 76
	retlw 0x2C		;Datenpunkt 77
	retlw 0xA2		;Datenpunkt 78
	retlw 0x19		;Datenpunkt 79
	retlw 0x92		;Datenpunkt 80
	retlw 0x0C		;Datenpunkt 81
	retlw 0x86		;Datenpunkt 82
	retlw 0x02		;Datenpunkt 83
	retlw 0x7F		;Datenpunkt 84
	retlw 0xFD		;Datenpunkt 85
	retlw 0x7C		;Datenpunkt 86
	retlw 0xFD		;Datenpunkt 87
	retlw 0x7E		;Datenpunkt 88
	retlw 0x01		;Datenpunkt 89
	retlw 0x85		;Datenpunkt 90
	retlw 0x0A		;Datenpunkt 91
	retlw 0x90		;Datenpunkt 92
	retlw 0x17		;Datenpunkt 93
	retlw 0xA0		;Datenpunkt 94
	retlw 0x2A		;Datenpunkt 95
	retlw 0xB5		;Datenpunkt 96
	retlw 0x41		;Datenpunkt 97
	retlw 0xCF		;Datenpunkt 98
	retlw 0x5E		;Datenpunkt 99
	retlw 0xEE		;Datenpunkt 100
	retlw 0x80		;Datenpunkt 101
	retlw 0x13		;Datenpunkt 102
	retlw 0xA7		;Datenpunkt 103
	retlw 0x3D		;Datenpunkt 104
	retlw 0xD3		;Datenpunkt 105
	retlw 0x6C		;Datenpunkt 106
	retlw 0x05		;Datenpunkt 107
	retlw 0xA0		;Datenpunkt 108
	retlw 0x3D		;Datenpunkt 109
	retlw 0xDB		;Datenpunkt 110
	retlw 0x7A		;Datenpunkt 111
	retlw 0x1B		;Datenpunkt 112
	retlw 0xBD		;Datenpunkt 113
	retlw 0x60		;Datenpunkt 114
	retlw 0x05		;Datenpunkt 115
	retlw 0xAC		;Datenpunkt 116
	retlw 0x54		;Datenpunkt 117
	retlw 0xFE		;Datenpunkt 118
	retlw 0xA9		;Datenpunkt 119
	retlw 0x55		;Datenpunkt 120
	retlw 0x04		;Datenpunkt 121
	retlw 0xB4		;Datenpunkt 122
	retlw 0x65		;Datenpunkt 123
	retlw 0x18		;Datenpunkt 124
	retlw 0xCD		;Datenpunkt 125
	retlw 0x83		;Datenpunkt 126
	retlw 0x3B		;Datenpunkt 127
	retlw 0xF4		;Datenpunkt 128
	retlw 0xB0		;Datenpunkt 129
	retlw 0x6C		;Datenpunkt 130
	retlw 0x2B		;Datenpunkt 131
	retlw 0xEB		;Datenpunkt 132
	retlw 0xAE		;Datenpunkt 133
	retlw 0x71		;Datenpunkt 134
	retlw 0x37		;Datenpunkt 135
	retlw 0xFE		;Datenpunkt 136
	retlw 0xC8		;Datenpunkt 137
	retlw 0x93		;Datenpunkt 138
	retlw 0x60		;Datenpunkt 139
	retlw 0x2E		;Datenpunkt 140
	retlw 0xFF		;Datenpunkt 141
	retlw 0xD1		;Datenpunkt 142
	retlw 0xA6		;Datenpunkt 143
	retlw 0x7C		;Datenpunkt 144
	retlw 0x54		;Datenpunkt 145
	retlw 0x2E		;Datenpunkt 146
	retlw 0x0B		;Datenpunkt 147
	retlw 0xE9		;Datenpunkt 148
	retlw 0xC9		;Datenpunkt 149
	retlw 0xAB		;Datenpunkt 150
	retlw 0x8F		;Datenpunkt 151
	retlw 0x76		;Datenpunkt 152
	retlw 0x5E		;Datenpunkt 153
	retlw 0x48		;Datenpunkt 154
	retlw 0x35		;Datenpunkt 155
	retlw 0x24		;Datenpunkt 156
	retlw 0x15		;Datenpunkt 157
	retlw 0x08		;Datenpunkt 158
	retlw 0xFD		;Datenpunkt 159
	retlw 0xF4		;Datenpunkt 160
	retlw 0xEE		;Datenpunkt 161
	retlw 0xEA		;Datenpunkt 162
	retlw 0xE8		;Datenpunkt 163
	retlw 0xE9		;Datenpunkt 164
	retlw 0xEC		;Datenpunkt 165
	retlw 0xF1		;Datenpunkt 166
	retlw 0xF8		;Datenpunkt 167
	retlw 0x02		;Datenpunkt 168
	retlw 0x0F		;Datenpunkt 169
	retlw 0x1D		;Datenpunkt 170
	retlw 0x2E		;Datenpunkt 171
	retlw 0x42		;Datenpunkt 172
	retlw 0x58		;Datenpunkt 173
	retlw 0x71		;Datenpunkt 174
	retlw 0x8C		;Datenpunkt 175
	retlw 0xAA		;Datenpunkt 176
	retlw 0xCA		;Datenpunkt 177
	retlw 0xED		;Datenpunkt 178
	retlw 0x13		;Datenpunkt 179
	retlw 0x3B		;Datenpunkt 180
	retlw 0x66		;Datenpunkt 181
	retlw 0x94		;Datenpunkt 182
	retlw 0xC4		;Datenpunkt 183
	retlw 0xF7		;Datenpunkt 184
	retlw 0x2D		;Datenpunkt 185
	retlw 0x65		;Datenpunkt 186
	retlw 0xA1		;Datenpunkt 187
	retlw 0xDF		;Datenpunkt 188
	retlw 0x20		;Datenpunkt 189
	retlw 0x65		;Datenpunkt 190
	retlw 0xAC		;Datenpunkt 191
	retlw 0xF6		;Datenpunkt 192
	retlw 0x43		;Datenpunkt 193
	retlw 0x93		;Datenpunkt 194
	retlw 0xE6		;Datenpunkt 195
	retlw 0x3C		;Datenpunkt 196
	retlw 0x95		;Datenpunkt 197
	retlw 0xF1		;Datenpunkt 198
	retlw 0x51		;Datenpunkt 199
	retlw 0xB3		;Datenpunkt 200
	retlw 0x19		;Datenpunkt 201
	retlw 0x82		;Datenpunkt 202
	retlw 0xEE		;Datenpunkt 203
	retlw 0x5E		;Datenpunkt 204
	retlw 0xD1		;Datenpunkt 205
	retlw 0x47		;Datenpunkt 206
	retlw 0xC1		;Datenpunkt 207
	retlw 0x3E		;Datenpunkt 208
	retlw 0xBE		;Datenpunkt 209
	retlw 0x42		;Datenpunkt 210
	retlw 0xCA		;Datenpunkt 211
	retlw 0x55		;Datenpunkt 212
	retlw 0xE3		;Datenpunkt 213
	retlw 0x76		;Datenpunkt 214
	retlw 0x0C		;Datenpunkt 215
	retlw 0xA5		;Datenpunkt 216
	retlw 0x42		;Datenpunkt 217
	retlw 0xE3		;Datenpunkt 218
	retlw 0x88		;Datenpunkt 219
	retlw 0x30		;Datenpunkt 220
	retlw 0xDD		;Datenpunkt 221
	retlw 0x8D		;Datenpunkt 222
	retlw 0x41		;Datenpunkt 223
	retlw 0xF9		;Datenpunkt 224
	retlw 0xB5		;Datenpunkt 225
	retlw 0x75		;Datenpunkt 226
	retlw 0x39		;Datenpunkt 227
	retlw 0x01		;Datenpunkt 228
	retlw 0xCE		;Datenpunkt 229
	retlw 0x9E		;Datenpunkt 230
	retlw 0x73		;Datenpunkt 231
	retlw 0x4C		;Datenpunkt 232
	retlw 0x29		;Datenpunkt 233
	retlw 0x0A		;Datenpunkt 234
	retlw 0xF0		;Datenpunkt 235
	retlw 0xDA		;Datenpunkt 236
	retlw 0xC9		;Datenpunkt 237
	retlw 0xBC		;Datenpunkt 238
	retlw 0xB4		;Datenpunkt 239
	retlw 0xB0		;Datenpunkt 240
	retlw 0xB1		;Datenpunkt 241
	retlw 0xB6		;Datenpunkt 242
	retlw 0xC0		;Datenpunkt 243
	retlw 0xCF		;Datenpunkt 244
	retlw 0xE2		;Datenpunkt 245
	retlw 0xFB		;Datenpunkt 246
	retlw 0x18		;Datenpunkt 247
	retlw 0x3A		;Datenpunkt 248
	retlw 0x61		;Datenpunkt 249
	retlw 0x8D		;Datenpunkt 250
	retlw 0xBE		;Datenpunkt 251
	retlw 0xF4		;Datenpunkt 252
	retlw 0x2F		;Datenpunkt 253
	retlw 0x70		;Datenpunkt 254
	retlw 0xB5		;Datenpunkt 255


	ORG 0x1EFF

log_tab_H				;Logarithmentabelle (1.000-10.000) High Byte
	addwf PCL,f
	retlw 0x19		;Datenpunkt 0	
	retlw 0x19		;Datenpunkt 1	
	retlw 0x1A		;Datenpunkt 2	
	retlw 0x1A		;Datenpunkt 3	
	retlw 0x1A		;Datenpunkt 4	
	retlw 0x1A		;Datenpunkt 5	
	retlw 0x1B		;Datenpunkt 6	
	retlw 0x1B		;Datenpunkt 7	
	retlw 0x1B		;Datenpunkt 8	
	retlw 0x1B		;Datenpunkt 9	
	retlw 0x1C		;Datenpunkt 10	
	retlw 0x1C		;Datenpunkt 11	
	retlw 0x1C		;Datenpunkt 12	
	retlw 0x1C		;Datenpunkt 13	
	retlw 0x1D		;Datenpunkt 14	
	retlw 0x1D		;Datenpunkt 15	
	retlw 0x1D		;Datenpunkt 16	
	retlw 0x1D		;Datenpunkt 17	
	retlw 0x1E		;Datenpunkt 18	
	retlw 0x1E		;Datenpunkt 19	
	retlw 0x1E		;Datenpunkt 20	
	retlw 0x1E		;Datenpunkt 21	
	retlw 0x1F		;Datenpunkt 22	
	retlw 0x1F		;Datenpunkt 23	
	retlw 0x1F		;Datenpunkt 24	
	retlw 0x20		;Datenpunkt 25	
	retlw 0x20		;Datenpunkt 26	
	retlw 0x20		;Datenpunkt 27	
	retlw 0x20		;Datenpunkt 28	
	retlw 0x21		;Datenpunkt 29	
	retlw 0x21		;Datenpunkt 30	
	retlw 0x21		;Datenpunkt 31	
	retlw 0x22		;Datenpunkt 32	
	retlw 0x22		;Datenpunkt 33	
	retlw 0x22		;Datenpunkt 34	
	retlw 0x23		;Datenpunkt 35	
	retlw 0x23		;Datenpunkt 36	
	retlw 0x23		;Datenpunkt 37	
	retlw 0x24		;Datenpunkt 38	
	retlw 0x24		;Datenpunkt 39	
	retlw 0x24		;Datenpunkt 40	
	retlw 0x25		;Datenpunkt 41	
	retlw 0x25		;Datenpunkt 42	
	retlw 0x25		;Datenpunkt 43	
	retlw 0x26		;Datenpunkt 44	
	retlw 0x26		;Datenpunkt 45	
	retlw 0x26		;Datenpunkt 46	
	retlw 0x27		;Datenpunkt 47	
	retlw 0x27		;Datenpunkt 48	
	retlw 0x27		;Datenpunkt 49	
	retlw 0x28		;Datenpunkt 50	
	retlw 0x28		;Datenpunkt 51	
	retlw 0x28		;Datenpunkt 52	
	retlw 0x29		;Datenpunkt 53	
	retlw 0x29		;Datenpunkt 54	
	retlw 0x29		;Datenpunkt 55	
	retlw 0x2A		;Datenpunkt 56	
	retlw 0x2A		;Datenpunkt 57	
	retlw 0x2B		;Datenpunkt 58	
	retlw 0x2B		;Datenpunkt 59	
	retlw 0x2B		;Datenpunkt 60	
	retlw 0x2C		;Datenpunkt 61	
	retlw 0x2C		;Datenpunkt 62	
	retlw 0x2D		;Datenpunkt 63	
	retlw 0x2D		;Datenpunkt 64	
	retlw 0x2D		;Datenpunkt 65	
	retlw 0x2E		;Datenpunkt 66	
	retlw 0x2E		;Datenpunkt 67	
	retlw 0x2F		;Datenpunkt 68	
	retlw 0x2F		;Datenpunkt 69	
	retlw 0x30		;Datenpunkt 70	
	retlw 0x30		;Datenpunkt 71	
	retlw 0x30		;Datenpunkt 72	
	retlw 0x31		;Datenpunkt 73	
	retlw 0x31		;Datenpunkt 74	
	retlw 0x32		;Datenpunkt 75	
	retlw 0x32		;Datenpunkt 76	
	retlw 0x33		;Datenpunkt 77	
	retlw 0x33		;Datenpunkt 78	
	retlw 0x34		;Datenpunkt 79	
	retlw 0x34		;Datenpunkt 80	
	retlw 0x35		;Datenpunkt 81	
	retlw 0x35		;Datenpunkt 82	
	retlw 0x36		;Datenpunkt 83	
	retlw 0x36		;Datenpunkt 84	
	retlw 0x36		;Datenpunkt 85	
	retlw 0x37		;Datenpunkt 86	
	retlw 0x37		;Datenpunkt 87	
	retlw 0x38		;Datenpunkt 88	
	retlw 0x39		;Datenpunkt 89	
	retlw 0x39		;Datenpunkt 90	
	retlw 0x3A		;Datenpunkt 91	
	retlw 0x3A		;Datenpunkt 92	
	retlw 0x3B		;Datenpunkt 93	
	retlw 0x3B		;Datenpunkt 94	
	retlw 0x3C		;Datenpunkt 95	
	retlw 0x3C		;Datenpunkt 96	
	retlw 0x3D		;Datenpunkt 97	
	retlw 0x3D		;Datenpunkt 98	
	retlw 0x3E		;Datenpunkt 99	
	retlw 0x3E		;Datenpunkt 100	
	retlw 0x3F		;Datenpunkt 101	
	retlw 0x40		;Datenpunkt 102	
	retlw 0x40		;Datenpunkt 103	
	retlw 0x41		;Datenpunkt 104	
	retlw 0x41		;Datenpunkt 105	
	retlw 0x42		;Datenpunkt 106	
	retlw 0x43		;Datenpunkt 107	
	retlw 0x43		;Datenpunkt 108	
	retlw 0x44		;Datenpunkt 109	
	retlw 0x44		;Datenpunkt 110	
	retlw 0x45		;Datenpunkt 111	
	retlw 0x46		;Datenpunkt 112	
	retlw 0x46		;Datenpunkt 113	
	retlw 0x47		;Datenpunkt 114	
	retlw 0x48		;Datenpunkt 115	
	retlw 0x48		;Datenpunkt 116	
	retlw 0x49		;Datenpunkt 117	
	retlw 0x49		;Datenpunkt 118	
	retlw 0x4A		;Datenpunkt 119	
	retlw 0x4B		;Datenpunkt 120	
	retlw 0x4C		;Datenpunkt 121	
	retlw 0x4C		;Datenpunkt 122	
	retlw 0x4D		;Datenpunkt 123	
	retlw 0x4E		;Datenpunkt 124	
	retlw 0x4E		;Datenpunkt 125	
	retlw 0x4F		;Datenpunkt 126	
	retlw 0x50		;Datenpunkt 127	
	retlw 0x50		;Datenpunkt 128	
	retlw 0x51		;Datenpunkt 129	
	retlw 0x52		;Datenpunkt 130	
	retlw 0x53		;Datenpunkt 131	
	retlw 0x53		;Datenpunkt 132	
	retlw 0x54		;Datenpunkt 133	
	retlw 0x55		;Datenpunkt 134	
	retlw 0x56		;Datenpunkt 135	
	retlw 0x56		;Datenpunkt 136	
	retlw 0x57		;Datenpunkt 137	
	retlw 0x58		;Datenpunkt 138	
	retlw 0x59		;Datenpunkt 139	
	retlw 0x5A		;Datenpunkt 140	
	retlw 0x5A		;Datenpunkt 141	
	retlw 0x5B		;Datenpunkt 142	
	retlw 0x5C		;Datenpunkt 143	
	retlw 0x5D		;Datenpunkt 144	
	retlw 0x5E		;Datenpunkt 145	
	retlw 0x5F		;Datenpunkt 146	
	retlw 0x60		;Datenpunkt 147	
	retlw 0x60		;Datenpunkt 148	
	retlw 0x61		;Datenpunkt 149	
	retlw 0x62		;Datenpunkt 150	
	retlw 0x63		;Datenpunkt 151	
	retlw 0x64		;Datenpunkt 152	
	retlw 0x65		;Datenpunkt 153	
	retlw 0x66		;Datenpunkt 154	
	retlw 0x67		;Datenpunkt 155	
	retlw 0x68		;Datenpunkt 156	
	retlw 0x69		;Datenpunkt 157	
	retlw 0x6A		;Datenpunkt 158	
	retlw 0x6A		;Datenpunkt 159	
	retlw 0x6B		;Datenpunkt 160	
	retlw 0x6C		;Datenpunkt 161	
	retlw 0x6D		;Datenpunkt 162	
	retlw 0x6E		;Datenpunkt 163	
	retlw 0x6F		;Datenpunkt 164	
	retlw 0x70		;Datenpunkt 165	
	retlw 0x71		;Datenpunkt 166	
	retlw 0x72		;Datenpunkt 167	
	retlw 0x74		;Datenpunkt 168	
	retlw 0x75		;Datenpunkt 169	
	retlw 0x76		;Datenpunkt 170	
	retlw 0x77		;Datenpunkt 171	
	retlw 0x78		;Datenpunkt 172	
	retlw 0x79		;Datenpunkt 173	
	retlw 0x7A		;Datenpunkt 174	
	retlw 0x7B		;Datenpunkt 175	
	retlw 0x7C		;Datenpunkt 176	
	retlw 0x7D		;Datenpunkt 177	
	retlw 0x7E		;Datenpunkt 178	
	retlw 0x80		;Datenpunkt 179	
	retlw 0x81		;Datenpunkt 180	
	retlw 0x82		;Datenpunkt 181	
	retlw 0x83		;Datenpunkt 182	
	retlw 0x84		;Datenpunkt 183	
	retlw 0x85		;Datenpunkt 184	
	retlw 0x87		;Datenpunkt 185	
	retlw 0x88		;Datenpunkt 186	
	retlw 0x89		;Datenpunkt 187	
	retlw 0x8A		;Datenpunkt 188	
	retlw 0x8C		;Datenpunkt 189	
	retlw 0x8D		;Datenpunkt 190	
	retlw 0x8E		;Datenpunkt 191	
	retlw 0x8F		;Datenpunkt 192	
	retlw 0x91		;Datenpunkt 193	
	retlw 0x92		;Datenpunkt 194	
	retlw 0x93		;Datenpunkt 195	
	retlw 0x95		;Datenpunkt 196	
	retlw 0x96		;Datenpunkt 197	
	retlw 0x97		;Datenpunkt 198	
	retlw 0x99		;Datenpunkt 199	
	retlw 0x9A		;Datenpunkt 200	
	retlw 0x9C		;Datenpunkt 201	
	retlw 0x9D		;Datenpunkt 202	
	retlw 0x9E		;Datenpunkt 203	
	retlw 0xA0		;Datenpunkt 204	
	retlw 0xA1		;Datenpunkt 205	
	retlw 0xA3		;Datenpunkt 206	
	retlw 0xA4		;Datenpunkt 207	
	retlw 0xA6		;Datenpunkt 208	
	retlw 0xA7		;Datenpunkt 209	
	retlw 0xA9		;Datenpunkt 210	
	retlw 0xAA		;Datenpunkt 211	
	retlw 0xAC		;Datenpunkt 212	
	retlw 0xAD		;Datenpunkt 213	
	retlw 0xAF		;Datenpunkt 214	
	retlw 0xB1		;Datenpunkt 215	
	retlw 0xB2		;Datenpunkt 216	
	retlw 0xB4		;Datenpunkt 217	
	retlw 0xB5		;Datenpunkt 218	
	retlw 0xB7		;Datenpunkt 219	
	retlw 0xB9		;Datenpunkt 220	
	retlw 0xBA		;Datenpunkt 221	
	retlw 0xBC		;Datenpunkt 222	
	retlw 0xBE		;Datenpunkt 223	
	retlw 0xBF		;Datenpunkt 224	
	retlw 0xC1		;Datenpunkt 225	
	retlw 0xC3		;Datenpunkt 226	
	retlw 0xC5		;Datenpunkt 227	
	retlw 0xC7		;Datenpunkt 228	
	retlw 0xC8		;Datenpunkt 229	
	retlw 0xCA		;Datenpunkt 230	
	retlw 0xCC		;Datenpunkt 231	
	retlw 0xCE		;Datenpunkt 232	
	retlw 0xD0		;Datenpunkt 233	
	retlw 0xD2		;Datenpunkt 234	
	retlw 0xD3		;Datenpunkt 235	
	retlw 0xD5		;Datenpunkt 236	
	retlw 0xD7		;Datenpunkt 237	
	retlw 0xD9		;Datenpunkt 238	
	retlw 0xDB		;Datenpunkt 239	
	retlw 0xDD		;Datenpunkt 240	
	retlw 0xDF		;Datenpunkt 241	
	retlw 0xE1		;Datenpunkt 242	
	retlw 0xE3		;Datenpunkt 243	
	retlw 0xE5		;Datenpunkt 244	
	retlw 0xE7		;Datenpunkt 245	
	retlw 0xE9		;Datenpunkt 246	
	retlw 0xEC		;Datenpunkt 247	
	retlw 0xEE		;Datenpunkt 248	
	retlw 0xF0		;Datenpunkt 249	
	retlw 0xF2		;Datenpunkt 250	
	retlw 0xF4		;Datenpunkt 251	
	retlw 0xF6		;Datenpunkt 252	
	retlw 0xF9		;Datenpunkt 253	
	retlw 0xFB		;Datenpunkt 254	
	retlw 0xFD		;Datenpunkt 255	


	ORG 0x1E00

delta_t_tab				;Tabellen fr deltaT Timer1 und deltaT/2 Timer0
	addwf PCL,f
	retlw 0x01			;DP 0: TMR1, dt = 1ms, WOB_ZEIT_H
	retlw 0x01			;DP 1: TMR1, dt = 2ms, WOB_ZEIT_H
	retlw 0x01			;DP 2: TMR1, dt = 5ms, WOB_ZEIT_H
	retlw 0x01			;DP 3: TMR1, dt = 10ms, WOB_ZEIT_H
	retlw 0x02			;DP 4: TMR1, dt = 20ms, WOB_ZEIT_H
	retlw 0x04			;DP 5: TMR1, dt = 50ms, WOB_ZEIT_H
	retlw 0x07			;DP 6: TMR1, dt = 100ms, WOB_ZEIT_H
	retlw 0x0D			;DP 7: TMR1, dt = 200ms, WOB_ZEIT_H
	retlw 0x1F			;DP 8: TMR1, dt = 500ms, WOB_ZEIT_H
	retlw 0x3E			;DP 9: TMR1, dt = 1000ms, WOB_ZEIT_H
	retlw 0x7B			;DP 10: TMR1, dt = 2000ms, WOB_ZEIT_H
	retlw 0xF0			;DP 11: TMR1, dt = 1ms, WOB_ZEIT_LN
	retlw 0xE0			;DP 12: TMR1, dt = 2ms, WOB_ZEIT_LN
	retlw 0xB1			;DP 13: TMR1, dt = 5ms, WOB_ZEIT_LN
	retlw 0x63			;DP 14: TMR1, dt = 10ms, WOB_ZEIT_LN
	retlw 0xC7			;DP 15: TMR1, dt = 20ms, WOB_ZEIT_LN
	retlw 0xF2			;DP 16: TMR1, dt = 50ms, WOB_ZEIT_LN
	retlw 0xE5			;DP 17: TMR1, dt = 100ms, WOB_ZEIT_LN
	retlw 0xCB			;DP 18: TMR1, dt = 200ms, WOB_ZEIT_LN
	retlw 0x7B			;DP 19: TMR1, dt = 500ms, WOB_ZEIT_LN
	retlw 0xF7			;DP 20: TMR1, dt = 1000ms, WOB_ZEIT_LN
	retlw 0xEE			;DP 21: TMR1, dt = 2000ms, WOB_ZEIT_LN
	retlw 0x02			;DP 22: TMR1, dt = 1ms, WOB_LOOP
	retlw 0x04			;DP 23: TMR1, dt = 2ms, WOB_LOOP
	retlw 0x08			;DP 24: TMR1, dt = 5ms, WOB_LOOP
	retlw 0x04			;DP 25: TMR1, dt = 10ms, WOB_LOOP
	retlw 0x02			;DP 26: TMR1, dt = 20ms, WOB_LOOP
	retlw 0x04			;DP 27: TMR1, dt = 50ms, WOB_LOOP
	retlw 0x02			;DP 28: TMR1, dt = 100ms, WOB_LOOP
	retlw 0x01			;DP 29: TMR1, dt = 200ms, WOB_LOOP
	retlw 0x02			;DP 30: TMR1, dt = 500ms, WOB_LOOP
	retlw 0x01			;DP 31: TMR1, dt = 1000ms, WOB_LOOP
	retlw 0x01			;DP 32: TMR1, dt = 2000ms, WOB_LOOP
	retlw 0x00			;DP 33: TMR0, dt = 1ms, H-Byte
	retlw 0x00			;DP 34: TMR0, dt = 2ms, H-Byte
	retlw 0x00			;DP 35: TMR0, dt = 5ms, H-Byte
	retlw 0x00			;DP 36: TMR0, dt = 10ms, H-Byte
	retlw 0x00			;DP 37: TMR0, dt = 20ms, H-Byte
	retlw 0x00			;DP 38: TMR0, dt = 50ms, H-Byte
	retlw 0x01			;DP 39: TMR0, dt = 100ms, H-Byte
	retlw 0x03			;DP 40: TMR0, dt = 200ms, H-Byte
	retlw 0x07			;DP 41: TMR0, dt = 500ms, H-Byte
	retlw 0x0F			;DP 42: TMR0, dt = 1000ms, H-Byte
	retlw 0x1E			;DP 43: TMR0, dt = 2000ms, H-Byte
	retlw 0x03			;DP 44: TMR0, dt = 1ms, M-Byte
	retlw 0x07			;DP 45: TMR0, dt = 2ms, M-Byte
	retlw 0x13			;DP 46: TMR0, dt = 5ms, M-Byte
	retlw 0x27			;DP 47: TMR0, dt = 10ms, M-Byte
	retlw 0x4E			;DP 48: TMR0, dt = 20ms, M-Byte
	retlw 0xC3			;DP 49: TMR0, dt = 50ms, M-Byte
	retlw 0x86			;DP 50: TMR0, dt = 100ms, M-Byte
	retlw 0x0D			;DP 51: TMR0, dt = 200ms, M-Byte
	retlw 0xA1			;DP 52: TMR0, dt = 500ms, M-Byte
	retlw 0x42			;DP 53: TMR0, dt = 1000ms, M-Byte
	retlw 0x84			;DP 54: TMR0, dt = 2000ms, M-Byte
	retlw 0xE8			;DP 55: TMR0, dt = 1ms, L-Byte
	retlw 0xD0			;DP 56: TMR0, dt = 2ms, L-Byte
	retlw 0x88			;DP 57: TMR0, dt = 5ms, L-Byte
	retlw 0x10			;DP 58: TMR0, dt = 10ms, L-Byte
	retlw 0x20			;DP 59: TMR0, dt = 20ms, L-Byte
	retlw 0x50			;DP 60: TMR0, dt = 50ms, L-Byte
	retlw 0xA0			;DP 61: TMR0, dt = 100ms, L-Byte
	retlw 0x40			;DP 62: TMR0, dt = 200ms, L-Byte
	retlw 0x20			;DP 63: TMR0, dt = 500ms, L-Byte
	retlw 0x40			;DP 64: TMR0, dt = 1000ms, L-Byte
	retlw 0x80			;DP 65: TMR0, dt = 2000ms, L-Byte

	ORG 0x1E43

cursor_tab				;Tabellen fr Cursorposition in den Menpunkten
	addwf PCL,f
	retlw 0x80			;DP 0: DDS Wellenform
	retlw 0x86			;DP 1: DDS Pegel
	retlw 0x8A			;DP 2: DDS Phase 100er
	retlw 0x8B			;DP 3: DDS Phase 10er
	retlw 0x8C			;DP 4: DDS Phase 1er
	retlw 0x8E			;DP 5: DDS Phase 0,1er
	retlw 0xC2			;DP 6: DDS Frequenz 10 MHz
	retlw 0xC3			;DP 7: DDS Frequenz 1 MHz
	retlw 0xC5			;DP 8: DDS Frequenz 100 kHz
	retlw 0xC6			;DP 9: DDS Frequenz 10 kHz
	retlw 0xC7			;DP 10: DDS Frequenz 1 kHz
	retlw 0xC9			;DP 11: DDS Frequenz 100 Hz
	retlw 0xCA			;DP 12: DDS Frequenz 10 Hz
	retlw 0xCB			;DP 13: DDS Frequenz 1 Hz
	retlw 0xCD			;DP 14: DDS Frequenz 0,1 Hz
	retlw 0x05			;DP 15: DDS Offset Vorzeichen
	retlw 0x07			;DP 16: DDS Offset 10er
	retlw 0x08			;DP 17: DDS Offset 1er
	retlw 0x0F			;DP 18: DDS TTL On/Off
	retlw 0x80			;DP 19: VCO/WOB 
	retlw 0x87			;DP 20: VCO/WOB Kennlinie
	retlw 0x8F			;DP 21: VCO/WOB Dekaden
	retlw 0xC4			;DP 22: VCO/WOB Fu MHz
	retlw 0xC6			;DP 23: VCO/WOB Fu 100kHz
	retlw 0xC7			;DP 24: VCO/WOB Fu 10kHz
	retlw 0xC8			;DP 25: VCO/WOB Fu 1kHz
	retlw 0xCA			;DP 26: VCO/WOB Fu 100Hz
	retlw 0xCB			;DP 27: VCO/WOB Fu 10Hz
	retlw 0xCC			;DP 28: VCO/WOB Fu 1Hz
	retlw 0x05			;DP 29: VCO/WOB Steps
	retlw 0x07			;DP 30: VCO/WOB Auto/Man/B
	retlw 0x0C			;DP 31: VCO/WOB dt
	retlw 0xC1			;DP 32: MCLK Frequenz 10 MHz
	retlw 0xC2			;DP 33: MCLK Frequenz 1 MHz
	retlw 0xC4			;DP 34: MCLK Frequenz 100 kHz
	retlw 0xC5			;DP 35: MCLK Frequenz 10 kHz
	retlw 0xC6			;DP 36: MCLK Frequenz 1 kHz
	retlw 0xC8			;DP 37: MCLK Frequenz 100 Hz
	retlw 0xC9			;DP 38: MCLK Frequenz 10 Hz
	retlw 0xCA			;DP 39: MCLK Frequenz 1 Hz

	END                	; Ende des Programms


